
/*
  Mistral is a constraint satisfaction and optimisation library
  Copyright (C) 2003-2005  Emmanuel Hebrard
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  The author can be contacted electronically at 
  ehebrard@cse.unsw.edu.au.
*/



/** \file mod.h
    \brief Header for the variables and constraints wrappers and build objects.
*/

/**********************************************
 * Wrappers & Build object
 **********************************************/

#ifndef _Wrapper_H
#define _Wrapper_H

#include <dvo.h>


class CSP;
class Variable;
class VarArray;

/**********************************************
 * MistralGacStack
 **********************************************/

/// Stack of Variable for computing the GAC closure.
template <class T>
class MistralGacStack {
public:
  T *MistralGacStack_;
  int        *trigger;
  int            size;
  
  /**@name Constructors*/
  //@{
  ///
  MistralGacStack()
  {
    size=0;
    MistralGacStack_=NULL;
    trigger=NULL;
  }
  ///
  MistralGacStack(int n)
  {
    initStack(n);
  }
  ///
  ~MistralGacStack()
  {
    delete [] trigger;
    delete [] MistralGacStack_;
  }
  ///
  void initStack(int n)
  { 
    trigger = new int[n];
    std::fill( trigger, trigger+n, 0 );
    MistralGacStack_ = new T[n];
    size = 0;
  }
  //@}
  
  void push(T v, int event)
  {
    if( !trigger[v->id] )
      MistralGacStack_[size++] = v;    
    trigger[v->id] |= event;
  }

  void push(T v, VariableInt* x)
  {
    if( !trigger[v->id] )
      MistralGacStack_[size++] = v;    
    trigger[v->id] = x->id;
  }


  T pop( int& event )
  {
    T last = MistralGacStack_[--size];
    event = trigger[last->id];
    trigger[last->id] = 0;
    return last;
  }

  T back( int& event )
  {
    T last = MistralGacStack_[size-1];
    event = trigger[last->id];
    return last;
  }

  bool empty() const
  {
    return (!size);
  }

  void clear()
  {
    while( size )
      trigger[MistralGacStack_[--size]->id] = 0; 
  }
};


////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
//////                                                                            //////
//////  There are 3 layers:                                                       //////
//////  1/ Wrappers are simply syntactic sugar that allow to                      //////
//////  a/ hide 'new' operations in models                                        //////
//////  b/ allow to overload operator (+, [], ...)                                //////
//////  2/ Build objects are 'temporary' variables and/or constraints.             //////
//////  They contain only the necessary information (domain, scope, ...)          //////
//////  and the method 'build' actually create the real object                    //////
//////  used during search. They allow to do some re-modelling                    //////
//////  before the solving phase.                                                 //////
//////  For instance, if a variable is an interval and is                         //////
//////  constrained only by BC constraints, the RangeVar                          //////
//////  implementation will be used instead of BitVar.                            //////
//////   + BuildObject stand for variables. They keep a representation            //////
//////     of their domain, as well as a number of informations in the            //////
//////     struct state.                                                          //////
//////   + BuildObjectConstraints stand for constraints' *relation*,              //////
//////     that is, at most one of each is created, no matter how many            //////
//////     such constraints are added to the model.                               //////
//////   + BuildObjectPredicate stand for constraint scope. They ihnerit          //////
//////     from BuildObject and therefore can also represent variables            //////
//////     themselves.                                                            //////
//////     - When a constraint, or variable wrapper is declared,                  //////
//////       the corresponding BuildObject is created                             //////
//////     - When a (tree of) objects are added to the model                      //////
//////       the top-level one is marked as constraint only, whilst               //////
//////       all descendants are marked as variables.                             //////


//////     - During the prebuilding phase, two things happen                      //////
//////       * The method propagateInfo() is called on all                        //////
//////         top-level objects, which recursively call their                    //////
//////         children.                                                          //////
//////         . Constraints that become incorrect when reasoning on bounds       //////
//////         . Constraints that can profit from holes                           //////
//////       * The method prebuild() is called on all objects from                //////
//////         top to bottom to perform rewriting when needed.                    //////

/*
  + transformations: (limited propagation from top to bottom + rewriting thereof)
  > general:
  - if all arguments are constants, delete the declaration (check consistency?)
  unsetRel(): the relation part should not be build, because it is 
  a totology
  deReference() all arguments. Each argument loses a reference 
  > Not
  - if set to true or at toplevel (!isVar()):
  unsetRel(), set argument to false (0).
  - if set to false:
  unsetRel(), set argument to false (0).
  > And 
  - if set to true or at toplevel (!isVar()):
  unsetRel(), set arguments to true (1).
  > Or
  - if set to false:
  unsetRel(), set arguments to false (0).
  > Add 
  - if an argument is  a constant:
  turn into an offset predicate
  > Sub
  - if an argument is  a constant:
  turn into an offset predicate
  > EqualConstant
  - if set to true or at toplevel (!isVar()):
  unsetRel(), set argument to val.
  - if set to false:
  unsetRel(), remove val in arg's domain.
  > NotEqualConstant
  - if set to true or at toplevel (!isVar()):
  unsetRel(), remove val in arg's domain.
  - if set to false:
  unsetRel(), set argument to val.
  > Equal
  - if set to true or at toplevel (!isVar()):
  unsetRel(), merge arguments.
  - if an argument is  a constant:
  turn into an EqualConstant predicate
  > NotEqual
  - if set to true or at toplevel (!isVar()), and if one of the agrs is a constant
  unsetRel(), and remove the constant from the other agr's domain.
  - if an argument is  a constant:
  turn into an NotEqualConstant predicate
  > SumEq
  - absorb constants.
  > Sum
  - if the last arg is a constant and is not referenced otherwise:
  turns into a SumEq constraint
  - absorb constants then according to the arity,
  change to offset/factor/add/sub/sumeq
  > if N is constant:
  - turns into an equal predicate

*/

//////                                                                            //////
//////  3/ Actual variables and constraints (csp.h / var.h / con.h)               //////
//////                                                                            //////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
/* 

   propagate info

   * needed because when rewriting, the state of an object can change,
   the change must be propagated downstream.


   mechanism to decide if the representation should be predicate 
   or constraint.
   * if not at top level -> always variable
   * some object are variables even at the top level 
   (actual variables, element, sum, etc)
   * top level known when adding, but it can change 
   (for instance with a AND constraint)
*/


class BuildObjectConstraint;
/**********************************************
 * ConstraintStore
 **********************************************/
/*! \class ConstraintStore
  \brief A pool of constraint build objects

  We need only one instance of each constraint
  buld object.
*/
class ConstraintStore
{
public: 
  // LOGIC
  static const int AND         =  2 ;
  static const int OR          = 21 ;
  static const int NOT         = 19 ;

  // ARITHMETIC
  static const int SUB         = 31 ;
  static const int ADD         = 32 ;
  static const int ABS         =  0 ;
  static const int DIV         =  5 ;
  static const int MOD         = 16 ;
  static const int MODCONSTANT = 38 ;
  static const int MUL         = 17 ;
  static const int NEG         = 18 ;
  static const int POW         = 22 ;
  static const int FACTOR      =  9 ;
  static const int OFFSET      = 20 ;

  // RELATION
  static const int DISJUNCTIVE =  4 ;
  static const int EQUAL       =  8 ;
  static const int EQCONSTANT  = 36 ;
  static const int NEQ         = 30 ;
  static const int NEQCONSTANT = 37 ;
  static const int PRECEDENCE  = 23 ;
  static const int LOWERBOUND  = 34 ;
  static const int UPPERBOUND  = 35 ;
  static const int NEQW        = 27 ;

  // GLOBAL
  static const int ALLDIFF     =  1 ;
  static const int CLAUSE      =  3 ;
  static const int EDGEFINDER  =  6 ;
  static const int ELEMENT     =  7 ;
  static const int GCC         = 10 ;
  static const int LEX         = 11 ;
  static const int MAX         = 12 ;
  static const int MIN         = 13 ;
  static const int MAXOCCUR    = 14 ;
  static const int MINOCCUR    = 15 ;
  static const int SUM         = 24 ;
  static const int SUMEQ       = 33 ;
  static const int MAXDIFF     = 25 ;
  static const int MINDIFF     = 26 ;
  static const int SLIDE       = 28 ;
  static const int REGULAR     = 29 ;
  static const int IFTHENELSE  = 39 ;

  // COUNT
  static const int NUMCONS     = 40 ;


  // private:
  MistralStack<BuildObjectConstraint *> store;
  BuildObjectConstraint *getConstraint(const int c);

public:
  ConstraintStore() ;
  virtual ~ConstraintStore() ;
  BuildObjectConstraint *operator[]( const int c ) ;
  int addConstraint( BuildObjectConstraint *c ) ;
  void remConstraint( int idx ) ;
  void flush();
};


//static ConstraintStore ENV;


/********************************************************
 *
 *  BuildObject for Variables, Constraints 
 *  and Predicates.
 *
 ********************************************************/

class BuildObjectPredicate;
/**********************************************
 * BuildObject
 **********************************************/
class BuildObject
{

protected:
 
  CSP *model;
  int size_;
  int min_;
  int max_;
  BuildObject *veqptr_;
  int refs;

  int state;


public:

  MistralSet *domain_;  
  VariableInt *varptr_;

  BuildObject *getBuildObject();
  const BuildObject *getConstBuildObject() const ;

  MistralStack<BuildObjectPredicate*> parent;
  int id;

  int getVarId() const ;
  int getState() ;
  void setState( int nstate ) ;

  static const int VRANGE=   1;
  static const int CRANGE=1024;
  static const int NOT   =   2;
  static const int NEG   =   4;
  static const int VAR   =   8;
  static const int CONST =  16;
  static const int AWV   =  32;
  static const int CONVEX=  64;
  static const int VLIST = 128;
  static const int CLIST =2048;
  static const int REL   = 256;
  static const int PAR   = 512;
  static const int NEQ   =4096;
  static const int FIXED =8192;

  static const int VALUETRIGGER =7;
  static const int RANGETRIGGER =3;
  static const int DOMAINTRIGGER=1;
  static const int LISTTRIGGER  =2;
  static const int CONTTRIGGER =4;
  void releaseEvent();
  void releaseEvent( const int );

  /*!
   * set to 1 iff it is an interval and consider only intervals
   */
  virtual bool isConvex() const ;
  virtual void setConvex() ;
  virtual void unsetConvex() ;

  /*!
   * set to 1 iff it preserves intervals
   */
  virtual bool isCRange() const ;
  virtual void setCRange() ;
  virtual void unsetCRange() ;

  /*!
   * set to 1 iff it can be implemented as an interval
   */
  virtual bool isVRange() const ;
  virtual void setVRange() ;
  virtual void unsetVRange() ;

  /*!
   * set to 1 iff the predicate can absord a "not" predicate
   */  
  virtual bool isNot() const ;
  virtual void setNot() ;
  virtual void unsetNot() ;

  /*!
   * set to 1 iff the predicate can absord a "-" predicate
   */
  virtual bool isNeg() const ;
  virtual void setNeg() ;
  virtual void unsetNeg() ;

  /*!
   * set to 1 iff it has a domain, and will be a variable
   */
  virtual bool isVar() const ;
  virtual void setVar() ;
  virtual void unsetVar() ;

  /*!
   * set to 1 iff it has a domain, and will be a variable
   */
  virtual bool isAwv() const ;
  virtual void setAwv() ;
  virtual void unsetAwv() ;

  /*!
   * set to 1 iff it is a constant
   */
  virtual bool isConst() const ;
  virtual void setConst() ;
  virtual void unsetConst() ;

  /*!
   * set to 1 iff it can be a list
   */
  virtual bool isVList() const ;
  virtual void setVList() ;
  virtual void unsetVList() ;

  /*!
   * set to 1 iff its args can be lists
   */
  virtual bool isCList() const ;
  virtual void setCList() ;
  virtual void unsetCList() ;

  /*!
   * set to 1 iff it is a constraint
   */
  virtual bool isRel() const ;
  virtual void setRel() ;
  virtual void unsetRel() ;

  /*!
   * set to 1 iff it can be replaced by a parameter 
   */
  virtual bool isPar() const ;
  virtual void setPar() ;
  virtual void unsetPar() ;

  /*!
   * set to 1 iff it is a top level neq constraint 
   */
  virtual bool isNeq() const ;
  virtual void setNeq() ;
  virtual void unsetNeq() ;

  /*!
   * set to 1 iff it cannot be simplified anymore
   */
  virtual bool isFixed() const ;
  virtual void setFixed() ;
  virtual void unsetFixed() ;


public:

  inline void deReference() ;
  inline int isReferenced() ;

  virtual void add( CSP *model, const int l, const int inf );
  virtual bool preBuild() ;


  inline bool notIn() ;
  virtual bool isBoolean() ;
  virtual bool isFDomain() ;
  virtual bool isRange() ;
  virtual bool isSearchable() ;
  virtual bool isConstraint() ;
  virtual VariableInt *_getVariable() ;
  virtual VariableInt *getVariable() ;
  
  void initVar(const int l, const int u);
  BuildObject() ;
  BuildObject( const int lo, const int up ) ;
  BuildObject( const int* array, const int length ) ;
  BuildObject( const int* array, const int length, const int lo, const int up ) ;
  virtual ~BuildObject() ;

  virtual void build( Solver *s );

  virtual int size() const ;
  virtual int domsize() const ;
  virtual int value() const ;
  virtual int first() const ;
  virtual int last() const ;
  virtual int min() const ;
  virtual int max() const ;
  virtual int minCapacity() const ;
  virtual int maxCapacity() const ;
  virtual int minPosAbs() const ;
  virtual int minNegAbs() const ;
  virtual bool remove(const int v);
  virtual bool setDomain(const int v);
  virtual bool setMin(const int l);
  virtual bool setMax(const int u);
  virtual bool setDomain(const MistralSet& s);
  virtual void setEqual(BuildObject *v); 
  virtual bool removeSet(const MistralSet& s);
  virtual bool removeRange(const int l, const int u);

  virtual int getNext(const int v) const ;
  virtual bool good(const int v) const ;
  virtual bool setNext(int& v) const ;

  virtual bool isRange() const ;
  virtual bool isGround() const ;

  virtual int _size() const ;
  virtual int _domsize() const ;
  virtual int _value() const ;
  virtual int _first() const ;
  virtual int _last() const ;
  virtual int _minCapacity() const ;
  virtual int _maxCapacity() const ;
  virtual int _min() const ;
  virtual int _max() const ;
  virtual int _minPosAbs() const ; 
  virtual int _minNegAbs() const ; 
  virtual bool _remove(const int v);
  virtual bool _setDomain(const int v);
  virtual bool _setMin(const int l);
  virtual bool _setMax(const int u);
  virtual bool _setDomain(const MistralSet& s);
  virtual void _setEqual(BuildObject *v); 
  virtual bool _removeSet(const MistralSet& s);
  virtual bool _removeRange(const int l, const int u);

  virtual int _getNext(const int v) const ;
  virtual bool _good(const int v) const ;
  virtual bool _setNext(int& v) const ;

  virtual bool _isRange() const ;
  virtual bool _isGround() const ;


  virtual bool setDomain(const int* a, const int l) { return true; }
  virtual bool setDomain(VariableInt* x) const { return true; }
  virtual void makeDecision(int&) {}
  virtual bool makeComplementary(int&) { return true; }


  virtual bool contain(const int v) const ;
  virtual bool intersect(const int lo, const int up) const { return true; } // TODO
  virtual bool included(const int lo, const int up) const { return true; } // TODO
  virtual bool intersect(const VariableInt* x) const { return true; } // TODO
  virtual bool included(const VariableInt* x) const { return true; } // TODO
  virtual bool intersect(const MistralSet* s) const { return true; } // TODO
  virtual bool included(const MistralSet* s) const { return true; } // TODO
  virtual bool intersect(const MistralSet& s) const { return true; } // TODO
  virtual bool intersect(const MistralSet& s, int& r) const { return true; } // TODO
  virtual bool included(const MistralSet& s) const { return true; } // TODO
  virtual void intersectTo( MistralSet* s ) const {} // TODO
  virtual void unionTo( MistralSet* s ) const {} // TODO
  virtual void intersectTo( MistralSet& s ) const {} // TODO
  virtual void unionTo( MistralSet& s ) const {} // TODO

  virtual bool _contain(const int v) const ;
  /*   virtual bool _intersect(const int lo, const int up) const ; */
  /*   virtual bool _included(const int lo, const int up) const ; */
  /*   virtual bool _intersect(const VariableInt* x) const ; */
  /*   virtual bool _included(const VariableInt* x) const ; */
  /*   virtual bool _intersect(const MistralSet* s) const ; */
  /*   virtual bool _included(const MistralSet* s) const ; */
  /*   virtual bool _intersect(const MistralSet& s) const ; */
  /*   virtual bool _intersect(const MistralSet& s, int& r) const ; */
  /*   virtual bool _included(const MistralSet& s) const ; */
  /*   virtual void _intersectTo( MistralSet* s ) const ; */
  /*   virtual void _unionTo( MistralSet* s ) const ; */
  /*   virtual void _intersectTo( MistralSet& s ) const ; */
  /*   virtual void _unionTo( MistralSet& s ) const ; */

  virtual void printshort(std::ostream& o) const ;
  virtual void print(std::ostream& o) const ;

  virtual string xmlPred( int& idx, int level, int *sub ) const ;
  virtual string xmlCon( MistralSet& scope, int *index ) const ;
  virtual string xmlDom( ) const ;
  virtual void printstate( std::ostream& o ) const ;
  bool operator==( BuildObject& x ) const ;

};


class BuildObjectPredicate;
/**********************************************
 * Constraint BuildObject
 **********************************************/
class BuildObjectConstraint 
{

public:

  BuildObjectConstraint() ;
  virtual ~BuildObjectConstraint() ;

  virtual void add( CSP *model, const int l, BuildObjectPredicate *pred ) ;
  virtual void build( Solver *s, VariableInt **tmp, 
		      BuildObjectPredicate *pred ) = 0;
  virtual bool preBuild( BuildObjectPredicate *pred ) ;
  virtual int computeValue( BuildObjectPredicate *pred ) const ;

  virtual void print(std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
};



/**********************************************
 * Predicate BuildObject
 **********************************************/
class BuildObjectPredicate : public BuildObject
{

public:
 
  BuildObjectConstraint *relation;
  BuildObject **scope;
  int *params;

  // includes itself? NO
  int arity;

  int nestedArity() const ;
  virtual void unsetCRange() ;
  virtual void unsetCList() ;
  virtual void setFixed() ;
  
  BuildObjectPredicate(BuildObject **x, 
		       const int, const int, const int, 
		       BuildObjectConstraint*, int*, 
		       int istate=0);
  virtual ~BuildObjectPredicate() ;

  virtual void build( Solver *s ) ;
  virtual void add( CSP *model, const int l, const int inf ) ;  
  virtual bool preBuild( ) ; 

  virtual bool isConstraint() ;

  virtual void printshort(std::ostream& o) const ;
  virtual void print(std::ostream& o) const ;
  virtual string xmlPred( int& idx, int level, int *sub ) const ;
  virtual string xmlCon( MistralSet& scp, int *index ) const ;

};




/********************************************************
 *
 *  Unary Predicates BuildObjects & Wrappers
 *
 ********************************************************/

/**********************************************
 *  Offset Predicate BuildObject
 **********************************************/ 
class BuildObjectOffset : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Factor Predicate BuildObject
 **********************************************/ 
class BuildObjectFactor : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate *) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Negation Predicate BuildObject
 **********************************************/ 
class BuildObjectNegation : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Not Predicate BuildObject
 **********************************************/ 
class BuildObjectNot : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Abs Predicate BuildObject
 **********************************************/ 
class BuildObjectAbs : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};


/********************************************************
 *
 *  Binary Predicates BuildObjects 
 *
 ********************************************************/

//////////// LOGIC PREDICATES /////////////////
/**********************************************
 * And Constraint BuildObject
 **********************************************/
class BuildObjectAnd : public BuildObjectConstraint {

public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 * Or Constraint BuildObject
 **********************************************/
class BuildObjectOr : public BuildObjectConstraint {

public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 * IfThenElse Constraint BuildObject
 **********************************************/
class BuildObjectIfThenElse : public BuildObjectConstraint {

public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};


//////////// COMPARISON PREDICATES ////////////
/**********************************************
 *  Disjunctive Predicate BuildObject
 **********************************************/ 
class BuildObjectDisjunctive : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Precedence Predicate BuildObject
 **********************************************/ 
class BuildObjectPrecedence : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  LowerBound Predicate BuildObject
 **********************************************/ 
class BuildObjectLowerBound : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  UpperBound Predicate BuildObject
 **********************************************/ 
class BuildObjectUpperBound : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Add Predicate BuildObject
 **********************************************/
class BuildObjectAdd : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Sub Predicate BuildObject
 **********************************************/
class BuildObjectSub : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Mul Predicate BuildObject
 **********************************************/
class BuildObjectMul : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Div Predicate BuildObject
 **********************************************/
class BuildObjectDiv : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Pow Predicate BuildObject
 **********************************************/
class BuildObjectPow : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Mod Predicate BuildObject
 **********************************************/
class BuildObjectMod : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  ModConstant Predicate BuildObject
 **********************************************/
class BuildObjectModConstant : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Equal Predicate BuildObject
 **********************************************/ 
class BuildObjectEqual : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  Equal Constant Predicate BuildObject
 **********************************************/ 
class BuildObjectEqualConstant : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  NotEqual Predicate BuildObject
 **********************************************/ 
class BuildObjectNotEqual : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  NotEqual Predicate BuildObject
 **********************************************/ 
class BuildObjectNotEqualConstant : public BuildObjectConstraint {
 
public:

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 *  NotEqualWildcard Predicate BuildObject
 **********************************************/ 
class BuildObjectNotEqualWildcard : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};


/********************************************************
 *
 *  N-ary Predicates BuildObjects & Wrappers
 *
 ********************************************************/

/**********************************************
 * SumEq Constraint BuildObject
 **********************************************/
class BuildObjectSumEq : public BuildObjectConstraint {

public:

  virtual bool  preBuild(BuildObjectPredicate *) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
};

/**********************************************
 * Sum Constraint BuildObject
 **********************************************/
class BuildObjectSum : public BuildObjectConstraint {

public:

  virtual bool  preBuild(BuildObjectPredicate *) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 * Min Constraint BuildObject
 **********************************************/
class BuildObjectMin : public BuildObjectConstraint {

public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};


/**********************************************
 * Max Constraint BuildObject
 **********************************************/
class BuildObjectMax : public BuildObjectConstraint {

public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};


#ifdef _CPLEX

/**********************************************
 * CPlexMinDifference Constraint BuildObject
 **********************************************/
class BuildObjectCPlexMinDifference : public BuildObjectConstraint {

public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
};

#endif


#ifdef _COINLP

/**********************************************
 * MaxDifference Constraint BuildObject
 **********************************************/
class BuildObjectMaxDifference : public BuildObjectConstraint {

public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
};

/**********************************************
 * MinDifference Constraint BuildObject
 **********************************************/
class BuildObjectMinDifference : public BuildObjectConstraint {

public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
};

#endif

/**********************************************
 * Element Constraint BuildObject
 **********************************************/
class BuildObjectElement : public BuildObjectConstraint {

public:
  virtual bool  preBuild(BuildObjectPredicate *) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ;
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
  virtual int computeValue( BuildObjectPredicate *pred ) const;
};


/********************************************************
 *
 *  Pure Constraints BuildObjects
 *
 ********************************************************/

/**********************************************
 * Clause BuildObject
 **********************************************/ 
class BuildObjectClause : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
};

/**********************************************
 *  Extensional Constraint BuildObject
 **********************************************/ 
class BuildObjectTable : public BuildObjectConstraint {
 
public:

  int spin;
  int arity;
  MistralStack<int*> tuples;

  MistralSet* supportList[2];
  int *mindomain;
  int *maxdomain;

  MistralSet matrix;
  int *var_sizes;
  int matrixmin;

  int *supports;
  MistralStack<int*>** supportTuples;


  BuildObjectTable(const int a) ;
  BuildObjectTable(const int a, const int nbt) ;
  virtual ~BuildObjectTable() ;
  void setMinDom(const int i, const int m) ;
  void setMaxDom(const int i, const int m) ;

  virtual bool  preBuild( BuildObjectPredicate *pred ) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 

  void add( const int* sol ) ;
};

/**********************************************
 * AllDiff Constraint BuildObject
 **********************************************/
class BuildObjectAllDiff : public BuildObjectConstraint {

public:

  virtual bool  preBuild(BuildObjectPredicate *) ;
  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 
};
 
/**********************************************
 * Gcc Constraint BuildObject
 **********************************************/
class BuildObjectGcc : public BuildObjectConstraint {

public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 
};

/**********************************************
 * EdgeFinder Constraint BuildObject
 **********************************************/
class BuildObjectEdgeFinder : public BuildObjectConstraint {

public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ; 
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ;
};

/**********************************************
 * Lex Constraint BuildObject
 **********************************************/
class BuildObjectLex : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 
};

/**********************************************
 * MaxOccurrence Constraint BuildObject
 **********************************************/
class BuildObjectMaxOccurrence : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 
  //virtual int computeValue( BuildObjectPredicate *pred ) const;
};


/**********************************************
 * MinOccurrence Constraint BuildObject
 **********************************************/
class BuildObjectMinOccurrence : public BuildObjectConstraint {
 
public:

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 
  //virtual int computeValue( BuildObjectPredicate *pred ) const;
};

/**********************************************
 * Slide Constraint BuildObject
 **********************************************/
class BuildObjectSlide : public BuildObjectConstraint {

public:

  CSP* sub_problem;

  void tuple2int(int& x, int*, int, int*) ;

  virtual void   build  (Solver *, VariableInt **, BuildObjectPredicate *) ;
  virtual void   print  (std::ostream&, const BuildObjectPredicate*) const ; 
  virtual string xmlPred(int&, int, const BuildObjectPredicate*, int*) const ; 
};


/**********************************************
 *  Objectives declaration
 **********************************************/
class BuildObjectObjective {
public:
  BuildObject* X;
  virtual void build( Solver *s ) = 0;

  BuildObjectObjective(BuildObject *x) ;

  virtual void print( std::ostream& o ) const ;
};

/// Maximisation declaration
class BuildObjectMaximiseVar : public BuildObjectObjective {
public:
  BuildObjectMaximiseVar(BuildObject *x) ;
  
  virtual void build( Solver *s ) ;
  virtual void print( std::ostream& o ) const ;
};

/// Minimisation declaration
class BuildObjectMinimiseVar : public BuildObjectObjective {
public:
  BuildObjectMinimiseVar(BuildObject *x) ;
  
  virtual void build( Solver *s ) ;
  virtual void print( std::ostream& o ) const ;
};


/**********************************************
 * Wrappers
 **********************************************/

/**********************************************
 * Variable
 **********************************************/
/*! \class Variable
  \brief Syntactic sugar used to hide the implementation of variables

  Arithmetic and logic C++ operators are overloaded
  so that an Ilog-like syntax is possible. 
*/
class Variable
{

public:

  BuildObject *var_ptr_;
  
  Variable() ;
  Variable( const Variable& v ) ;
  Variable( BuildObject *v ) ;
  Variable( const int nvals ) ;
  Variable( const int lo, const int up ) ;
  Variable( const int* array, const int length ) ;
  Variable( const int* array, const int length, const int lo, const int up ) ;
  virtual ~Variable() ;

  int value() ;
  int size() ;
  int min() ;
  int max() ;

  /*!@name Operators*/
  Variable& operator= ( const Variable& x ) ;
  Variable operator+ ( Variable x ) ;
  Variable operator- ( Variable x ) ;
  Variable operator* ( Variable x ) ; 
  Variable operator/ ( Variable x ) ; 
  Variable operator% ( Variable x ) ; 
  Variable operator^ ( Variable x ) ;
  Variable operator+ ( const int k ) ;
  Variable operator- ( const int k ) ;
  Variable operator* ( const int k ) ; 
  Variable operator/ ( const int k ) ; 
  Variable operator% ( const int k ) ; 
  Variable operator^ ( const int k ) ; 
  Variable operator- ( ) ;
  Variable operator&&( Variable x ) ;
  Variable operator||( Variable x ) ; 
  Variable operator! ( ) ; 
  Variable operator==( Variable x ) ;
  Variable operator!=( Variable x ) ;
  Variable operator<=( Variable x ) ;
  Variable operator< ( Variable x ) ;
  Variable operator>=( Variable x ) ;
  Variable operator> ( Variable x ) ;
  Variable operator==( const MistralSet& s ) ;
  Variable operator!=( const MistralSet& s ) ;
  Variable operator==( const int k ) ;
  Variable operator!=( const int k ) ;
  Variable operator<=( const int k ) ;
  Variable operator< ( const int k ) ;
  Variable operator>=( const int k ) ;
  Variable operator> ( const int k ) ;
  VarArray operator,( Variable x ) ;

  VariableInt *getVariable() ;
  virtual void printshort(std::ostream& o) const ;
  virtual void print(std::ostream& o) const ;

};


/**********************************************
 * VarArray
 **********************************************/
/*! \class VarArray
  \brief An array of variables (no wonder there)

*/
class VarArray 
{

public:
  
#ifdef _NARRAY
  MistralStack<BuildObject*> array_;
  Variable buffer;
#else
  MistralStack<BuildObject*> tmp_;
  MistralStack<Variable> array_;
#endif

  VarArray( ) ;
  VarArray( const int nvars ) ;
  VarArray( const int nvars, const int nvals ) ;
  VarArray( const int nvars, const int lo, const int up ) ;
  VarArray( const int nvars, const int* array, const int length ) ;
  VarArray( const int nvars, const int* array, const int length, const int lo, const int up ) ;

  virtual ~VarArray() ;

  BuildObject **getArgs( ) ;

  virtual void build( Solver *s ) ;

  void resize( const int n ) ;
  void clear() ;
  int size() const ;
  void add( const int k );
  void add( Variable x );
  void add( VarArray& a );
  
  /*!@name Operators*/
  Variable& operator[]( const int i ) ;
  Variable operator[]( Variable x ) ;  
  VarArray operator,( Variable x ) ;
  VarArray operator,( VarArray& x ) ;

  Variable operator< ( VarArray& x ) ; 
  Variable operator> ( VarArray& x ) ; 
  Variable operator<=( VarArray& x ) ; 
  Variable operator>=( VarArray& x ) ;

  VarArray& operator=( VarArray& x ) ;

  virtual void printshort(std::ostream& o) const ;
  virtual void print(std::ostream& o) const ;


};


/**********************************************
 * Matrix
 **********************************************/
/*! \class Matrix
  \brief A matrix of variables (no wonder there)

*/
class Matrix 
{

public:

  int nRows;
  int nCols;
  MistralStack<BuildObject*> array_;
  VarArray tmp_;

  Matrix( const int nr, const int nc ) ;
  Matrix( const int nr, const int nc, const int nv ) ;
  Matrix( const int nr, const int nc, const int lo, const int up ) ;
  Matrix( const int nr, const int nc, const int* matrix, const int length ) ;
  Matrix( const int nr, const int nc, const int* matrix, const int length, const int lo, const int up ) ;

  virtual ~Matrix() ;

  //void addRow( VarArray& x );
  //void addColumn( VarArray& x );
  
  /*!@name Operators*/
  Variable& cell( const int i, const int j ) ;
  VarArray& row( const int i ) ;
  VarArray& column( const int j ) ;


  virtual void printshort(std::ostream& o) const ;
  virtual void print(std::ostream& o) const ;
};


/**********************************************
 *  Offset Predicate Wrapper
 **********************************************/ 
class Offset : public Variable {
public:
  Offset( Variable& x, int o );
};
/**********************************************
 *  Factor Predicate Wrapper
 **********************************************/ 
class Factor : public Variable {
public:
  Factor( Variable& x_, int o );
};
/**********************************************
 *  Negation Predicate Wrapper
 **********************************************/ 
class Negation : public Variable {
public:
  Negation( Variable& x_ );
};
/**********************************************
 *  Not Predicate Wrapper
 **********************************************/ 
class Not : public Variable {
public:
  Not( Variable& x_ );
};
/**********************************************
 *  Abs Predicate Wrapper
 **********************************************/ 
class Abs : public Variable {
public:
  Abs( Variable x_ );
};
/**********************************************
 *  And Predicate Wrapper
 **********************************************/ 
class And : public Variable {
public:
  And( Variable& x_, Variable& y_ );
};
/**********************************************
 *  Or Predicate Wrapper
 **********************************************/ 
class Or : public Variable {
public:
  Or( Variable& x_, Variable& y_ );
};
/**********************************************
 *  IfThenElse Predicate Wrapper
 **********************************************/ 
class IfThenElse : public Variable {
public:
  IfThenElse( Variable& x_, Variable& y_, Variable& z_ );
};
/**********************************************
 *  Disjunctive Predicate Wrapper
 **********************************************/ 
class Disjunctive : public Variable {
public:
  Disjunctive( Variable& x, int dx, Variable& y, int dy );
};
/**********************************************
 *  Precedence Predicate Wrapper
 **********************************************/ 
class Precedence : public Variable {
public:
  Precedence( Variable& x, int d, Variable& y );
  Precedence( Variable& x, int b );
  Precedence( int b, Variable& x );
};
/**********************************************
 *  Add Predicate Wrapper
 **********************************************/
class Add : public Variable {
public:
  Add( Variable& x, Variable& y );
};
/**********************************************
 *  Sub Predicate Wrapper
 **********************************************/
class Sub : public Variable {
public:
  Sub( Variable& x, Variable& y );
};
/**********************************************
 *  Mul Predicate Wrapper
 **********************************************/
class Mul : public Variable {
public:
  Mul( Variable& x, Variable& y );
};
/**********************************************
 *  Div Predicate Wrapper
 **********************************************/
class Div : public Variable {
public:
  Div( Variable& x, Variable& y );
};
/**********************************************
 *  Pow Predicate Wrapper
 **********************************************/
class Pow : public Variable {
public:
  Pow( Variable& x_, Variable& y_ );
};
/**********************************************
 *  Mod Predicate Wrapper
 **********************************************/
class Mod : public Variable {
public:
  Mod( Variable& x_, Variable& y_ );
  Mod( Variable& x_, const int k );
};
/**********************************************
 *  Equal Predicate Wrapper
 **********************************************/ 
class Equal : public Variable {
public:
  Equal( Variable& x_, Variable& y_ );
  Equal( Variable& x_, int k );
};
/**********************************************
 *  NotEqual Predicate Wrapper
 **********************************************/ 
class NotEqual : public Variable {
public:
  NotEqual( Variable& x_, Variable& y_ );
  NotEqual( Variable& x_, int k );
};
/**********************************************
 *  NotEqualWildcard Predicate Wrapper
 **********************************************/ 
class NotEqualWildcard : public Variable {
public:
  NotEqualWildcard( Variable& x_, Variable& y_, const int w );
};
/**********************************************
 * Sum Predicate Wrapper
 **********************************************/
class Sum : public Variable {
public:
  Sum( VarArray& x, const int *os=NULL );
};
/**********************************************
 * Min Predicate Wrapper
 **********************************************/
class Min : public Variable {
public:
  Min( VarArray& x );
};
/**********************************************
 * Max Predicate Wrapper
 **********************************************/
class Max : public Variable {
public:
  Max( VarArray& x );
};
/**********************************************
 * Element Predicate Wrapper
 **********************************************/
class Element : public Variable {
public:
  Element( VarArray& x, Variable& y );
  Element( Variable& y, VarArray& x, Variable& z );
  Element( VarArray& x );
};
/**********************************************
 *  Clause Wrapper
 **********************************************/ 
class Clause : public Variable {
public:
  Clause( VarArray& x, const int* c );
};
/**********************************************
 *  Extensional Constraint Wrapper
 **********************************************/ 
class Table : public Variable {
private:
  BuildObjectTable *tabptr_;

public:
  Table();
  Table( VarArray& x, const bool sp=true );
  Table( VarArray& x, const int tidx, const bool sp );
  
  void add( const int* sol );
};
/**********************************************
 * AllDifferent Constraint Wrapper
 **********************************************/
class AllDifferent : public Variable {
public:
  AllDifferent( VarArray& x );
};
/**********************************************
 * BoundAllDifferent Constraint Wrapper
 **********************************************/
class BoundAllDifferent : public Variable {
public:
  BoundAllDifferent( VarArray& x );
};
/**********************************************
 * Gcc Constraint Wrapper
 **********************************************/
class Gcc : public Variable {
public:
  Gcc( VarArray& x,
       const int firstDomainValue, 
       const int lastDomainValue,
       const int* minOccurrences,
       const int* maxOccurrences );
};
/**********************************************
 * EdgeFinder Constraint Wrapper
 **********************************************/
class EdgeFinder : public Variable {
public:
  EdgeFinder( VarArray& x,
	      const int *d );
};
/**********************************************
 * Lex < Constraint Wrapper
 **********************************************/
class LexOrder : public Variable {
public:
  LexOrder( VarArray& x, VarArray& y, int eq=1 );
};
/**********************************************
 * MaxOccurrence < Constraint Wrapper
 **********************************************/
class MaxOccurrence : public Variable {
public:
  MaxOccurrence( VarArray& x, const int k, const int ub  );
};
/**********************************************
 * MinOccurrence < Constraint Wrapper
 **********************************************/
class MinOccurrence : public Variable {
public:
  MinOccurrence( VarArray& x, const int k, const int lb  );
};
/**********************************************
 * Slide Constraint Wrapper
 **********************************************/
class Slide : public Variable {
public:
  Slide( VarArray& x, const int n, const int m, CSP* p );
};


#ifdef _CPLEX
/**********************************************
 * CPlexMinDifference Predicate Wrapper
 **********************************************/
class CPlexMinDifference : public Variable {
public:
  CPlexMinDifference( VarArray& x, const int ft ) ;
};
#endif

#ifdef _COINLP
/**********************************************
 * MaxDifference Predicate Wrapper
 **********************************************/
class MaxDifference : public Variable {
public:
  MaxDifference( VarArray& x, const int mt, const int ft ) ;
};
/**********************************************
 * MinDifference Predicate Wrapper
 **********************************************/
class MinDifference : public Variable {
public:
  MinDifference( VarArray& x, const int mt, const int ft ) ;
};
#endif




class Objective {
public:
  BuildObject *X;
  BuildObjectObjective *obj;
  
  Objective(BuildObject *x);// : X(x) { obj = NULL; }
};

/// Maximisation wrapper
class Maximise : public Objective {
public:
  Maximise(Variable x) ;
};
/// Minimisation wrapper
class Minimise : public Objective {
public:
  Minimise(Variable x) ;
};


/**********************************************
 * CSP
 **********************************************/
/*! \class CSP
  \brief Implementation of a Constraint network.

  This implementation is composed
  of an array of pointers to Variables, and
  a list of pointers to Constraints
*/
class CSP {

public:

  static BuildObject* _Variable( const int nv );
  static BuildObject* _Variable( const int lb, const int ub );
  static BuildObject* _Variable( const int *array, const int l );
  static BuildObject* _Variable( const int *array, const int l,
				const int lb, const int ub );


  static BuildObject** _VarArray( const int n, const int nv );
  static BuildObject** _VarArray( const int n, const int lb, const int ub );
  static BuildObject** _VarArray( const int n, const int *array, const int length );
  static BuildObject** _VarArray( const int n, const int *array, const int length, 
				  const int lb, const int ub );
  
  static BuildObject* _Offset( BuildObject *x, int o );
  static BuildObject* _Factor( BuildObject *x, int o );
  static BuildObject* _Negation( BuildObject *x );
  static BuildObject* _Not( BuildObject *x );
  static BuildObject* _Abs( BuildObject *x );
  static BuildObject* _And( BuildObject *x, BuildObject *y );
  static BuildObject* _Or( BuildObject *x, BuildObject *y );
  static BuildObject* _IfThenElse( BuildObject *x, BuildObject *y, BuildObject *z );
  static BuildObject* _Disjunctive( BuildObject *x, int dx, BuildObject *y, int dy );
  static BuildObject* _Precedence( BuildObject *x, int d, BuildObject *y );
  static BuildObject* _Precedence( BuildObject *x, int b );
  static BuildObject* _Precedence( int b, BuildObject *x );
  static BuildObject* _Add( BuildObject *x, BuildObject *y );
  static BuildObject* _Sub( BuildObject *x, BuildObject *y );
  static BuildObject* _Mul( BuildObject *x, BuildObject *y );
  static BuildObject* _Div( BuildObject *x, BuildObject *y );
  static BuildObject* _Pow( BuildObject *x, BuildObject *y );
  static BuildObject* _Mod( BuildObject *x, BuildObject *y );
  static BuildObject* _Mod( BuildObject *x, const int k );
  static BuildObject* _Equal( BuildObject *x, BuildObject *y );
  static BuildObject* _Equal( BuildObject *x, int k );
  static BuildObject* _NotEqual( BuildObject *x, BuildObject *y );
  static BuildObject* _NotEqual( BuildObject *x, int k );
  static BuildObject* _NotEqualWildcard( BuildObject *x, 
					BuildObject *y, const int w );
  static BuildObject* _Sum( BuildObject **x, const int nx, const int *os=NULL );
  static BuildObject* _Min( BuildObject **x, const int nx );
  static BuildObject* _Max( BuildObject **x, const int nx );
  static BuildObject* _Min( BuildObject *x, BuildObject *y );
  static BuildObject* _Max( BuildObject *x, BuildObject *y );
  static BuildObject* _Element( BuildObject **x, const int nx, BuildObject *y );
  static BuildObject* _Element( BuildObject *y, BuildObject **x, 
			       const int nx, BuildObject *z );
  static BuildObject* _Element( BuildObject **x, const int nx );
  static BuildObject* _Clause( BuildObject **x, const int nx, const int* c );
  static BuildObject* _Table( BuildObject **x, const int nx, const bool sp=true );
  static BuildObject* _Table( BuildObject **x, const int nx, 
			     const int tidx, const bool sp );
  static int addTable( const int a, const int n );
  static void addTuple( const int idx, int *tuple );
  static BuildObject* _AllDifferent( BuildObject **x, const int nx );
  static BuildObject* _BoundAllDifferent( BuildObject **x, const int nx );
  static BuildObject* _Gcc( BuildObject **x, const int nx,
			   const int firstDomainValue, 
			   const int lastDomainValue,
			   const int* minOccurrences,
			   const int* maxOccurrences );
  static BuildObject* _EdgeFinder( BuildObject **x, const int nx, const int *d );
  static BuildObject* _LexOrder( BuildObject **x, BuildObject **y, 
				const int nx, int eq=1 );
  static BuildObject* _MaxOccurrence( BuildObject **x, const int nx, 
				     const int k, const int ub  );
  static BuildObject* _MinOccurrence( BuildObject **x, const int nx, 
				     const int k, const int lb  );
  static BuildObject* _Slide( BuildObject **x, const int nx, const int m, CSP* p );

#ifdef _CPLEX
  static BuildObject* _CPlexMinDifference( BuildObject **x, const int nx, const int ft ) ;
#endif

#ifdef _COINLP
  static BuildObject* _MaxDifference( BuildObject **x, const int nx, 
				     const int mt, const int ft ) ;
  static BuildObject* _MinDifference( BuildObject **x, const int nx, 
				     const int mt, const int ft ) ;
#endif

 static BuildObjectObjective* _Maximise( BuildObject *x );
 static BuildObjectObjective* _Minimise( BuildObject *x );
  
public:

  //static ConstraintStore ENVIRONMENT;

  /*!@name Parameters*/
  //@{
  // stats
  unsigned int nList;
  unsigned int eList;
  unsigned int nBit;
  unsigned int eBit;
  unsigned int nRange;
  unsigned int eRange;
  unsigned int nBoolean;
  unsigned int eBoolean;
  unsigned int nConstant;
  unsigned int eConstant;
  unsigned int nValues;
  unsigned int eValues;
  unsigned int nConstraint;
  unsigned int eConstraint;
  unsigned int nNeq;

  // objective function
  BuildObjectObjective *goald; 

  int numVars() const ;
  int numEVars() const ;

  // Model declaration
  MistralStack<BuildObject*> declarations; 
  MistralGacStack<BuildObject*> buildstack;

  // Infered not-equals
  MistralStack<BuildObject*> notequals;
  int inferAllDiffs( const int all, const int limit );

  inline void triggerEvent(BuildObject* x, int e) ;
  //inline void add( BuildObject *d, const int lvl ) ;
  void close() ;
  //@}

  /*!@name Constructors*/
  //@{
  CSP() ;
  virtual ~CSP() ; 
   
  void cleanUp() ;
  //@}
  
  /*!@name Accessors*/
  //@{
  int size() ;
  //@}

  /*!@name Accessors*/
  //@{
  /// Add a single variable x 
  void add( BuildObject *x ) ;
  void add( BuildObjectObjective *x ) ;
  void add( BuildObject **x, const int n ) ;

  void add( Variable x ) ;
  /// Add an array of variables 
  void add( VarArray& x ) ;
  /// Add an objective function g 
  void add( Objective g ) ;
  /// Build the data structure used during search;
  void build( Solver *s ) ;
  void preBuild( ) ;
  //@}
  
  /*!@name Miscellaneous*/
  //@{
  /// Printshort the CSP
  void print(std::ostream&) const ;
  void printXML(std::ostream&) const ;
  //@}
};


ostream& operator<< (ostream& os, const MistralGraph& g) ;
ostream& operator<< (ostream& os, const Variable& x) ;
ostream& operator<< (ostream& os, const VarArray& x) ;
ostream& operator<< (ostream& os, const Matrix& x) ;
ostream& operator<< (ostream& os, const CSP& p) ;


#endif // __Wrapper_H



