
/*
  Mistral is a constraint satisfaction and optimisation library
  Copyright (C) 2003-2005  Emmanuel Hebrard
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  The author can be contacted electronically at 
  ehebrard@cse.unsw.edu.au.
*/

/**********************************************
 * DVOs and Random number generator
 **********************************************/

#include <sol.h>
#include <csp.h>
#include <dvo.h>

#include <math.h>


Weighter::Weighter( Solver* s ) 
  : level(s->level)
{

}

WeighterDegree::WeighterDegree( Solver* s ) 
  : Weighter(s)
{
}

void WeighterDegree::notifyFailure( Constraint *con )
{
  ++con->weight;
  for(int i=0; i<con->arity; ++i)
    ++con->scope[i]->weight;
}

WeighterLevelDegree::WeighterLevelDegree( Solver* s )
  : Weighter(s)
{
  lmax = (int)(log2((double)(s->length)));
}

void WeighterLevelDegree::notifyFailure( Constraint *con )
{
  if( lmax < level ) 
    lmax = level;
  con->weight += (1 + lmax - level);
  for(int i=0; i<con->arity; ++i)
    con->scope[i]->weight += (1 + lmax - level);
}


void WeighterImpact::print( ostream& o ) const
{
  for(int i=0; i<ilength; ++i) 
    {
      cout << "c ";
      variables[i]->printshort( cout );
      cout << ":\t";
      int v=variables[i]->min();
      do {
	cout << " " << setprecision(2) << decision_impact[i][v] << ":" << v; 
      } while( variables[i]->setNext(v) );
      cout << endl;
    }
}

WeighterImpact::~WeighterImpact( )
{
  for(int i=0; i<ilength; ++i) 
    {
      decision_impact[i] += varmin[i];
      decision_count [i] += varmin[i];

      delete [] decision_impact[i];
      delete [] decision_count [i];
    }
  delete [] decision_impact;
  delete [] decision_count;
  delete [] X;
  delete [] domain_size;
  delete [] varmin;
  delete [] isRange;
}

WeighterImpact::WeighterImpact( Solver* s ) 
  : Weighter(s), 
    decision(s->decision.stack_),
   first(s->future), 
    last(s->empty),
    variables(s->variables.stack_)
{
  int i, m;

  ilength = s->length;
  varmin = new int[ilength];
  isRange = new bool[ilength];
  decision_impact = new double*[ilength];
  decision_count  = new unsigned int*[ilength];
  X = new VariableInt*[ilength];
  domain_size = new unsigned int[ilength];
    
  for(i=0; i<ilength; ++i) 
    {
      isRange[i] = ( s->variables[i]->getType() == VariableInt::RANGE );
      
      if( isRange[i] ) 
	{
	  decision_impact[i] = new double[2];
	  decision_count [i] = new unsigned int[2];
	  
	  std::fill( decision_count [i], decision_count [i]+2, 1 );
	  std::fill( decision_impact[i], decision_impact[i]+2, 1 );
	  
	  varmin[i] = 0;
	}
      else 
	{
	  m = (s->variables[i]->maxCapacity() - s->variables[i]->minCapacity());

	  decision_impact[i] = new double[m];
	  decision_count [i] = new unsigned int[m];
	  
	  std::fill( decision_count [i], decision_count [i]+m, 1 );
	  std::fill( decision_impact[i], decision_impact[i]+m, 1 );
	  
	  varmin[i] = s->variables[i]->minCapacity();

	  decision_impact[i] -= varmin[i];
	  decision_count [i] -= varmin[i];
	}
    }
}

void WeighterImpact::notifyFailure( Constraint *con )
{
  if( needUpdate )
    {
      // as a result of the decision, the problem is emptied
      // the inverse impact therefore is 0.
      int x = decision[level]->id;
      int v;

      if( isRange[x] ) 
	v = ((VariableRange*)(decision[level]))->bound();
      else 
	v = decision[level]->value();
      decision_impact[x][v] = 
	(decision_impact[x][v] * decision_count[x][v]) / ++decision_count[x][v];
      needUpdate = false;
    }
}

void WeighterImpact::notifySuccess( )
{
  // Compute the reduction Sa/Sb
  double reduction = 1;
  int x, v;
  
  for(x=0; x<nbvars; ++x)
    if( X[x]->domsize() < domain_size[x] ) {
      reduction *= ((double)(X[x]->domsize()) / (double)(domain_size[x]));
    }

  x = decision[level]->id;

  if( isRange[x] ) 
    v = ((VariableRange*)(decision[level]))->bound();
  else 
    v = decision[level]->value();
  decision_impact[x][v] = 
    (((decision_impact[x][v]) * (decision_count[x][v])) + reduction) / ++(decision_count[x][v]);

  needUpdate = false;
}

void WeighterImpact::notifyChoice( )
{
  VariableInt **var_iterator;
  nbvars=0;
  for( var_iterator = first; var_iterator != last; ++var_iterator ) {
    // store the var and its size
    X[nbvars] = (*var_iterator);
    domain_size[nbvars] = X[nbvars]->domsize();
    ++nbvars;
  }
  needUpdate =true;
} 

 bool WeighterSAC::isNotSac() { 
   return ins;
}

WeighterSAC::~WeighterSAC() 
{
  delete [] SACdomain;
  delete [] isRange;
}

WeighterSAC::WeighterSAC(Solver* s) 
  : Weighter(s)
{
  complete = false;
  ins = true;
  solver = s;
  ilength = s->length;
  needPruning = false;
  SACdomain = new MistralSet[ilength];
  isRange = new bool[ilength];
  sacvals = 0;
  domsize = 0;
  for(int i=0; i<ilength; ++i) 
    {
      isRange[i] = ( s->variables[i]->getType() == VariableInt::RANGE );

      if( isRange[i] )
	{
	  SACdomain[i].init( 0, 1, MistralSet::empt );
	  domsize += 2;
	}
      else 
	{
	  SACdomain[i].init( s->variables[i]->min(),
			     s->variables[i]->max(),
			     MistralSet::empt );
	  domsize += s->variables[i]->domsize();
	  if( s->variables[i]->isGround() ) {
	    ++sacvals;
	  }
	}
    }
}


void WeighterSAC::notifyChoice()
{
  int i, j;
  if( level == 1 && needPruning )
    {
      domsize = 0;
      if( complete ) 
	{
	  j=solver->decision[1]->id;
	  //sacvals = SACdomain[j].size();
	  sacvals = 0;
	  for(i=0; i<ilength; ++i)
	    {
	      domsize += solver->variables[i]->domsize();
	      //if( i != j ) 
	      SACdomain[i].clear();
	    }
	}
      else
	{
	  for(i=0; i<ilength; ++i)
	    domsize += solver->variables[i]->domsize();
	}
    }
  needPruning = false;
}


inline void WeighterSAC::notifyFailure( Constraint *con ) 
{
  int i, j, k;
  if( level > 1 ) 
    {
      ins =false;

      VariableInt **var_iterator = solver->sequence,
	**last = solver->future;
      
      while( var_iterator < last )
	{
	  j = (*var_iterator)->id;
	  if( isRange[j] )
	    k = ((VariableRange*)(*var_iterator))->bound();
	  else
	    k = (*var_iterator)->value();

	  ++var_iterator;
	  if( SACdomain[j].member(k) ) continue;

	  SACdomain[j].insert(k);
	  ins = true;
	  ++sacvals;
	}
      ins = (ins && (domsize > sacvals));
    }
  else if( level == 1 )
    {
      ++solver->FAILLIMIT;
      needPruning = true;
    }
}

DVOSingletonAC::DVOSingletonAC(Solver* s, WeighterSAC* w) 
  : DVO(s), SACdomain(w->SACdomain), isRange(w->isRange)
{
}

inline VariableInt* DVOSingletonAC::select() 
{
  VariableInt *var, **var_iterator=first;
  int i, j;

  do {
    var = *var_iterator;
    i=var->id;
  } while( 
	  ((isRange[i] && SACdomain[i].empty())
	   ||
	   (!isRange[i] && var->included( SACdomain[i] )))
	  && 
	  ++var_iterator != last );
  
  return var;
}


/**********************************************
 * DVOs Dynamic Variable Ordering
 **********************************************/

DVO::DVO(Solver *s) 
  : variables(s->variables.stack_),
    first(s->future), 
    last(s->empty), 
    decision(s->decision.stack_), 
    level(s->level)
{
}

inline VariableInt* DVONoOrder::select() 
{
  return *first;
}

inline VariableInt* DVOLexicographic::select() 
{
  int i=0;
  while( !variables[i]->isLinked() ) 
    {
      ++i;
    }
  return variables[i];
}

DVO* DomOverWLDeg::extract( Solver* s )
{
  s->setLearner( Weighter::WLD );
  if( size > 1 )
    return new GenericRandomDVO<VarSelectorDomainOverWeight>(s, size);
  else 
    return new GenericDVO<VarSelectorDomainOverWeight>(s);
}


DVO* DomOverWDeg::extract( Solver* s )
{
  s->setLearner( Weighter::WDG );
  if( size > 1 )
    return new GenericRandomDVO<VarSelectorDomainOverWeight>(s, size);
  else 
    return new GenericDVO<VarSelectorDomainOverWeight>(s);
}



void linkImpact( Solver* s, double**** bdi, double*** cdi, const int size ) 
{
  WeighterImpact *w = ((WeighterImpact*)(s->setLearner( Weighter::IPT )));
  for(int i=0; i<s->length; ++i)
    {
      if( s->sequence[i]->getType() != VariableInt::RANGE )
	s->sequence[i]->branch = new ValSelectorWeight( s->sequence[i], w->decision_impact[i] );
    }
  for(int i=0; i<size; ++i) 
    *(bdi[i]) = w->decision_impact;
  *cdi = w->decision_impact;
  //  cout << "o impact " << (w->decision_impact) << endl;
}

DVO* Impact::extract( Solver* s )
{
  DVO *h;
  if( !size ) size=1;
  double ***bdi[size];
  double ***cdi;
  GenericRandomDVO<VarSelectorImpact> *hr;
  if( size > 1 ) {
    GenericRandomDVO<VarSelectorImpact> *hr = 
      new GenericRandomDVO<VarSelectorImpact>(s, size);
    for(int i=0; i<size; ++i)
      bdi[i] = &(hr->bests[i].decision_impact);
    cdi = &(hr->current.decision_impact);
    h = hr;
  } else {
    GenericDVO<VarSelectorImpact> *hn = 
      new GenericDVO<VarSelectorImpact>(s);
    bdi[0] = &(hn->best.decision_impact);
    cdi = &(hn->current.decision_impact);
    h = hn;
  }
  linkImpact( s, bdi, cdi, size );

//   cout << "r impact " << (((GenericDVO<VarSelectorImpact> *)h)->best.decision_impact) << endl;
//   cout << "r impact " << (((GenericDVO<VarSelectorImpact> *)h)->current.decision_impact) << endl;

  return h;
}

DVO* ImpactOverDeg::extract( Solver* s )
{
  DVO *h;
  if( !size ) size=1;
  double ***bdi[size];
  double ***cdi;
  if( size > 1 ) {
    GenericRandomDVO<VarSelectorImpactOverDegree> *hr = 
      new GenericRandomDVO<VarSelectorImpactOverDegree>(s, size);
    for(int i=0; i<size; ++i)
      bdi[i] = &(hr->bests[i].decision_impact);
    cdi = &(hr->current.decision_impact);
    h = hr;
  } else {
    GenericDVO<VarSelectorImpactOverDegree> *hn = 
      new GenericDVO<VarSelectorImpactOverDegree>(s);
    bdi[0] = &(hn->best.decision_impact);
    cdi = &(hn->current.decision_impact);
    h = hn;
  }
  linkImpact( s, bdi, cdi, size );
  return h;
}

DVO* ImpactOverWLDeg::extract( Solver* s )
{
  DVO *h;
  if( !size ) size=1;
  double ***bdi[size];
  double ***cdi;
  if( size > 1 ) {
    GenericRandomDVO<VarSelectorImpactOverWeight> *hr = 
      new GenericRandomDVO<VarSelectorImpactOverWeight>(s, size);
    for(int i=0; i<size; ++i)
      bdi[i] = &(hr->bests[i].decision_impact);
    cdi = &(hr->current.decision_impact);
    h = hr;
  } else {
    GenericDVO<VarSelectorImpactOverWeight> *hn = 
      new GenericDVO<VarSelectorImpactOverWeight>(s);
    bdi[0] = &(hn->best.decision_impact);
    cdi = &(hn->current.decision_impact);
    h = hn;
  }
  linkImpact( s, bdi, cdi, size );
  return h;
}

DVO* ImpactOverWDeg::extract( Solver* s )
{
  DVO *h;
  if( !size ) size=1;
  double ***bdi[size];
  double ***cdi;
  if( size > 1 ) {
    GenericRandomDVO<VarSelectorImpactOverWeight> *hr = 
      new GenericRandomDVO<VarSelectorImpactOverWeight>(s, size);
    for(int i=0; i<size; ++i)
      bdi[i] = &(hr->bests[i].decision_impact);
    cdi = &(hr->current.decision_impact);
    h = hr;
  } else {
    GenericDVO<VarSelectorImpactOverWeight> *hn = 
      new GenericDVO<VarSelectorImpactOverWeight>(s);
    bdi[0] = &(hn->best.decision_impact);
    cdi = &(hn->current.decision_impact);
    h = hn;
  }
  linkImpact( s, bdi, cdi, size );
  return h;
}

DVO* Probe::extract( Solver* s )
{
  s->setLearner( ltype );

  switch( htype ) {
  case Weighter::NO : return new DVORandom(s);
  case Weighter::WDG : return new GenericDVO<VarSelectorDomainOverWeight>(s);
  case Weighter::WLD : return new GenericDVO<VarSelectorDomainOverWeight>(s);
  case Weighter::IPT : return new GenericDVO<VarSelectorImpact>(s);
  }
}


DVORandom::DVORandom(Solver* s) : DVO(s) 
{
}


inline VariableInt* DVORandom::select() 
{
  return first[(rand() % ( last - first ) )];
}


/**********************************************
 * Objective Functions 
 **********************************************/

int ObjectiveFunction::update()
{
  int nscore = score();
  if( nscore < upper_bound )
    upper_bound = nscore;
  if(upper_bound == 0) 
    return Solver::SAT;
  return Solver::UNSAT; 
}

MaximiseVar::MaximiseVar(Solver *s, VariableInt *x) 
{
  cout << "NEW" << endl;
  umore = new UnaryConstraintMore(s, x, x->min());
  maxX = x->max();
  upper_bound = (maxX - x->min());
}

MaximiseVar::~MaximiseVar() 
{
  delete umore;
}

int MaximiseVar::score() 
{
  return (maxX - umore->X->max());
}

int MaximiseVar::update() 
{
  int res = ObjectiveFunction::update();
  if( res == Solver::UNSAT ) {
    umore->bound = (maxX - upper_bound + 1);
    umore->activate();
  }

  return res; 
}

MinimiseVar::MinimiseVar(Solver *s, VariableInt *x) 
{
  uless = new UnaryConstraintLess(s, x, x->max());
  minX = x->min();
  upper_bound = x->max();
}

MinimiseVar::~MinimiseVar() 
{
  delete uless;
}

int MinimiseVar::score() 
{
  return uless->X->min();
}

int MinimiseVar::update() 
{
  int res = ObjectiveFunction::update();

  if( res == Solver::UNSAT ) {
    uless->bound = (upper_bound - 1);
    uless->activate();
  }
  return res; 
}


