
/*
  Mistral is a constraint satisfaction and optimisation library
  Copyright (C) 2003-2005  Emmanuel Hebrard
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  The author can be contacted electronically at 
  ehebrard@cse.unsw.edu.au.
*/

#include <mod.h>
#include <sol.h>


static ConstraintStore ENVIRONMENT;



void BuildObject::add( CSP *m, const int l, const int info )
{
  ++refs;

  if( l ) setVar();
  else if(isRel()) unsetVar();
  
  if( notIn() ) {
    model = m;
    id = model->declarations.size() ;
    model->declarations.push( this ) ;
    //model->add( this, l );
  }
}


/**********************************************
 * Variable BuildObject
 **********************************************/

void BuildObject::initVar(const int lb, const int ub) 
{
  refs = 0;
  id = -1;
  min_ = lb;
  max_ = ub;
  size_ = (ub-lb+1);
  state = (CRANGE | CLIST | PAR | VLIST | ((size_ > 0 ? VRANGE : 0) | (size_ == 1 ? CONST : 0)));
  veqptr_ = NULL;
  varptr_ = NULL;
  domain_ = NULL;
  model = NULL;
}

BuildObject::BuildObject() 
{
  initVar(0,-1);
}

BuildObject::BuildObject( const int lo, const int up ) 
{
  initVar(lo, up);
}

BuildObject::BuildObject( const int* array, const int length ) 
{
  min_ = INT_MAX;
  max_ = -INT_MAX;
  for(int i=0; i<length; ++i) {
    if(min_ > array[i]) min_ = array[i];
    if(max_ < array[i]) max_ = array[i];
  }
  initVar(min_, max_);
  size_ = length;
  if( length < (max_ - min_ + 1) ) {
    state |= (size_ == 1 ? CONST : 0); 
    domain_ = new MistralSet(min_, max_, MistralSet::empt);
    for(int i=0; i<length; ++i) domain_->insert( array[i] );
  } else state |= (VRANGE | (size_ == 1 ? CONST : 0));
}

BuildObject::BuildObject( const int* array, 
			  const int length, 
			  const int lo, 
			  const int up ) 
{
  initVar(lo, up);
  size_ = length;
  if( length < (max_ - min_ + 1) ) {
    state |= (size_ == 1 ? BuildObject::CONST : 0);
    domain_ = new MistralSet(min_, max_, MistralSet::empt);
    for(int i=0; i<length; ++i) domain_->insert( array[i] );
  } else state |= (VRANGE | (size_ == 1 ? CONST : 0));
}

BuildObject* BuildObject::getBuildObject()
{
  return (veqptr_ ? veqptr_->getBuildObject() : this);
}

const BuildObject* BuildObject::getConstBuildObject() const
{
  return (veqptr_ ? veqptr_->getConstBuildObject() : this);
}

void BuildObject::build(Solver *s)
{
  if( !veqptr_ && (isRel() || isReferenced()) ) {
    if( size_ == 1) {
      map<int,VariableInt*>::iterator iter = s->constants.find( (int)min_ );
      if( iter != s->constants.end() ) {
	varptr_ = iter->second;
      } else {// this constant as not yet been created
	varptr_ = new Constant( s, min_ );
	s->constants[(int)min_] = varptr_;
      }
    } else if( size_ == 2 && min_ == 0 && max_ == 1 ) {
      varptr_ = new VariableBool(s);
    } else if( size_ == (max_ - min_ + 1) ) {
      if( isVRange() && size_ > 64 ) {
	varptr_ = new VariableRange( s, min_, max_ );
      } else {
	if( isVList() ) {
	  varptr_ = new VariableList( s, min_, max_ );
	} else {
	  varptr_ = new VariableBit( s, min_, max_ );
	}
      }
    } else {
      if( isVList() ) {
	varptr_ = new VariableList( s, domain_, size_, min_, max_ );
      } else {
	varptr_ = new VariableBit( s, domain_, size_, min_, max_ );
      }
    }
  }
}

void BuildObject::releaseEvent( const int sb )
{
  int sn = size(), evt;
  if( sb != sn ) {
    evt = (sn == 1 ? 7 : 1);
    model->triggerEvent( this, evt );
    if( veqptr_ ) model->triggerEvent( getBuildObject(), evt );
  }
}

void BuildObject::releaseEvent( )
{
  model->triggerEvent( this, VALUETRIGGER );
  if( veqptr_ ) model->triggerEvent( getBuildObject(), VALUETRIGGER );
}


int BuildObject::_size() const
{
  return size_;
}

int BuildObject::size() const
{
  return getConstBuildObject()->_size();
}

int BuildObject::_min() const
{
  return min_;
}

int BuildObject::min() const
{
  return getConstBuildObject()->_min();
}

int BuildObject::_max() const
{
  return max_;
}

int BuildObject::max() const
{
  return getConstBuildObject()->_max();
}

int BuildObject::_domsize() const
{
  return size_;
}

int BuildObject::domsize() const
{
  return getConstBuildObject()->_domsize();
}

int BuildObject::_value() const
{
  return min_;
}

int BuildObject::value() const
{
  return getConstBuildObject()->_value();
}

int BuildObject::_first() const
{
  return min_;
}

int BuildObject::first() const
{
  return getConstBuildObject()->_first();
}

int BuildObject::_last() const
{
  return max_;
}

int BuildObject::last() const
{
  return getConstBuildObject()->_last();
}

int BuildObject::_minCapacity() const
{
  return min_;
}

int BuildObject::minCapacity() const
{
  return getConstBuildObject()->_minCapacity();
}

int BuildObject::_maxCapacity() const
{
  return max_;
}

int BuildObject::maxCapacity() const
{
  return getConstBuildObject()->_maxCapacity();
}

int BuildObject::_minPosAbs() const 
{
  if( max_ < 0 ) return max_;
  if( min_ > 0 ) return min_;
  if( !domain_ || domain_->member( 0 ) ) return 0;
  return domain_->next( 0 );
}

int BuildObject::minPosAbs() const
{
  return getConstBuildObject()->_minPosAbs();
}

int BuildObject::_minNegAbs() const 
{
  if( min_ > 0 ) return min_;
  if( max_ < 0 ) return max_;
  if( !domain_ || domain_->member( 0 ) ) return 0;
  return domain_->prev( 0 );
}

int BuildObject::minNegAbs() const
{
  return getConstBuildObject()->_minNegAbs();
}

bool BuildObject::_remove(const int v) 
{
  if( min_ <= v && max_ >= v ) {
    if( domain_ != NULL ) {
      if(domain_->member(v)) {
	domain_->erase( v );
	--size_;
	if( v == min_ )
	  min_ = domain_->min();
	if( v == max_ )
	  max_ = domain_->max();
      }
    } else {
      if( v == min_ ) {
	--size_;
	++min_;
      } else if( v == max_ ) {
	--size_;
	--max_;
      } else {
	domain_ = new MistralSet(min_, max_, MistralSet::full);
	domain_->erase( v );
	--size_;
      }
    }
  }
 
  return size_ > 0;
}

bool BuildObject::remove(const int v) 
{
  int sb = size();
  bool rval = getBuildObject()->_remove( v );
  releaseEvent( sb );
  return rval;
}

bool BuildObject::_setDomain(const int v) 
{
  if( domain_ != NULL )
    delete domain_;
  size_ = 1;
  min_ = v;
  max_ = v;
  return size_ > 0;
}

bool BuildObject::setDomain(const int v) 
{
  if( size() > 1 ) {
    releaseEvent();
    setConst();
  }
  return getBuildObject()->_setDomain( v );
}

void BuildObject::_setEqual(BuildObject *v) 
{
  if( !v->isVRange() )
    unsetVRange();
  if( v->domain_ ) {
    if( domain_ == NULL ) 
      domain_ = new MistralSet( min_, max_, MistralSet::full );
    domain_->intersectWith( v->domain_ );
    min_ = domain_->min();
    max_ = domain_->max();
    size_ = domain_->size();
  } else {
    _setMin( v->min_ );
    _setMax( v->max_ );
  }
  v->veqptr_ = this;
}

void BuildObject::setEqual(BuildObject *v) 
{
  if( getConstBuildObject() != v->getConstBuildObject() )
    getBuildObject()->_setEqual( v->getBuildObject() );
}

bool BuildObject::_setMin(const int l) 
{
  if( l > min_ ) { 
    if( domain_ != NULL ) {
      domain_->setMin(l);
      size_ = domain_->size();
      min_ = domain_->min();
    } else {
      size_ -= ( l - min_ );
      min_ = l;
    }
  }
  return size_ > 0;
}

bool BuildObject::setMin(const int l) 
{
  return getBuildObject()->_setMin( l );
}

bool BuildObject::_setMax(const int u)
{
  if( u < max_ ) { 
    if( domain_ != NULL ) {
      domain_->setMax(u);
      size_ = domain_->size();
      max_ = domain_->max();
    } else {
      size_ -= ( max_ - u);
      max_ = u;
    }
  }
  return size_ > 0;
}

bool BuildObject::setMax(const int u) 
{
  return getBuildObject()->_setMax( u );
}

bool BuildObject::_setDomain(const MistralSet& s) 
{
  if( domain_ == NULL ) 
    domain_ = new MistralSet(min_, max_, MistralSet::full);
  domain_->intersectWith( s );
  size_ = domain_->size();
  min_ = domain_->min();
  max_ = domain_->max();
  return size_ > 0;
}

bool BuildObject::setDomain(const MistralSet& s) 
{
  return getBuildObject()->_setDomain( s );
}

bool BuildObject::_removeSet(const MistralSet& s) 
{
  if( domain_ == NULL ) 
    domain_ = new MistralSet(min_, max_, MistralSet::full);
  domain_->setminusWith( s );
  size_ = domain_->size();
  min_ = domain_->min();
  max_ = domain_->max();
  return size_ > 0;
}

bool BuildObject::removeSet(const MistralSet& s) 
{
  return getBuildObject()->_removeSet( s );
}

bool BuildObject::_removeRange(const int l, const int u) 
{
  if( l <= min_ )
    setMin( u+1 );
  else if( u >= max_ )
    setMax( l-1 );
  else {
    if( domain_ == NULL ) 
      domain_ = new MistralSet( min_, max_, MistralSet::full );
    domain_->removeInterval( l, u );
    size_ = domain_->size();
  }
  return size_ > 0;
}

bool BuildObject::removeRange(const int l, const int u) 
{
  return getBuildObject()->_removeRange(l,u);
}

bool BuildObject::_contain(const int v) const 
{ 
  return (v >= min_ &&
	  v <= max_ &&
	  ( !domain_ || domain_->member(v) ));
}

bool BuildObject::contain(const int v) const
{
  return getConstBuildObject()->_contain( v );
}

int BuildObject::_getNext(const int v) const 
{ 
  if( domain_ != NULL )
    return ( domain_->next(v) ); 
  else return ( v < max_ ? v+1 : INT_MAX );
}

int BuildObject::getNext(const int v) const
{
  return getConstBuildObject()->_getNext( v );
}

bool BuildObject::_good(const int v) const 
{ 
  return ( v != INT_MAX ); 
}

bool BuildObject::good(const int v) const
{
  return getConstBuildObject()->_good(v);
}

bool BuildObject::_setNext(int& v) const 
{ 
  if( domain_ != NULL )
    return ( (v = domain_->next(v)) != INT_MAX ); 
  else return ( ++v <= max_ );
}

bool BuildObject::setNext(int& v) const
{
  return getConstBuildObject()->_setNext(v);
}

bool BuildObject::_isRange() const
{
  return (domain_ == NULL);
}

bool BuildObject::_isGround() const
{
  return (size_ <= 1);
}

bool BuildObject::isRange() const
{
  return getConstBuildObject()->_isRange();
}

bool BuildObject::isGround() const
{
  return getConstBuildObject()->_isGround();
}

int BuildObject::getVarId() const { return getConstBuildObject()->id; } 
int BuildObject::getState() { return state; }
void BuildObject::setState( int nstate ) { state = nstate; }

bool BuildObject::isConvex() const { return (state & CONVEX); }
void BuildObject::setConvex() { if( !(state & CONVEX) ) state |= CONVEX; }
void BuildObject::unsetConvex() { if( state & CONVEX ) state &= (0xffffffff ^ CONVEX); }

bool BuildObject::isCRange() const { return (state & CRANGE); }
void BuildObject::setCRange() { if( !(state & CRANGE) ) state |= CRANGE; }
void BuildObject::unsetCRange() { if(state & CRANGE) state &= (0xffffffff ^ CRANGE); }

bool BuildObject::isVRange() const { return (state & VRANGE); }
void BuildObject::setVRange() { if( !(state & VRANGE) ) state |= VRANGE; }
void BuildObject::unsetVRange() { if(state & VRANGE) state &= (0xffffffff ^ VRANGE); }

bool BuildObject::isNot() const { return (state & NOT); }
void BuildObject::setNot() { if( !(state & NOT) ) state |= NOT; }
void BuildObject::unsetNot() { if( state & NOT ) state &= (0xffffffff ^ NOT); }

bool BuildObject::isNeg() const { return (state & NEG); }
void BuildObject::setNeg() { if( !(state & NEG) ) state |= NEG; }
void BuildObject::unsetNeg() { if( state & NEG ) state &= (0xffffffff ^ NEG); }

bool BuildObject::isVar() const { return (state & VAR); }
void BuildObject::setVar() { if( !(state & VAR) ) state |= VAR; }
void BuildObject::unsetVar() { if( (state & VAR) && !(state & AWV) ) state &= (0xffffffff ^ VAR); }

bool BuildObject::isAwv() const { return (state & AWV); }
void BuildObject::setAwv() { if( !(state & AWV) ) state |= (AWV | VAR); }
void BuildObject::unsetAwv() { if( state & AWV ) state &= (0xffffffff ^ AWV); }

bool BuildObject::isConst() const { return (state & CONST); }
void BuildObject::setConst() { if( !(state & CONST) ) state |= CONST; }
void BuildObject::unsetConst() { if( state & CONST ) state &= (0xffffffff ^ CONST); }

bool BuildObject::isVList() const { return (state & VLIST); }
void BuildObject::setVList() { if( !(state & VLIST) ) state |= VLIST; }
void BuildObject::unsetVList() { if( state & VLIST ) state &= (0xffffffff ^ VLIST); }

bool BuildObject::isCList() const { return (state & CLIST); }
void BuildObject::setCList() { if( !(state & CLIST) ) state |= CLIST; }
void BuildObject::unsetCList() { if( state & CLIST ) state &= (0xffffffff ^ CLIST); }

bool BuildObject::isRel() const { return (state & REL); }
void BuildObject::setRel() { if( !(state & REL) ) state |= REL; }
void BuildObject::unsetRel() { if( state & REL ) state &= (0xffffffff ^ REL); }

bool BuildObject::isPar() const { return (state & PAR); }
void BuildObject::setPar() { if( !(state & PAR) ) state |= PAR; }
void BuildObject::unsetPar() { if( state & PAR ) state &= (0xffffffff ^ PAR); }

bool BuildObject::isNeq() const { return (state & NEQ); }
void BuildObject::setNeq() { if( !(state & NEQ) ) state |= NEQ; }
void BuildObject::unsetNeq() { if( state & NEQ ) state &= (0xffffffff ^ NEQ); }

bool BuildObject::isFixed() const { return (state & FIXED); }
void BuildObject::setFixed() { if( !(state & FIXED) ) state |= FIXED; }
void BuildObject::unsetFixed() { if( state & FIXED ) state &= (0xffffffff ^ FIXED); }

inline void BuildObject::deReference() { --refs; }
inline int BuildObject::isReferenced() { return refs; }
bool BuildObject::preBuild() { return true; }


bool BuildObject::notIn()
{
  return (id < 0);
}

bool BuildObject::isBoolean()
{
  return (size() == 2 && min() == 0 && max() == 1);
}

bool BuildObject::isFDomain()
{
  return (size() > 1 && !isBoolean() && !isRange());
}

bool BuildObject::isRange()
{
  return (size() == (max() - min() + 1) && isVRange());
}

bool BuildObject::isSearchable()
{
  return (size_ > 1 && veqptr_ == NULL);
}

bool BuildObject::isConstraint()
{
  return false;
}

VariableInt* BuildObject::_getVariable()
{
  return varptr_;
}

VariableInt* BuildObject::getVariable()
{
  return getBuildObject()->_getVariable();
}


void BuildObject::printshort(std::ostream& o) const   
{
  if( veqptr_ ) {
    o << "*";
    veqptr_->BuildObject::printshort( o );
  } else if( size_ == 1 ) {
    o << "c" << getVarId() << "(" << min_ << ")";
  } else {
    o << "x" << getVarId();
  } 
}

void BuildObject::print(std::ostream& o) const 
{
  if( varptr_ )
    varptr_->print(o);
  else {
    BuildObject::printshort( o );
    const BuildObject *self = getConstBuildObject();
    if( self->domain_ != NULL ) {
      o << " in " ;
      self->domain_->print( o );
    } else if( self->size_ != 1 ) {
      o << " in {" << self->min_ << ",.." << self->max_ << "}";
    } 
  }
}

string BuildObject::xmlPred( int& idx, int level, int *sub ) const 
{
  string s = ("X" + int2string(idx++)); 
  return s;
}

string BuildObject::xmlCon( MistralSet& scope, int *index ) const 
{
  int i=getVarId();
  if( size() > 1 ) scope.insert( index[i] );
  string s = (size() > 1 ? ("V" + int2string(index[i])) : (int2string(min_)) ); 
  return s;
}

string BuildObject::xmlDom( ) const 
{
  
  string s;
  if( domain_ != NULL ) {
    if( getConstBuildObject()->varptr_ )
      s += ((VariableDomain*)(getConstBuildObject()->varptr_))->values.toString();
    else
      s += getConstBuildObject()->domain_->toString();
  } else if( size() > 2 ) {
    s += ((int2string(min())) + ".." + (int2string(max()))); 
  } else if( size() == 2 ) {
    s += ((int2string(min())) + " " + (int2string(max())));
  } else {
    s += (int2string(min()));
  }
  
  return s;
}

void BuildObject::printstate( std::ostream& o ) const 
{
  o << "\t" << ((state & VRANGE) ? ( state & CONVEX ? " [] " : " <> ") : " }{ " )
    << (isNot() ? " (yes/no) " : "." )
    << (isNeg() ? " (+/-) " : "." )
    << (isConst() ? " const " : "." )
    << (isVList() ? " list " : " bit " );
  if( isVar() && isRel() ) o << " pred";
  else if( isVar() ) o << " var";
  else if( isRel() ) o << " cons";
  o << ( isPar() ? " par " : " !par" ); 
}

BuildObject::~BuildObject() 
{
  if( domain_ ) {
    domain_->neg_words = 0;
    domain_->pos_words = 0;
    domain_->table = NULL;
    delete domain_;
  }
}


bool BuildObject::operator==( BuildObject& x ) const
{
  return (getConstBuildObject() == x.getConstBuildObject());
}

/**********************************************
 * Constraint BuildObject
 **********************************************/ 

BuildObjectConstraint::BuildObjectConstraint() {}

BuildObjectConstraint::~BuildObjectConstraint() {}

int BuildObjectConstraint::computeValue( BuildObjectPredicate *pred ) const 
{ 
  return -1; 
}

void BuildObjectConstraint::add( CSP *m, const int l, 
				 BuildObjectPredicate* pred ) 
{
  int i, n=pred->arity;
  for(i=0; i<n; ++i) {
    pred->scope[i]->add( m, l+1, pred->getState() );
  }
}

bool BuildObjectConstraint::preBuild( BuildObjectPredicate *pred )
{  
  BuildObject **x = pred->scope;
  bool allconstant = true;
  int n=pred->arity, i=n;
  while( allconstant && i-- )
    allconstant = (x[i]->size() == 1);  
  if( allconstant )
    {
      pred->unsetRel();
      i=n;
      while( i-- ) {
	x[i]->deReference();
      }
    }
  return !allconstant;
}


void BuildObjectConstraint::print(std::ostream&, const BuildObjectPredicate*) const {}
string BuildObjectConstraint::xmlPred(int&, int, const BuildObjectPredicate*, int*) const
{
  string s; 
  return s;
}


/**********************************************
 * Predicate BuildObject
 **********************************************/
BuildObjectPredicate::BuildObjectPredicate(BuildObject **x, const int n, 
					   const int lb, const int ub, 
					   BuildObjectConstraint *r, int *p, 
					   int istate)
  : BuildObject(lb, ub)
{
  arity = n;
  params = p;
  scope = x;
  scope[n] = this;
  relation = r;
  setRel();

  // should never been a top level constraint
  if( istate & AWV ) setAwv();

  // preserve ranges
  if( istate & CRANGE ) setCRange();
  else unsetCRange();

  // reasons only about bounds
  if( istate & CONVEX ) setConvex();
  else unsetConvex();

  if( istate & CLIST ) setCList();
  else unsetCList();
}

BuildObjectPredicate::~BuildObjectPredicate() 
{
  delete [] scope;
  delete [] params;
}

void BuildObjectPredicate::add( CSP *m, const int l, const int inf )
{
  for(int i=0; i<arity; ++i)
    scope[i]->parent.push( this );
  relation->add( m, l, this );
  BuildObject::add( m, l, inf );
}

bool BuildObjectPredicate::preBuild( ) 
{
  bool res = true;
  if( !isFixed() ) {
    res = relation->preBuild( this );
    if( !res ) setFixed();
  }
  return res;
}


void BuildObjectPredicate::build(Solver *s) 
{
  if( isVar() ) {
    BuildObject::build( s );
  }

  if( isRel() ) {
    int n=arity+isVar();
    VariableInt *tmpvars[n];
    for(int i=0; i<n; ++i) { 
      tmpvars[i] = scope[i]->getVariable();
    }
    relation->build( s, tmpvars, this );
  }
}

int BuildObjectPredicate::nestedArity() const
{
  return arity+isVar();
}
  
void BuildObjectPredicate::unsetCRange() { 
  if(state & CRANGE) {
    state &= (0xffffffff ^ CRANGE); 
    int i = arity+isVar();
    while( i-- ) 
      scope[i]->unsetVRange();
  }
}

void BuildObjectPredicate::unsetCList() { 
  if( state & CLIST ) {
    state &= (0xffffffff ^ CLIST); 
    int n=arity+isVar();
    for(int i=0; i<n; ++i)
      scope[i]->unsetVList();
  }
}

void BuildObjectPredicate::setFixed() 
{ 
  if( !(state & FIXED) ) {
    state |= FIXED; 
    setDomain( relation->computeValue( this ) );
  }
}

bool BuildObjectPredicate::isConstraint()
{
  // BUGGY
  return ( !isVar() || scope[arity]->size() == 1 || !isPar() );
}

string BuildObjectPredicate::xmlCon( MistralSet& scp, int *index ) const
{
  string s;
  if( arity > 3 )
    s = BuildObject::xmlCon( scp, index );
  else {
    int n=arity-1;
    for(int i=0; i<n; ++i)
      s += (scope[i]->xmlCon( scp, index ) + " ");
    s += scope[n]->xmlCon( scp, index );
  }
  return s;
}

void BuildObjectPredicate::printshort(std::ostream& o) const
{
  if( isRel() )
    relation->print( o, this );
  else if( isVar() )
    BuildObject::printshort( o );
}
 
void BuildObjectPredicate::print(std::ostream& o) const
{

  if( getConstBuildObject()->varptr_ ) {
    getConstBuildObject()->varptr_->print( o );    
    o << " <-> "; 
  } else {
    if( isVar() ) {
      BuildObject::print( o );
      o << " <-> "; 
    }
  }
  if( isRel() ) {
    o << "(" << arity << ") "; 
    relation->print( o, this );
  }
}

string BuildObjectPredicate::xmlPred( int& idx, int level, int *sub ) const
{
  return relation->xmlPred( idx, level, this, sub );
}


/**********************************************
 * Variable Wrapper
 **********************************************/

Variable::Variable() { var_ptr_=NULL; }

Variable::~Variable() {}

int Variable::value()
{
  int v = INT_MAX;
  if( var_ptr_->getVariable() != NULL ) {
    v = var_ptr_->getVariable()->value();
  } 
  return v;
}

int Variable::size()
{
  int s = INT_MAX;
  if( var_ptr_->getVariable() != NULL )
    s = var_ptr_->getVariable()->domsize();
  else s = var_ptr_->size();
  return s;
}

int Variable::min()
{
  int m = var_ptr_->min();
  if( var_ptr_->getVariable() != NULL )
    m = var_ptr_->getVariable()->min();
  else m = var_ptr_->min();
  return m;
}

int Variable::max()
{
  int m = var_ptr_->max();
  if( var_ptr_->getVariable() != NULL )
    m = var_ptr_->getVariable()->max();
  else m = var_ptr_->max();
  return m;
}

Variable& Variable::operator= ( const Variable& x )
{
  var_ptr_ = x.var_ptr_;
  return *this;
}

VarArray Variable::operator,( Variable x )
{
  VarArray table(2);
  table[0] = *this;
  table[1] = x;
  return table;
}

Variable::Variable( const int nvals ) 
{
  var_ptr_ = CSP::_Variable( nvals );
  //  var_ptr_ = new BuildObject( 0, nvals-1 );
  //  var_ptr_->setVar();
}

Variable::Variable( const Variable& v )
{
  var_ptr_ = v.var_ptr_;
}

Variable::Variable( BuildObject *v )
{
  var_ptr_ = v;
}

Variable::Variable( const int lb, const int ub ) 
{
  var_ptr_ = CSP::_Variable( lb, ub );
  //var_ptr_ = new BuildObject(lo, up);
  //var_ptr_->setVar();
}

Variable::Variable( const int* array, const int length ) 
{
  var_ptr_ = CSP::_Variable( array, length );
  //var_ptr_ = new BuildObject(array, length);
  //var_ptr_->setVar();
}

Variable::Variable( const int* array, const int length, 
		    const int lb, const int ub ) 
{
  var_ptr_ = CSP::_Variable( array, length, lb, ub );
  //var_ptr_ = new BuildObject(array, length, lo, up);
  //var_ptr_->setVar();
}

VariableInt *Variable::getVariable() 
{
  return var_ptr_->getVariable();
}

void Variable::printshort(std::ostream& o) const 
{
  if( var_ptr_->getVariable() != NULL )
    var_ptr_->getVariable()->printshort( o );
  else var_ptr_->printshort( o );
}

void Variable::print(std::ostream& o) const 
{
  if( var_ptr_->getVariable() != NULL )
    var_ptr_->getVariable()->print( o );
  else var_ptr_->print( o );
}


/**********************************************
 * VarArray Wrapper
 **********************************************/

VarArray::VarArray( ) {}

VarArray::VarArray( const int nvars )
{
  array_.resize( nvars );
}

VarArray::VarArray( const int nvars, const int nvals )
{
#ifdef _NARRAY
  array_.resize( nvars );
  int i=nvars;
  while( i-- ) {
    array_[i] = CSP::_Variable( nvals );
  }
#else
  array_.resize( nvars );
  tmp_.resize( nvars );
  int i=nvars;
  while( i-- ) {
//     tmp_[i] = CSP::_Variable( nvals );
//     array_[i] = Variable( tmp_[i] );
    array_[i] = Variable( nvals );
  }
#endif
}

VarArray::VarArray( const int nvars, const int lo, const int up ) 
{
#ifdef _NARRAY
  array_.resize( nvars );
  int i=nvars;
  while( i-- ) {
    array_[i] = CSP::_Variable( lo, up );
  }
#else
  array_.resize( nvars );
  int i=nvars;
  while( i-- ) {
//     tmp_[i] = CSP::_Variable( lo, up );
//     array_[i] = Variable( tmp_[i] );
    array_[i] = Variable( lo, up );
  }
#endif
}

VarArray::VarArray( const int nvars, const int* array, const int length ) 
{
#ifdef _NARRAY
  array_.resize( nvars );
  int i=nvars;
  while( i-- ) {
    array_[i] = CSP::_Variable( array, length );
  }
#else
  array_.resize( nvars );
  int i=nvars;
  while( i-- ) {
//     tmp_[i] = CSP::_Variable( array, length );
//     array_[i] = Variable( tmp_[i] );
    array_[i] = Variable( array, length );
  }
#endif
}

VarArray::VarArray( const int nvars, const int* array, const int length, const int lo, const int up )
{
#ifdef _NARRAY
  array_.resize( nvars );
  int i=nvars;
  while( i-- ) {
    array_[i] = CSP::_Variable( array, length, lo, up );
  }
#else
  array_.resize( nvars );
  int i=nvars;
  while( i-- ) {
//     tmp_[i] = CSP::_Variable( array, length, lo, up );
//     array_[i] = Variable( tmp_[i] );
    array_[i] = Variable( array, length, lo, up );
  }
#endif
}

VarArray::~VarArray() {}

BuildObject **VarArray::getArgs() 
{
#ifdef _NARRAY
  return array_.stack_;
#else
  int n = array_.size();
  tmp_.clear();
  tmp_.resize( n );
  for( int i=0; i<n; ++i ) 
    tmp_[i] = array_[i].var_ptr_;
    //tmp_.push( array_[i].var_ptr_ );
  return tmp_.stack_;
//   int n = array_.size();
//   BuildObject **tmp = new BuildObject*[n+1];
//   while( n-- ) tmp[n] = array_[n].var_ptr_;  
//   return tmp;
#endif
}

void VarArray::add( const int k )
{
#ifdef _NARRAY
  array_.push_back( CSP::_Variable(k, k) );
#else
//   tmp_.push_back( CSP::_Variable(k, k) );
//   array_.push_back( Variable( tmp_.back() ) );
  array_.push_back( Variable(k, k) );
#endif
}

void VarArray::add( Variable x )
{
#ifdef _NARRAY
  array_.push_back( x.var_ptr_ );
#else
  //tmp_.push_back( x.var_ptr_ );
  array_.push_back( x );
#endif
}

void VarArray::add( VarArray& a )
{
#ifdef _NARRAY
  int i=0, n=a.size();
  for(i=0; i<n; ++i)
    add( a.array_[i] );
#else
  int i=0, n=a.size();
  for(i=0; i<n; ++i) {
    //tmp_.push_back( a.tmp_[i] );
    add( a[i] );
  }
#endif
}

void VarArray::build( Solver *s ) 
{
#ifdef _NARRAY
  int n=array_.size();
  for(int i=0; i<n; ++i)
    array_[i]->build( s );
#else
  int n=array_.size();
  for(int i=0; i<n; ++i)
    array_[i].var_ptr_->build( s );
  //tmp_[i]->build( s );
#endif
}  

void VarArray::resize( const int n )
{
#ifdef _NARRAY
  array_.resize( n );
#else
  array_.resize( n );
  tmp_.resize( n );
#endif
}

void VarArray::clear()
{
#ifdef _NARRAY
  array_.clear();
#else
  array_.clear();
  //tmp_.clear();
#endif
}

int VarArray::size() const
{
  return array_.size();
}


/*!@name Operators*/
Variable& VarArray::operator[]( const int i ) 
{
#ifdef _NARRAY
  buffer.var_ptr_ = array_[i];
  return buffer; 
#else
  return array_[i];
#endif
}

VarArray VarArray::operator,( Variable x )
{
#ifdef _NARRAY
  int i=size();
  VarArray table(i+1);
  table.array_[i] = x.var_ptr_;
  while( i-- ) 
    table.array_[i] = array_[i];
  return table;
#else
  int i=size();
  VarArray table(i+1);
  table[i] = x;
  //table.tmp_[i] = x.var_ptr_;
  while( i-- ) {
    table[i] = array_[i];
    //table.tmp_[i] = tmp_[i];
  }
  return table;
#endif
}

VarArray VarArray::operator,( VarArray& x )
{
#ifdef _NARRAY
  int i=size(), j=x.size();
  VarArray table(i+j);
  while( j-- )
    table.array_[i+j] = x.array_[j];
  while( i-- )
    table.array_[i] = array_[i];
  return table;
#else
  int i=size(), j=x.size();
  VarArray table(i+j);
  while( j-- ) {
    //table.tmp_[i+j] = tmp_[i+j];
    table[i+j] = x.array_[j];
  }
  while( i-- ) {
    //table.tmp_[i] = tmp_[i];
    table[i] = array_[i];
  }
  return table;
#endif
}

VarArray& VarArray::operator=( VarArray& x )
{
#ifdef _NARRAY
  array_ = x.array_;
#else
  array_ = x.array_;
  //tmp_ = x.tmp_;
#endif

  return *this;
}

void VarArray::printshort(std::ostream& o) const 
{
  int i=0, n=size()-1;
  if( n >= 0 ) {
    o << "[";
    o.flush();
    for(i=0; i<n; ++i)
      {
#ifdef _NARRAY
	array_[i]->printshort( o );
#else
	array_[i].printshort( o );
#endif
	o << ", ";
	o.flush();
      }
#ifdef _NARRAY
	array_[n]->printshort( o );
#else
	array_[n].printshort( o );
#endif
    o << "]";
    o.flush();
  }
}
  
void VarArray::print(std::ostream& o) const 
{
  int i=0, n=size()-1;
  if( n >= 0 ) {
    o << "[";
    o.flush();
    for(i=0; i<n; ++i)
      {
#ifdef _NARRAY
	array_[i]->print( o );
#else
	array_[i].print( o );
#endif
	o << ", ";
	o.flush();
      }
#ifdef _NARRAY
	array_[n]->print( o );
#else
	array_[n].print( o );
#endif
    o << "]";
    o.flush();
  }
}


/**********************************************
 * Matrix Wrapper
 **********************************************/

Matrix::Matrix( const int nr, const int nc ) 
{
  nCols=nc;
  nRows=nr;
  array_.resize( nr*nc );
  tmp_.resize( std::max(nr, nc) );
}

Matrix::Matrix( const int nr, const int nc, const int nv ) 
{
  nCols=nc;
  nRows=nr;
  array_.resize( nr*nc );
  tmp_.resize( std::max(nr, nc) );
  int i=nc*nr;
  while( i-- ) 
    array_[i] = CSP::_Variable( nv );
}

Matrix::Matrix( const int nr, const int nc, const int lo, const int up ) 
{
  nCols=nc;
  nRows=nr;
  array_.resize( nr*nc );
  tmp_.resize( std::max(nr, nc) );
  int i=nc*nr;
  while( i-- ) 
    array_[i] = CSP::_Variable( lo, up );
}

Matrix::Matrix( const int nr, const int nc, const int* matrix, const int length ) 
{
  nCols=nc;
  nRows=nr;
  array_.resize( nr*nc );
  tmp_.resize( std::max(nr, nc) );
  int i=nc*nr;
  while( i-- ) 
    array_[i] = CSP::_Variable( matrix, length );
}

Matrix::Matrix( const int nr, const int nc, const int* matrix, const int length, const int lo, const int up ) 
{
  nCols=nc;
  nRows=nr;
  array_.resize( nr*nc );
  tmp_.resize( std::max(nr, nc) );
  int i=nc*nr;
  while( i-- ) 
    array_[i] = CSP::_Variable( matrix, length, lo, up );
}


Matrix::~Matrix() {}

Variable& Matrix::cell( const int i, const int j ) 
{
  tmp_[0] = Variable( array_[i*nCols+j] );
  return tmp_[0];
}

VarArray& Matrix::row( const int i ) 
{
  tmp_.resize( nCols );
  for( int j=0; j<nCols; ++j )
    tmp_[j] = Variable( array_[i*nCols+j] );
  return tmp_;
}
VarArray& Matrix::column( const int j ) 
{
  tmp_.resize( nRows );
  for( int i=0; i<nRows; ++i )
    tmp_[i] = Variable( array_[i*nCols+j] );
  return tmp_;
}

void Matrix::printshort(std::ostream& o) const {}
void Matrix::print(std::ostream& o) const 
{
  for( int i=0; i<nRows; ++i ) {
    for( int j=0; j<nCols; ++j ) {
      array_[i*nCols+j]->print( cout );
      cout << " ";
    }
    cout << endl;
  }
}

/**********************************************
 *  Offset Predicate BuildObject
 **********************************************/ 
void BuildObjectOffset::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isRel() ) {
    Constraint *con = new PredicateOffset( s, tmp, pred->params[0] );
  }
}

bool BuildObjectOffset::preBuild( BuildObjectPredicate *pred )
{
  int res = BuildObjectConstraint::preBuild( pred );
  if( res ) {
    BuildObject **scope=pred->scope;
    if( !pred->params[0] ) {
      pred->unsetRel();
      int i = ( scope[0]->getVarId() < scope[1]->getVarId() );
      scope[1-i]->setEqual( scope[i] );
    }
  }
  return res;
}

int BuildObjectOffset::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() + pred->params[0]);
}

void BuildObjectOffset::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  if( pred->params[0] < 0 )
    o << " - " << (-1 * pred->params[0]);
  else 
    o << " + " << pred->params[0];
  o << ")";
}

string BuildObjectOffset::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "add(" + x1 + "," + int2string(pred->params[0]) + ")" );
  return s;
}
  

/**********************************************
 *  Factor Predicate BuildObject
 **********************************************/ 
void BuildObjectFactor::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isRel() )
    Constraint *con = new PredicateFactor( s, tmp, pred->params[0] );
}

void BuildObjectFactor::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  o << pred->params[0] << " * ";
  pred->scope[0]->printshort( o );
  o << ")";
}

bool BuildObjectFactor::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );
  if( res ) {
    BuildObject **scope=pred->scope;
    if( pred->params[0] == 1 ) {
      pred->unsetRel();
      int i = ( scope[0]->getVarId() < scope[1]->getVarId() );
      scope[1-i]->setEqual( scope[i] );
    }
  }
  return res;
}

int BuildObjectFactor::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() * pred->params[0]);
}

string BuildObjectFactor::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "mul(" + int2string(pred->params[0]) + "," +  x1 + ")" );
  return s;
}


/**********************************************
 *  Negation Predicate BuildObject
 **********************************************/ 
void BuildObjectNegation::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred )
{
  Constraint *con = new PredicateNegation( s, tmp );
} 

bool BuildObjectNegation::preBuild( BuildObjectPredicate *pred )
{
  return true;
}

int BuildObjectNegation::computeValue( BuildObjectPredicate *pred ) const
{
  return (-pred->scope[0]->value());
}

void BuildObjectNegation::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "-";
  pred->scope[0]->printshort( o );
}

string BuildObjectNegation::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "neg(" + x1 + ")" );
  return s;
}


/**********************************************
 *  Not Predicate BuildObject
 **********************************************/ 
void BuildObjectNot::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() )
    Constraint *con = new PredicateNot( s, tmp );
}

bool BuildObjectNot::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred ); 
  if( res ) {
    int pmax = !pred->max();
    res = ( !pmax && pred->min() != 1 && pred->isVar() );
    if( !res ) {
      pred->unsetRel();
      BuildObject *x=pred->scope[0];
      if( pmax ) {
	x->remove( 0 );
      }
      else {
	x->setDomain( 0 );
      }
    } 
  }
  return res;
}

int BuildObjectNot::computeValue( BuildObjectPredicate *pred ) const
{
  return (!pred->scope[0]->value());
}

void BuildObjectNot::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "!";
  pred->scope[0]->printshort( o );
}

string BuildObjectNot::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "not(" + x1 + ")" );
  return s;
}


/**********************************************
 *  LowerBound Predicate BuildObject
 **********************************************/ 
void BuildObjectLowerBound::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() )
    Constraint *con = new PredicateLowerBound( s, tmp, pred->params[0] );
}

bool BuildObjectLowerBound::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );
  if( res ) {
    int pmax = !pred->max();
    if( pmax || pred->min() )
      pred->unsetVar();
    if( !pred->isVar() ) {
      pred->unsetRel();
      if( pmax ) pred->scope[0]->setMax( pred->params[0]-1 );
      else pred->scope[0]->setMin( pred->params[0] );
      res = false;
    }
  }
  return res;
}

int BuildObjectLowerBound::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() >= pred->params[0]);
}

void BuildObjectLowerBound::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " >= " << (pred->params[0]) << ")";
}

string BuildObjectLowerBound::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "ge(" + x1 + "," + int2string(pred->params[0]) + ")" );
  return s;
}


/**********************************************
 *  UpperBound Predicate BuildObject
 **********************************************/ 
void BuildObjectUpperBound::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() )
    Constraint *con = new PredicateUpperBound( s, tmp, pred->params[0] );
}

bool BuildObjectUpperBound::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );
  if( res ) {
    int pmax = !pred->max();
    if( pmax || pred->min() )
      pred->unsetVar();
    if( !pred->isVar() ) {
      pred->unsetRel();
      if( pmax ) pred->scope[0]->setMin( pred->params[0]+1 );
      else pred->scope[0]->setMax( pred->params[0] );
      res = false;
    }
  }
  return res;
}

int BuildObjectUpperBound::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() <= pred->params[0]);
}

void BuildObjectUpperBound::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " <= " << (pred->params[0]) << ")";
}

string BuildObjectUpperBound::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "le(" + x1 + "," + int2string(pred->params[0]) + ")" );
  return s;
}


/**********************************************
 *  Abs Predicate BuildObject
 **********************************************/ 
void BuildObjectAbs::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateAbs( s, tmp );
}

int BuildObjectAbs::computeValue( BuildObjectPredicate *pred ) const
{
  return (abs(pred->scope[0]->value()));
}

void BuildObjectAbs::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "|";
  pred->scope[0]->printshort( o );
  o << "|";
}

string BuildObjectAbs::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "abs(" + x1 + ")" );
  return s;
}

/**********************************************
 * And Constraint BuildObject
 **********************************************/
void BuildObjectAnd::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() )
    Constraint *con = new PredicateAnd( s, tmp );
}

bool BuildObjectAnd::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );
  if( res ) {
    if( pred->min() == 1 || ( pred->max() && !pred->isVar() ) ) {
      pred->unsetRel();
      for( int i=0; i<2; ++i ) {
	pred->scope[i]->deReference();
	pred->scope[i]->remove( 0 );
      }
      res = false;
    }
  }
  return res;
}

int BuildObjectAnd::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() && pred->scope[1]->value());
}

void BuildObjectAnd::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " & ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectAnd::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "and(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 * Or Constraint BuildObject
 **********************************************/
void BuildObjectOr::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() ) {
    Constraint *con = new PredicateOr( s, tmp );
  } else {
    Constraint *con = new ConstraintOr( s, tmp );
  }
}

bool BuildObjectOr::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );
  int i;
  if( res && !pred->max() ) {
    pred->unsetRel();
    for( i=0; i<2; ++i ) {
      pred->scope[i]->deReference();
      pred->scope[i]->setDomain( 0 );
    }
    res = false;
  } else {
    for( i=0; i<2; ++i ) 
      if( !pred->scope[i]->max() ) {
	pred->unsetRel();
	pred->scope[0]->deReference();
	pred->scope[1]->deReference();
	pred->scope[1-i]->remove( 0 );
	pred->scope[1-i]->preBuild();
	res = false;
      }
  } 
  return res;
}

int BuildObjectOr::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() || pred->scope[1]->value());
}

void BuildObjectOr::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " || ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectOr::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "or(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 * IfThenElse Constraint BuildObject
 **********************************************/
void BuildObjectIfThenElse::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() )
    Constraint *con = new PredicateIfThenElse( s, tmp );
}

bool BuildObjectIfThenElse::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );
  return res;
}

int BuildObjectIfThenElse::computeValue( BuildObjectPredicate *pred ) const
{
  if( pred->scope[0]->value() )
    return pred->scope[1]->value();
  else 
    return pred->scope[2]->value();
}

void BuildObjectIfThenElse::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " ? ";
  pred->scope[1]->printshort( o );
  o << " : ";
  pred->scope[2]->printshort( o );
  o << ")";
}

string BuildObjectIfThenElse::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string x3 = pred->scope[2]->xmlPred(idx, level+1, sub);
  string s = ( "if(" + x1 + "," + x2 + "," + x3 + ")" );
  return s;
}


/**********************************************
 *  Disjunctive Predicate BuildObject
 **********************************************/ 
void BuildObjectDisjunctive::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateDisjunctive( s, tmp, pred->params );
}

int BuildObjectDisjunctive::computeValue( BuildObjectPredicate *pred ) const
{
  return ( pred->scope[0]->value() + pred->params[0] <= pred->scope[1]->value() );
}

void BuildObjectDisjunctive::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " + " << pred->params[0] << " <= ";
  pred->scope[1]->printshort( o );
  o << " || ";
  pred->scope[1]->printshort( o );
  o << " + " << pred->params[1] << " <= ";
  pred->scope[0]->printshort( o );
  o << ")";
}
string BuildObjectDisjunctive::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "or(le(add(" + x1 + "," + int2string(pred->params[0]) + ")," + x2 + ")," +
	       "le(add(" + x2 + "," + int2string(pred->params[1]) + ")," + x1 + "))" );
  return s;
}


/**********************************************
 *  Precedence Predicate BuildObject
 **********************************************/ 
void BuildObjectPrecedence::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( !(pred->isVar()) ) { 
    if( !pred->max() )
      {
	VariableInt *aux = tmp[0];
	tmp[0] = tmp[1];
	tmp[1] = aux;
	pred->params[0] = (1-pred->params[0]);
      }
    Constraint *con = new ConstraintLess( s, tmp, pred->params[0] );
  } else {
    Constraint *con = new PredicateLess( s, tmp, pred->params[0] );
  }
}

bool BuildObjectPrecedence::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );

  if( res ) {
    BuildObject **scope = pred->scope;
    if( scope[0]->size() == 1 )
      {
	pred->relation = ENVIRONMENT[ConstraintStore::LOWERBOUND];
	pred->params[0] += scope[0]->min();
	scope[0]->deReference();
	scope[0] = scope[1];
	scope[1] = scope[2];
	--pred->arity;
	res = pred->relation->preBuild( pred );
      }
    else if( scope[1]->size() == 1 )
      {
	pred->relation = ENVIRONMENT[ConstraintStore::UPPERBOUND];
	pred->params[0] = (scope[1]->min() - pred->params[0]);
	scope[1]->deReference();
	scope[1] = scope[2];
	--pred->arity;
	res = pred->relation->preBuild( pred );
      }
    else if( !pred->max() || pred->min() ) {
      pred->unsetVar();
    }
    if( !pred->isVar() && pred->params[0] > 0 )
      {
	pred->setNeq();
      }
  }
  return res;
}

int BuildObjectPrecedence::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() + pred->params[0] <= pred->scope[1]->value());
}

void BuildObjectPrecedence::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  if( pred->params[0] == 1 )
    o << " < ";
  else {
    if( pred->params[0] )
      o << " + " << pred->params[0] ;
    o << " <= ";
  }
  pred->scope[1]->printshort( o );
  o << ")";
}    

string BuildObjectPrecedence::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s;
  if(pred->params[0] == 0 )
    s = ( "le(" + x1 + "," + x2 + ")" );
  else if(pred->params[0] == 1)
    s = ( "lt(" + x1 + "," + x2 + ")" );
  else
    s = ( "le(add(" + x1 + "," + int2string(pred->params[0]) + ")," + x2 + ")" );
  return s;
}


/**********************************************
 *  Add Predicate BuildObject
 **********************************************/
void BuildObjectAdd::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateAdd( s, tmp );
}

bool BuildObjectAdd::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );

  if( res ) {
    BuildObject **scope = pred->scope;
    if( scope[0]->size() == 1 ) {   
      pred->relation = ENVIRONMENT[ConstraintStore::OFFSET];
      pred->params = new int[1];
      pred->params[0] = scope[0]->min();
      scope[0]->deReference();
      scope[0] = scope[1];
      scope[1] = scope[2];
      --pred->arity;
      res = pred->preBuild( );
    } else if( scope[1]->size() == 1 ) {
      pred->relation = ENVIRONMENT[ConstraintStore::OFFSET];
      pred->params = new int[1];
      pred->params[0] = scope[1]->min();
      scope[1]->deReference();
      scope[1] = scope[2];
      --pred->arity;
      res = pred->relation->preBuild( pred );
    }
  }

  return res;
}

int BuildObjectAdd::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() + pred->scope[1]->value());
}

void BuildObjectAdd::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " + ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectAdd::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "add(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  Sub Predicate BuildObject
 **********************************************/
void BuildObjectSub::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateSub( s, tmp );
}

bool BuildObjectSub::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );

  if( res ) {
    BuildObject **scope = pred->scope;
    if( scope[1]->size() == 1 ) {
      pred->relation = ENVIRONMENT[ConstraintStore::OFFSET];
      pred->params = new int[1];
      pred->params[0] = -scope[1]->min();
      scope[1]->deReference();
      scope[1] = scope[2];
      --pred->arity;
      res = pred->relation->preBuild( pred );
    }
  }
  return res;
}

int BuildObjectSub::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() - pred->scope[1]->value());
}

void BuildObjectSub::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " - ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectSub::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "sub(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  Mul Predicate BuildObject
 **********************************************/
void BuildObjectMul::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateMul( s, tmp );
}

int BuildObjectMul::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() * pred->scope[1]->value());
}

void BuildObjectMul::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " * ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectMul::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "mul(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  Div Predicate BuildObject
 **********************************************/
void BuildObjectDiv::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateDiv( s, tmp );
}

int BuildObjectDiv::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() / pred->scope[1]->value());
}

void BuildObjectDiv::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " / ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectDiv::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "div(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  Pow Predicate BuildObject
 **********************************************/
void BuildObjectPow::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
}

int BuildObjectPow::computeValue( BuildObjectPredicate *pred ) const
{
  return (int)(pow((double)(pred->scope[0]->value()), (double)(pred->scope[1]->value())));
}

void BuildObjectPow::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " ^ ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectPow::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "pow(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  Mod Predicate BuildObject
 **********************************************/
void BuildObjectMod::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateMod( s, tmp );
}

int BuildObjectMod::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() % pred->scope[1]->value());
}

void BuildObjectMod::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " % ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectMod::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "mod(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  ModConstant Predicate BuildObject
 **********************************************/
void BuildObjectModConstant::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateModConstant( s, tmp, pred->params[0] );
}

int BuildObjectModConstant::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() % pred->params[0]);
}

void BuildObjectModConstant::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " % ";
  pred->params[0];
  o << ")";
}

string BuildObjectModConstant::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "mod(" + x1 + "," + int2string(pred->params[0]) + ")" );
  return s;
}


/**********************************************
 *  EqualConstant Predicate BuildObject
 **********************************************/ 
void BuildObjectEqualConstant::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() )
    Constraint *con = new PredicateEqualConstant( s, tmp, pred->params[0], 1 );
}

bool BuildObjectEqualConstant::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );

  if( res ) {
    int val = pred->params[0];
    BuildObject *x=pred->scope[0];
    int pmax = !pred->max();
    if( pmax || pred->min() )
      pred->unsetVar();
    if( !pred->isVar() ) {
      x->deReference();
      pred->unsetRel();
      if( pmax ) x->remove( val );
      else x->setDomain( val );
      res = false;
    } else if( val > x->min() && val < x->max() ) 
      pred->unsetCRange();
  }
  return res;
}

int BuildObjectEqualConstant::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() == pred->params[0]);
}

void BuildObjectEqualConstant::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " == " << (pred->params[0]) << ")";
}

string BuildObjectEqualConstant::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "eq(" + x1 + "," + int2string(pred->params[0]) + ")" );
  return s;
}


/**********************************************
 *  NotEqualConstant Predicate BuildObject
 **********************************************/ 
void BuildObjectNotEqualConstant::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() ) 
    Constraint *con = new PredicateEqualConstant( s, tmp, pred->params[0], 0 );
}

bool BuildObjectNotEqualConstant::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );

  if( res ) {
    int pmax = !pred->max();
    if( pmax || pred->min() == 1 )
      pred->unsetVar();
    if( !pred->isVar() ) {
      pred->scope[0]->deReference();
      pred->unsetRel();
      if( pmax ) pred->scope[0]->setDomain( pred->params[0] );
      else pred->scope[0]->remove( pred->params[0] );
      res = false;
    } else pred->unsetCList();
  } 
  return res;
}

int BuildObjectNotEqualConstant::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() != pred->params[0]);
}

void BuildObjectNotEqualConstant::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " =/= " << (pred->params[0]) << ")";
}

string BuildObjectNotEqualConstant::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string s = ( "ne(" + x1 + "," + int2string(pred->params[0]) + ")" );
  return s;
}


/**********************************************
 *  Equal Predicate BuildObject
 **********************************************/ 
void BuildObjectEqual::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( pred->isVar() )
    Constraint *con = new PredicateEqual( s, tmp, 1 );
}

bool BuildObjectEqual::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );

  if( res ) {
    BuildObject **scope = pred->scope;
    if( pred->min() == 1 )
      pred->unsetVar();
    if( !pred->isVar() ) {
      pred->unsetRel();
      int i = ( scope[0]->getVarId() < scope[1]->getVarId() );

      scope[1-i]->setEqual( scope[i] );

      res = false;
    } else {
      if( pred->isVar() )
	pred->unsetCRange();
      if( scope[0]->size() == 1 ) {    
	pred->relation = ENVIRONMENT[ConstraintStore::EQCONSTANT];
	pred->params = new int[1];
	pred->params[0] = scope[0]->min();
	scope[0]->deReference();
	scope[0] = scope[1];
	scope[1] = scope[2];
	--pred->arity;
	res = pred->relation->preBuild( pred );
      } else if( scope[1]->size() == 1 ) {
	pred->relation = ENVIRONMENT[ConstraintStore::EQCONSTANT];
	pred->params = new int[1];
	pred->params[0] = scope[1]->min();
	scope[1]->deReference();
	scope[1] = scope[2];
	--pred->arity;
	res = pred->relation->preBuild( pred );
      }
    }
  }
  return res;
}

int BuildObjectEqual::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() == pred->scope[1]->value());
}

void BuildObjectEqual::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " == " ;
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectEqual::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "eq(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  NotEqual Predicate BuildObject
 **********************************************/ 
void BuildObjectNotEqual::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( !(pred->isVar()) ) 
    Constraint *con = new ConstraintNotEqual( s, tmp );
  else 
    Constraint *con = new PredicateEqual( s, tmp, 0 );
}

bool BuildObjectNotEqual::preBuild( BuildObjectPredicate *pred )
{
  bool res = BuildObjectConstraint::preBuild( pred );

  if( res ) {
    BuildObject **scope = pred->scope;
    int i, constant=-1;
    if( pred->min() == 1 )
      pred->unsetVar();
    if( !pred->isVar() ) {
      for(i=0; i<2; ++i)
	if( pred->scope[i]->size() == 1 )
	  constant = i;
      if( constant >= 0 ) {
	pred->scope[1-constant]->remove( pred->scope[constant]->min() );
	pred->unsetRel();
	res = false;
      } else {
	pred->setNeq();
      }
    } else {
      pred->unsetCList();
      if( scope[0]->size() == 1 ) {    
	pred->relation = ENVIRONMENT[ConstraintStore::NEQCONSTANT];
	pred->params = new int[1];
	pred->params[0] = scope[0]->min();
	scope[0]->deReference();
	scope[0] = scope[1];
	scope[1] = scope[2];
	--pred->arity;
	res = pred->relation->preBuild( pred );
      } else if( scope[1]->size() == 1 ) {
	pred->relation = ENVIRONMENT[ConstraintStore::NEQCONSTANT];
	pred->params = new int[1];
	pred->params[0] = scope[1]->min();
	scope[1]->deReference();
	scope[1] = scope[2];
	--pred->arity;
	res = pred->relation->preBuild( pred );
      }
    }
  } 
  return res;
}

int BuildObjectNotEqual::computeValue( BuildObjectPredicate *pred ) const
{
  return (pred->scope[0]->value() != pred->scope[1]->value());
}

void BuildObjectNotEqual::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " =/= ";
  pred->scope[1]->printshort( o );
  o << ")";
}

string BuildObjectNotEqual::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "ne(" + x1 + "," + x2 + ")" );
  return s;
}


/**********************************************
 *  NotEqualWildcard Predicate BuildObject
 **********************************************/ 
void BuildObjectNotEqualWildcard::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  if( !(pred->isVar()) ) {
    Constraint *con = new ConstraintNotEqualWildcard( s, tmp, pred->params[0] );
  } else {
    // TODO
  }
}

int BuildObjectNotEqualWildcard::computeValue( BuildObjectPredicate *pred ) const
{
  int x = pred->scope[0]->value();
  return (x == pred->params[0] || x != pred->scope[1]->value());
}

void BuildObjectNotEqualWildcard::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  o << " =/= ";
  pred->scope[1]->printshort( o );
  o << " or ";
  pred->scope[0]->printshort( o );
  o << " = ";
  o << pred->params[0] ;
  o << ")";
}

string BuildObjectNotEqualWildcard::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
  string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
  string s = ( "or(ne(" + x1 +  "," + x2 + "), eq(" + x1 + "," + int2string(pred->params[0]) + "))" );
  return s;
}


/**********************************************
 * SumEq Constraint BuildObject
 **********************************************/
void BuildObjectSumEq::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con;
  bool isBoolean = true, isWeighted=false;
  int i=pred->arity;
  while( i-- ) {
    isBoolean &= ( tmp[i]->getType() == VariableInt::BOOL );
    isWeighted |= pred->params[i] != 1;
  }

  if( isWeighted ) {
    con = new ConstraintWeightedSumEqual( s, tmp, pred->arity, 
					  pred->params );
  } else if( isBoolean ) {
    con = new ConstraintBoolSumEqual( s, tmp, pred->arity, 
				      pred->params[pred->arity] );
  } else {
    con = new ConstraintSumEqual( s, tmp, pred->arity, 
				  pred->params[pred->arity] );
  }
}

bool BuildObjectSumEq::preBuild( BuildObjectPredicate *pred )
{
  int j, i, n=pred->arity, *params=pred->params;
  BuildObject **scope=pred->scope, *aux;

  i=n;
  // absorb constants
  while( i-- ) {
    if( scope[i]->size() == 1 ) {
      scope[i]->deReference();
      params[n] -= (params[i] * scope[i]->min());
      params[i] = params[n-1];
      scope[i] = scope[n-1];
      params[n-1] = params[n];
      scope[n-1] = scope[n];
      --n;
    } else {
      j=-1;
      while( ++j < i )
	if( *(scope[i]) == *(scope[j]) ) {
	  break;
	}

      if( j < i ) {
	++params[j];
	scope[i]->deReference();
	params[i] = params[n-1];
	scope[i] = scope[n-1];
	params[n-1] = params[n];
	scope[n-1] = scope[n];
	--n;
      }
    }
  }
  pred->arity = n;

  if( n == 1 )
    {
      scope[0]->deReference();
      pred->unsetRel();
      if( params && params[0] )
	scope[0]->setDomain( params[1]/params[0] );
      n=0;
    }
  
  return n>0;
}

void BuildObjectSumEq::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";
  pred->scope[0]->printshort( o );
  for(int i=1; i<pred->arity; ++i) {
    if( pred->params ) {
      if( pred->params[i] == -1 )
	o << " - ";
      else if( pred->params[i] < 0 )
	o << " - " << (-1 * pred->params[i]) << "*";
      else if( pred->params[i] > 1 )
	o << " + " << pred->params[i] << "*";
      else o << " + " ;
    } else 
      o << " + " ;
    pred->scope[i]->printshort( o );  
  }
  o << ") = " << pred->params[pred->arity]; 
}


string BuildObjectSumEq::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "global:sumeq";
  return s;
}

/**********************************************
 * Sum Constraint BuildObject
 **********************************************/
void BuildObjectSum::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con;
  if( pred->params ) 
    con = new PredicateWeightedSum( s, tmp, pred->arity+1, pred->params );
  else 
    con = new PredicateSimpleSum( s, tmp, pred->arity+1 );
}

bool BuildObjectSum::preBuild( BuildObjectPredicate *pred )
{
  int n=pred->arity, i=n, *params=pred->params;
  BuildObject **scope=pred->scope, *aux;

  // check if it should be a sum equality constraint
  if( scope[i]->size() == 1 ) {
    if( !params ) {
      pred->params = new int[n+1];
      std::fill(pred->params, pred->params+n, 1);
      pred->params[n] = 0;
    }
    pred->params[i] = scope[i]->min();
    pred->relation = ENVIRONMENT[ConstraintStore::SUMEQ];
    scope[i]->deReference();

    pred->unsetAwv();
    pred->unsetVar();
    pred->relation->preBuild( pred );
  } else {
    
    if( !params ) {
      pred->params = new int[n+1];
      std::fill(pred->params, pred->params+n, 1);
      pred->params[n] = 0;
    }

    // absorb constants
    while( i-- ) {
      if( scope[i]->size() == 1 ) {
	scope[i]->deReference();
	params[n] -= (params[i] * scope[i]->min());
	params[i] = params[n-1];
	scope[i] = scope[n-1];
	params[n-1] = params[n];
	scope[n-1] = scope[n];
	--n;
      }
    }
    pred->arity = n;

    // change to equal, factor or offset?
    if( n == 1 )
      {
	if( params && params[0] == 1 ) {
	  pred->relation = ENVIRONMENT[ConstraintStore::OFFSET];
	  params[0] = -params[1];
	  params[1] = 0;
	  pred->relation->preBuild( pred );
	} else if( params && params[1] == 0 ) { 
	  pred->relation = ENVIRONMENT[ConstraintStore::FACTOR];
	  pred->relation->preBuild( pred );
	}
      }
    // change to add or sub?
    else if( n == 2 && params && params[n] == 0 ) 
      {
	if( !params || (params[0] == 1 && params[1] == 1) ) {
	  pred->relation = ENVIRONMENT[ConstraintStore::ADD];
	  pred->relation->preBuild( pred );
	} else if( params && params[0] == 1 && params[1] == -1 ) {
	  pred->relation = ENVIRONMENT[ConstraintStore::SUB];
	  pred->relation->preBuild( pred );
	} else if( params && params[0] == -1 && params[1] == 1 ) {
	  pred->relation = ENVIRONMENT[ConstraintStore::SUB];
	  aux = scope[0];
	  scope[0] = scope[1];
	  scope[1] = aux;
	  pred->relation->preBuild( pred );
	}
      }
  }
  return pred->arity>0;
}

int BuildObjectSum::computeValue( BuildObjectPredicate *pred ) const
{
  int i = pred->arity, total = 0;
  while( i-- )
    total += (pred->params ? pred->params[i] * pred->scope[i]->value() : pred->scope[i]->value());
  return total;
}

void BuildObjectSum::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "(";

  if( pred->params ){
    if( pred->params[0] == -1 )
      o << "-";
    else if( pred->params[0] != 1 )
      o << pred->params[0] << "*";
  }
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );

    if( pred->params ){
      if( pred->params[i+1] == -1 )
	o << " - ";
      else if( pred->params[i+1] < 0 )
	o << " - " << (-1 * pred->params[i+1]) << "*";
      else if( pred->params[i+1] > 1 )
	o << " + " << pred->params[i+1] << "*";
      else o << " + " ;
    } else 
      o << " + " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}


string BuildObjectSum::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  
  string s;
  if( pred->arity > 2 )
    s = "global:sum";
  else {
    string x1 = pred->scope[0]->xmlPred(idx, level+1, sub);
    string x2 = pred->scope[1]->xmlPred(idx, level+1, sub);
    if( pred->params && pred->params[0] == 1 && pred->params[1] == -1 )
      s = ("sub(" + x1 + "," + x2 + ")");
    else
      s = ("add(" + ((pred->params && pred->params[0] != 1) ? 
		     ("mul(" + int2string(pred->params[0]) + "," + x1 + ")")  
		     : x1 )
	   + "," + ((pred->params && pred->params[1] != 1) ? 
		    ("mul(" + int2string(pred->params[1]) + "," + x2 + ")")  
		    : x2 )
	   + ")" );
  }
  return s;
}


/**********************************************
 * Min Constraint BuildObject
 **********************************************/
void BuildObjectMin::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateMin( s, tmp, pred->arity+1 );
}

int BuildObjectMin::computeValue( BuildObjectPredicate *pred ) const
{
  int i = pred->arity, themin = pred->scope[0]->value();
  while( --i )
    if( themin > pred->scope[i]->value() )
      themin = pred->scope[i]->value();
  return themin;
}

void BuildObjectMin::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "min(";
    for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}

string BuildObjectMin::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x[pred->arity];
  for(int i=0; i<pred->arity; ++i) 
    x[i] = pred->scope[i]->xmlPred( idx, level+1, sub );
  string s;
  for(int i=0; i<pred->arity-1; ++i) 
    s += ("min(" + x[i] + ",");
  s += x[pred->arity-1];
  for(int i=0; i<pred->arity-1; ++i) 
    s += ")";
  return s;
}


/**********************************************
 * Max Constraint BuildObject
 **********************************************/
void BuildObjectMax::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateMax( s, tmp, pred->arity+1 );
}

int BuildObjectMax::computeValue( BuildObjectPredicate *pred ) const
{
  int i = pred->arity, themax = pred->scope[0]->value();
  while( --i )
    if( themax < pred->scope[i]->value() )
      themax = pred->scope[i]->value();
  return themax;
}

void BuildObjectMax::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "max(";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}

string BuildObjectMax::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string x[pred->arity];
  for(int i=0; i<pred->arity; ++i) 
    x[i] = pred->scope[i]->xmlPred( idx, level+1, sub );
  string s;
  for(int i=0; i<pred->arity-1; ++i) 
    s += ("max(" + x[i] + ",");
  s += x[pred->arity-1];
  for(int i=0; i<pred->arity-1; ++i) 
    s += ")";
  return s;
}


#ifdef _CPLEX

/**********************************************
 * CPlexMinDifference Constraint BuildObject
 **********************************************/
void BuildObjectCPlexMinDifference::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateCPlexMinDiff( s, pred->params[0], tmp, pred->arity+1 );
}

void BuildObjectCPlexMinDifference::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "#[=/=](";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " ";
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}

#endif


#ifdef _COINLP

/**********************************************
 * MaxDifference Constraint BuildObject
 **********************************************/
void BuildObjectMaxDifference::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateMaxDiff( s, pred->params[0], tmp, 
					  pred->arity+1, pred->params[1] );
}

void BuildObjectMaxDifference::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "#[=/=](";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " ";
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}


/**********************************************
 * MinDifference Constraint BuildObject
 **********************************************/
void BuildObjectMinDifference::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateMinDiff( s, pred->params[0], tmp, 
					  pred->arity+1, pred->params[1] );
}

void BuildObjectMinDifference::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "#[=/=](";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " ";
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}

#endif

/**********************************************
 * Element Constraint BuildObject
 **********************************************/
void BuildObjectElement::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new PredicateElement( s, tmp, pred->arity+1 );
}

bool BuildObjectElement::preBuild( BuildObjectPredicate *pred )
{
  bool res = true;
  int v=pred->arity, n=v-1, i;
  BuildObject **scope=pred->scope;
  if( scope[n]->size() == 1 )
    {
      i = scope[n]->min();
      while( n-- ) 
	if( n != i )
	  scope[n]->deReference();
      scope[0] = scope[i];
      scope[1] = scope[v];

      pred->unsetRel();
      i = ( scope[0]->getVarId() < scope[1]->getVarId() );
      scope[1-i]->setEqual( scope[i] );
    }
  else res = BuildObjectConstraint::preBuild( pred );
  return res;
}

int BuildObjectElement::computeValue( BuildObjectPredicate *pred ) const
{
  return pred->scope[pred->scope[pred->arity-1]->value()]->value();
}

void BuildObjectElement::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "T[";
  pred->scope[pred->arity-1]->printshort( o );
  o << "], T={ ";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( cout );
    cout << " ";
  }
  o << "}";
}

string BuildObjectElement::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s;
  if( !level ) 
    s = "global:element";
  else
    s = ("X" + int2string(idx++)); 
  return s;
}


/**********************************************
 * Clause BuildObject
 **********************************************/ 
void BuildObjectClause::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new ConstraintClause( s, tmp, pred->arity, pred->params );
}

void BuildObjectClause::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " =/= ";
    o << pred->params[i];
    o << " or ";
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << " =/= ";
  o << pred->params[pred->arity-1];
}

string BuildObjectClause::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "table";
  return s;
}


/**********************************************
 *  Extensional Constraint BuildObject
 **********************************************/ 
BuildObjectTable::BuildObjectTable(const int a) 
{ 
  mindomain = new int[a];
  maxdomain = new int[a];
  for(int i=0; i<a; ++i) {
    mindomain[i] = INT_MAX;
    maxdomain[i] = -INT_MAX;
  }
  arity = a; 
  supports = NULL;
  supportTuples = NULL;
  supportList[0] = NULL;
  supportList[1] = NULL;
  var_sizes = NULL;
  spin=0;
}
BuildObjectTable::BuildObjectTable(const int a, const int nbt) 
{ 
  mindomain = new int[a];
  maxdomain = new int[a];
  for(int i=0; i<a; ++i) {
    mindomain[i] = INT_MAX;
    maxdomain[i] = -INT_MAX;
  }
  arity = a; 
  supports = NULL;
  supportTuples = NULL;
  supportList[0] = NULL;
  supportList[1] = NULL;
  var_sizes = NULL;
  tuples.init(0, nbt);
  spin=0;
}

void BuildObjectTable::setMinDom(const int i, const int m)
{
  if(mindomain[i] > m) mindomain[i] = m;
}
void BuildObjectTable::setMaxDom(const int i, const int m)
{
  if(maxdomain[i] < m) maxdomain[i] = m;
}
bool BuildObjectTable::preBuild( BuildObjectPredicate *pred )
{
  int i, j, k;
  bool res = BuildObjectConstraint::preBuild( pred );
  if( res ) {
    if( !spin && !tuples.size() ) {
      pred->unsetRel();
      res = false;
    } else if( !spin && arity == 2 ) {
      for(k=0; k<2; ++k) 
	if( pred->scope[k]->size() == 1 )
	  {
	    i=tuples.size();
	    j = pred->scope[k]->min();
	    pred->scope[k]->deReference();
	    pred->unsetRel();
	    while( i-- )
	      if( tuples[i][k] == j )
		pred->scope[1-k]->remove( tuples[i][1-k] );
	    res = false;
	  }
    }
    
    //  else {
    if( res ) {  
      res = BuildObjectConstraint::preBuild( pred );
      if( res ) {
	if( arity == 2 ) {
	  BuildObject **scope = pred->scope;
	  i=tuples.size();
	  if( spin ) {
	    while( i-- )
	      if(tuples[i][0] == tuples[i][1]) break;
	    if( i < 0 )
	      pred->setNeq();
	  } else {
	    int borneinf = std::max( scope[0]->min(), scope[1]->min() );
	    int bornesup = std::min( scope[0]->max(), scope[1]->max() );
	    k=(bornesup - borneinf + 1);
	    int values[k];
	    int *vals = values-borneinf;
	    for(j=borneinf; j<=bornesup; ++j)
	      if( scope[0]->contain(j) && scope[1]->contain(j) )
		vals[j] = 1;
	      else {
		vals[j] = 0;
		--k;
	      }
	    while( k && i-- ) {
	      j = tuples[i][0];
	      if(j == tuples[i][1] && j >= borneinf && j <= bornesup && vals[j]) 
		{
		  --k;
		  vals[j] = 0;
		}
	    }
	    if( !k )
	      pred->setNeq();
	  }
	}
      }
    }
  }
  return res;
}
void BuildObjectTable::build( Solver *s, VariableInt **tmp, 
			      BuildObjectPredicate *pred ) 
{
  long int matrixsize = 1;
  int i, j, n=pred->arity, tightness;
  for(i=0; matrixsize > 0 && i<n; ++i)
    matrixsize *= (maxdomain[i] - mindomain[i]);
  if( matrixsize < 0 ) tightness = INT_MAX;
  else {
    if( spin )
      tightness = (matrixsize / tuples.size());
    else
      tightness = (matrixsize / (matrixsize - tuples.size()));
  }

  if( spin || tuples.size() ) {    
    if( !spin && tuples.size() == 1 ) { // CLAUSE 
      ConstraintClause *con = new ConstraintClause( s, tmp, n, tuples[0] );
    } else if( tightness < 1000 && n == 2 ) { // SUPPORT BITSETS
      if( supportList[0] == NULL ) {  
	i=2;
	while( i-- ) { 
	  supportList[i] = new MistralSet[maxdomain[i] - mindomain[i]];
	  supportList[i]-= mindomain[i];
	  j = maxdomain[i];
	  while( j-- > mindomain[i] )
	    supportList[i][j].init( mindomain[1-i],
				    maxdomain[1-i]-1,
				    (spin ? 0 : 0xffffffff ) );
	}
 	i=tuples.size();
	while( i-- ) {
	  j=2;
	  while( j-- ) {
	    if( spin )
	      supportList[j][tuples[i][j]].insert( tuples[i][1-j] );
	    else 
	      supportList[j][tuples[i][j]].erase( tuples[i][1-j] );
	  }
	}
      }
      ConstraintBinaryExtensional *con = 
	new ConstraintBinaryExtensional( s, tmp, supportList );
      con->init();
    } else if( !spin || (matrixsize < 32000000 && tightness < 64) ) { // N-ARY MATRIX
      if( var_sizes == NULL ) {
	var_sizes = new int[n-1];
	var_sizes[n-2] = (maxdomain[n-1] - mindomain[n-1]);
	for(i=n-3; i >= 0; --i)
	  var_sizes[i] = var_sizes[i+1] * 
	    (maxdomain[i+1] - mindomain[i+1]);
	matrixmin = mindomain[n-1];
	int matrixmax = maxdomain[n-1]-1;
	
	i = n-1;
	while( i-- ) {
	  matrixmax += (maxdomain[i])*var_sizes[i]; 
	  matrixmin += (mindomain[i])*var_sizes[i]; 
	}

	matrix.init(matrixmin, matrixmax, MistralSet::empt);
      }

      ConstraintExtensional *con = new ConstraintExtensional( s, tmp, n, 
							      matrix, 
							      var_sizes );
     
      con->useResidualSupports();
      
      if( spin ) {
	con->fillNogood();
	i=tuples.size();
	while( i-- )
	  con->addSupport( tuples[i] );
      } else {
	con->fillSupport();
	i=tuples.size();
	while( i-- )
	  con->addNogood( tuples[i] );
      }
    } else { // LIST OF TUPLES
      if( !supportTuples ) {
	supportTuples = new MistralStack<int*>*[n];
	for(i=0; i<n; ++i) {
	  supportTuples[i] = new MistralStack<int*> [maxdomain[i] - mindomain[i]];
	  supportTuples[i] -= mindomain[i];
	  for(j=mindomain[i]; j<maxdomain[i]; ++j)
	    assert( supportTuples[i][j].size_ == 0 );
	}
	i=tuples.size();
	while( i-- )
	  for(j=0; j<n; ++j) {
	    supportTuples[j][tuples[i][j]].push( tuples[i] );
	  }
      }
      ConstraintLargeExtensional *con = 
	new ConstraintLargeExtensional( s, tmp, n, supportTuples );
    }
  }
}

BuildObjectTable::~BuildObjectTable() 
{
  int i;
  i=tuples.size();
  while( i-- ) {
    delete [] tuples[i];
  }
  i=arity;  
  while( i-- ) {
    if( i<2 && supportList[i] ) {
      supportList[i] += mindomain[i];
      delete [] supportList[i];
    }
    if( supportTuples ) {
      supportTuples[i] += mindomain[i];
      delete [] supportTuples[i];
    }
  }

  if( var_sizes ) {
    delete [] var_sizes;
  }
  delete [] mindomain;
  delete [] maxdomain;
  delete [] supports;
  delete [] supportTuples;
}

void BuildObjectTable::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "table(";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}

string BuildObjectTable::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "table";
  return s;
}

void BuildObjectTable::add( const int* sol )
{
  int i = arity, *ngd = new int[arity];

  while( i-- ) 
    ngd[i] = sol[i];

  tuples.push_back( ngd );
}


/**********************************************
 * AllDiff Constraint BuildObject
 **********************************************/
void BuildObjectAllDiff::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new ConstraintAllDiff( s, tmp, pred->arity );
  if( !pred->params ) {
    int i, j;
    VariableInt *scp[2];
    for(i=1; i<pred->arity; ++i)
      for(j=0; j<i; ++j) {
	scp[0] = tmp[j];
	scp[1] = tmp[i];
	con = new ConstraintNotEqual( s, scp );
      }
  }
}

bool BuildObjectAllDiff::preBuild( BuildObjectPredicate *pred )
{
  bool res = true;
  int n=pred->arity, i=n, j, v;
  if( n == 2 ) {
    pred->relation = ENVIRONMENT[ConstraintStore::NEQ];
    res = pred->relation->preBuild( pred );
  } else {
    BuildObject **scope=pred->scope;
    while( i-- )
      if( scope[i]->size() == 1 ) 
	{
	  j = n;
	  v = scope[i]->min();
	  while( j-- ) if( j != i ) scope[j]->remove( v );
	  scope[i]->deReference();
	  scope[i] = scope[--n];
	}
    pred->arity = n;
  }
  if( !pred->arity ) {
    pred->unsetRel();
    res = false;
  }
  return res;
}

void BuildObjectAllDiff::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "AllDifferent(";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}
string BuildObjectAllDiff::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s;
  if( !level ) 
    s = "global:allDifferent";
  else
    s = ("X" + int2string(idx++)); 
  return s;
}


/**********************************************
 * Gcc Constraint BuildObject
 **********************************************/
void BuildObjectGcc::build( Solver *s, VariableInt **tmp, 
			    BuildObjectPredicate *pred )
{
  int n=pred->arity, m = (pred->params[1]-pred->params[0]+1);
  Constraint *con = 
    new ConstraintGlobalCardinality( s, tmp, n,
				     pred->params[0],
				     pred->params[1],
				     &(pred->params[2]),
				     &(pred->params[2+m]) );
  for(int j=0; j<m; ++j) {
    if( pred->params[2+j] > 0 ) {
      VariableInt *B[n];
      VariableInt *scp[2];
      for(int i=0; i<n; ++i) {
	B[i] = new VariableBool(s);
	scp[0] = tmp[i];
	scp[1] = B[i];
	con = new PredicateEqualConstant(s, scp, pred->params[0]+j, 1);
      }
      con = new ConstraintBoolSumMore(s, B, n, pred->params[2+j]);
    }
    if( pred->params[2+m+j] < n ) {
      VariableInt *B[n];
      VariableInt *scp[2];
      for(int i=0; i<n; ++i) {
	B[i] = new VariableBool(s);
	scp[0] = tmp[i];
	scp[1] = B[i];
	con = new PredicateEqualConstant(s, scp, pred->params[0]+j, 1);
      }
      con = new ConstraintBoolSumLess(s, B, n, pred->params[2+m+j]);
    }
  }
}

void BuildObjectGcc::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "Gcc(";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ", [" << pred->params[0] << ".." 
    << pred->params[1] << "] >={";
  for(int i=0; i<(pred->params[1]-pred->params[0]+1); ++i) {
    o << " " << pred->params[2+i];
  }
  o << " }, <={";
  for(int i=0; i<(pred->params[1]-pred->params[0]+1); ++i) {
    o << " " << pred->params[3+pred->params[1]-pred->params[0]+i];
  }
  o << " })";
}
string BuildObjectGcc::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "global:gcc";
  return s;
}


/**********************************************
 * EdgeFinder Constraint BuildObject
 **********************************************/
void BuildObjectEdgeFinder::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Constraint *con = new ConstraintEdgeFinder( s, tmp, pred->arity, pred->params );
}

void BuildObjectEdgeFinder::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "EdgeFinder(";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}

string BuildObjectEdgeFinder::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "global:unary";
  return s;
}


/**********************************************
 * Lex Constraint BuildObject
 **********************************************/
void BuildObjectLex::build( Solver *s, VariableInt **tmp, 
			    BuildObjectPredicate *pred ) 
{
  Constraint *con;
  int i, m = (pred->arity >> 1);
  VariableInt *B[m+1];
  for(i=1; i<m; ++i)
    B[i] = new VariableBool(s);
  B[0] = new Constant(s,0);
  if( pred->params[0] ) {
    B[m] = new Constant(s,1);
  } else 
    B[m] = new VariableBool(s);
  
  for(i=0; i<m; ++i) {
    VariableInt *scp[4];
    scp[0] = tmp[i];
    scp[1] = tmp[i+m];
    scp[2] = B[i];
    scp[3] = B[i+1];
    con = new ConstraintLex(s, scp);
  }
}

void BuildObjectLex::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  int i;
  o << "[ ";
  for(i=0; i<(pred->arity/2-1); ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[i]->printshort( o );
  o << "]"; 
  if(pred->params[0])
    o << " <= ";
  else
    o << " < ";
  o << "[ ";
  for(i=(pred->arity/2); i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[i]->printshort( o );
  o << "]";
}

string BuildObjectLex::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "global:lex";
  return s;
}


/**********************************************
 * MaxOccurrence Constraint BuildObject
 **********************************************/
void BuildObjectMaxOccurrence::build( Solver *s, VariableInt **tmp, 
				      BuildObjectPredicate *pred ) 
{
  int n=pred->arity;
  Constraint *con;
  VariableInt *B[n];
  VariableInt *scp[2];
  for(int i=0; i<n; ++i) {
    B[i] = new VariableBool(s);
    scp[0] = tmp[i];
    scp[1] = B[i];
    con = new PredicateEqualConstant(s, scp, pred->params[0], 1);
  }
  con = new ConstraintBoolSumLess(s, B, n, pred->params[1]);
}

void BuildObjectMaxOccurrence::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  int i;
  o << "occ(" << pred->params[0] << ") <= " << pred->params[1] << " in [";
  for(i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << "," ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << "]";
}

string BuildObjectMaxOccurrence::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "global:maxOccurence";
  return s;
}


/**********************************************
 * MinOccurrence Constraint BuildObject
 **********************************************/
void BuildObjectMinOccurrence::build( Solver *s, VariableInt **tmp, 
				      BuildObjectPredicate *pred ) 
{
  int n=pred->arity;
  Constraint *con;
  VariableInt *B[n];
  VariableInt *scp[2];
  for(int i=0; i<n; ++i) {
    B[i] = new VariableBool(s);
    scp[0] = tmp[i];
    scp[1] = B[i];
    con = new PredicateEqualConstant(s, scp, pred->params[0], 1);
  }
  con = new ConstraintBoolSumMore(s, B, n, pred->params[1]);
}

void BuildObjectMinOccurrence::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  int i;
  o << "occ(" << pred->params[0] << ") <= " << pred->params[1] << " in [";
  for(i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << "," ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << "]";
}

string BuildObjectMinOccurrence::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "global:minOccurrence";
  return s;
}


/**********************************************
 * Slide Constraint BuildObject
 **********************************************/
inline void BuildObjectSlide::tuple2int(int& x, int* tuple, int n, int* exposant) {
  x = 0;
  while( n-- )
    x += (tuple[n] * exposant[n]);  
}

void BuildObjectSlide::build( Solver *s, VariableInt **tmp, BuildObjectPredicate *pred ) 
{
  Solver sub_solver( *sub_problem );

  int i, j, k, left, right, aux;

  int sequenceWidth = sub_solver.length;
  int nbVals = sub_solver.sequence[0]->domsize();
  int x, tuple[sequenceWidth];

  int *exposant = new int[sequenceWidth];
  for(i=0; i<sequenceWidth; ++i)
    exposant[i] = (int)pow(nbVals, i); 

  int* int2ind = new int[exposant[sequenceWidth-1]];
  int* ind2int = new int[exposant[sequenceWidth-1]];
  int* conflicts = new int[nbVals*exposant[sequenceWidth-1]]; 
  std::fill( ind2int, ind2int+exposant[sequenceWidth-1], -1 );
  std::fill( int2ind, int2ind+exposant[sequenceWidth-1], -1 );
  std::fill( conflicts, conflicts+(nbVals*exposant[sequenceWidth-1]), 1);


  sub_solver.startNewSearch();

  j=0; k=0;
  while( sub_solver.getNextSolution() ) {
    for(i=0; i<sequenceWidth; ++i) 
      tuple[i] = sub_solver.variables[i]->value();

    tuple2int(x, tuple, sequenceWidth, exposant);

    left = (x%exposant[sequenceWidth-1]);
    right = (x/nbVals);

    if( int2ind[left] < 0 ) {
      int2ind[left] = k;
      ind2int[k++] = left;
    }

    if( int2ind[right] < 0 ) {
      int2ind[right] = k;
      ind2int[k++] = right;
    }

    //ind2int[left]%nbVals + ind2int[right]*nbVals

    conflicts[(left%nbVals + right*nbVals)] = 0;

    //conflicts[ktuple(int2ind[left], int2ind[right])] = 0;

    cout << setw(6) <<  int2ind[left] << setw(6) << int2ind[right] ;
    for(i=0; i<sequenceWidth; ++i) {
      cout << setw(3) << tuple[i] ;
    }
    cout << endl;

  }

  Constraint *decomposition = new ConstraintSlideDecomp( s, tmp, pred->params[0], pred->params[1], 
							 nbVals, sequenceWidth, k,  
							 conflicts, int2ind, ind2int, exposant );

  //   int nSlideVars = m-sequenceWidth+2;
  //   VariableInt *slideVars[n*nSlideVars];
  //   for(i=0; i<n; ++i)
  //     for(j=0; j<nSlideVars; ++j)
  //       slideVars[i*nSlideVars+j] = new VariableBit( solver, 0, k-1 );
  
  //   VariableInt *scp[sequenceWidth];
  //   for(i=0; i<n; ++i) {
  //     for(j=0; j<nSlideVars; ++j) {
  //       for(k=0; k<sequenceWidth-1; ++k)
  // 	scp[k] = X[i*m+j+k];
  //       scp[sequenceWidth-1] = slideVars[i*nSlideVars+j];
  //       new PredicateTuple( solver, scp,
  // 			  sequenceWidth,
  // 			  nbVals,
  // 			  int2ind,
  // 			  ind2int,
  // 			  exposant ) );
    
  //     for(j=0; j<nSlideVars-1; ++j) {
  //       scp[0] = slideVars[i*nSlideVars+j];
  //       scp[1] = slideVars[i*nSlideVars+j+1];
  //       new SlideI( solver, scp, decomposition );
  //     }
  //   }
}

void BuildObjectSlide::print(std::ostream& o, const BuildObjectPredicate *pred) const
{
  o << "Slide(";
  for(int i=0; i<pred->arity-1; ++i) {
    pred->scope[i]->printshort( o );
    o << " " ;
  }
  pred->scope[pred->arity-1]->printshort( o );
  o << ")";
}

string BuildObjectSlide::xmlPred(int& idx, int level, const BuildObjectPredicate *pred, int *sub) const 
{
  string s = "global:slide";
  return s;
}


/**********************************************
 * Adding variables/constraints to the model
 **********************************************/

int CSP::numVars() const
{
  return (nList + nBit + nBoolean + nRange) ;
}

int CSP::numEVars() const
{
  return (eList + eBit + eBoolean + eRange) ;
}

inline void CSP::triggerEvent(BuildObject* x, int e) 
{
  buildstack.push( x, e );
}


void CSP::add( BuildObject *x ) 
{
  x->add( this, 0, (BuildObject::CRANGE | BuildObject::CLIST) );
}

void CSP::add( BuildObjectObjective *x ) 
{
  x->X->add( this, 1, (BuildObject::CRANGE | BuildObject::CLIST) );
  goald = x;
}

void CSP::add( BuildObject **x, const int n ) 
{
  for(int i=0; i<n; ++i) 
    x[i]->add( this, 0, (BuildObject::CRANGE | BuildObject::CLIST) );
}

void CSP::add( Variable x )
{
  x.var_ptr_->add( this, 0, (BuildObject::CRANGE | BuildObject::CLIST) );
}

void CSP::add( VarArray& x )
{
  int n=x.size();
  for(int i=0; i<n; ++i) {
    x[i].var_ptr_->add( this, 0, (BuildObject::CRANGE | BuildObject::CLIST) );
  }
}

void CSP::add( Objective g )
{
  g.X->add( this, 1, (BuildObject::CRANGE | BuildObject::CLIST) );
  goald = g.obj;
}

void CSP::close()
{
  buildstack.initStack( declarations.size() ) ;
}

int CSP::inferAllDiffs( const int all, const int limit )
{
  int i, n=declarations.size(), // m=notequals.size(),
    x, y, ncliques=0, maxsize;

  // compute nodes and edges of the graph
  vector< int > edges;
  vector< int > vertice;
  int vertice_index[n];
  std::fill(vertice_index, vertice_index+n, -1);
  
  i = notequals.size();
  while( i-- )
    {
      x = ((BuildObjectPredicate*)notequals[i])->scope[0]->id;
      y = ((BuildObjectPredicate*)notequals[i])->scope[1]->id;
      
      if( vertice_index[x] < 0  ) {
	vertice_index[x] = vertice.size();
	vertice.push_back( x );
      }
      if( vertice_index[y] < 0  ) {
	vertice_index[y] = vertice.size();
	vertice.push_back( y );
      }
      
      edges.push_back( vertice_index[x] );
      edges.push_back( vertice_index[y] );
    }
    
  if( vertice.size() < 4096 && edges.size() > (vertice.size() * 2) )
    {
      MistralGraph G( vertice.size(), edges );      
      vector< vector< int > > cliques;      
      if( all ) G.getAllCliques( cliques, limit );
      else G.getSomeCliques( cliques );
      
      x = cliques.size();
      while( x-- ) {
	i = n = cliques[x].size();
	maxsize = 0;
	while( i-- ) {
	  y = declarations[vertice[cliques[x][i]]]->size();
	  if(maxsize < y) maxsize = y;
	}

	if( n > 3 && maxsize <= n+1 ) {
	  i=n;
	  ++ncliques;
	  BuildObject **tmp = new BuildObject*[n+1];
	  while( i-- ) tmp[n-i-1] = declarations[vertice[cliques[x][i]]];
	  int *par = new int[1];
	  BuildObjectPredicate *cliqueAllDiff = 
	    new BuildObjectPredicate( tmp, n, 0, -1, 
				      ENVIRONMENT[ConstraintStore::ALLDIFF],
				      par, (BuildObject::CRANGE | BuildObject::CONVEX) ) ;
	  cliqueAllDiff->add( this, 0, (BuildObject::CRANGE | BuildObject::CLIST) );
	  
	}
      }
    }
  return ncliques;
}
  
void CSP::preBuild()
{
  close();

  int n=declarations.size(), i=n;

  while( i-- ) {

     declarations[i]->preBuild( );

  }

  while( !buildstack.empty() ) {
    int event;
    BuildObject* x = buildstack.pop( event );
    i=x->parent.size();
    while( i-- ) {
      if( x->parent[i]->isRel() ) {

	x->parent[i]->preBuild( );

      }
    }
  }


  i=n;
  while( i-- ) {

    if( declarations[i]->isNeq() ) {
      ++nNeq;
      notequals.push( declarations[i] );
    }


    if( declarations[i]->isVar() ) {
      if( declarations[i]->isSearchable() ) {
	nValues += declarations[i]->size();
	if( declarations[i]->isBoolean() ) {
	  if( declarations[i]->isRel() )
	    ++eBoolean;
	  else
	    ++nBoolean;
	  //cout << "Boolean " ;
	} else if( declarations[i]->isRange() ) {
	  if( declarations[i]->isRel() )
	    ++eRange;
	  else
	    ++nRange;
	  //cout << "Range " ;
	} else if( declarations[i]->isVList() ) {
	  if( declarations[i]->isRel() )
	    ++eList;
	  else
	    ++nList;
	  //cout << "Domain " ;
	} else {
	  if( declarations[i]->isRel() )
	    ++eBit;
	  else
	    ++nBit;
	  //cout << "Domain " ;
	}
      } else {
	//cout << "Constant " ;
	if( declarations[i]->isRel() )
	  ++eConstant;
	else
	  ++nConstant;
      }
    }
    //if( declarations[i]->isConstraint() ) {
    if( declarations[i]->isRel() ) {

      //cout << "Constraint " ;
      if( declarations[i]->isVar() )
	++eConstraint;
      else
	++nConstraint;
    }
    //    cout << endl;
  }
}


void CSP::build( Solver *s )
{
  int i, n=declarations.size();
  for(i=0; i<n; ++i) {
    if( 
       declarations[i]->isVar() || 
       declarations[i]->isRel() ) {

      declarations[i]->build( s );

    }
  }
  
  if( goald )
    goald->build( s );
}



/**********************************************
 * Operator overloading
 **********************************************/

Variable VarArray::operator[]( Variable x ) 
{
  return Element( *this, x );
}

Variable VarArray::operator< ( VarArray& x ) 
{
  return LexOrder( *this, x, 1 );
}
Variable VarArray::operator> ( VarArray& x ) 
{
  return LexOrder( x, *this, 1 );
}
Variable VarArray::operator<=( VarArray& x ) 
{
  return LexOrder( *this, x, 0 );
}
Variable VarArray::operator>=( VarArray& x )
{
  return LexOrder( x, *this, 0 );
}

Variable Variable::operator==( const MistralSet& s ) 
{
  // UNSOUND
  var_ptr_->setDomain( s );
  return *this;
}

Variable Variable::operator!=( const MistralSet& s ) 
{
  // UNSOUND
  var_ptr_->removeSet( s );
  return *this;
}

Variable Variable::operator==( const int k ) 
{
  return Equal( *this, k );
}

Variable Variable::operator%( const int k ) 
{
  return Mod( *this, k );
}

Variable Variable::operator!=( const int k ) 
{
  return NotEqual( *this, k );
}

Variable Variable::operator<=( const int k ) 
{
  return Precedence( *this, k );
}

Variable Variable::operator< ( const int k ) 
{
  return Precedence( *this, k+1 );
}

Variable Variable::operator>=( const int k ) 
{
  return Precedence( k, *this );
}

Variable Variable::operator> ( const int k ) 
{
  return Precedence( k+1, *this );
}

Variable Variable::operator+ ( Variable x )
{
  return Add( *this, x );
}

Variable Variable::operator- ( Variable x )
{
  return Sub( *this, x );
}

Variable Variable::operator* ( Variable x )
{
  return Mul( *this, x );
}

Variable Variable::operator/ ( Variable x )
{
  return Div( *this, x );
}

Variable Variable::operator^ ( Variable x )
{
  return Pow( *this, x );
}

Variable Variable::operator%( Variable x )
{
  return Mod( *this, x );
}

Variable Variable::operator&&( Variable x )
{
  return And( *this, x );
}

Variable Variable::operator||( Variable x )
{
  return Or( *this, x );
}

Variable Variable::operator==( Variable x ) 
{
  return Equal( *this, x );
}
 
Variable Variable::operator!=( Variable x ) 
{
  return NotEqual( *this, x );
}

Variable Variable::operator<=( Variable x ) 
{
  return Precedence( *this, 0, x );
}

Variable Variable::operator>=( Variable x ) 
{
  return Precedence( x, 0, *this );
}

Variable Variable::operator< ( Variable x ) 
{
  return Precedence( *this, 1, x );
}

Variable Variable::operator> ( Variable x ) 
{
  return Precedence( x, 1, *this );
}

Variable Variable::operator-( )
{
  return Negation( *this );
}

Variable Variable::operator!( )
{
  return Not( *this );
}

Variable Variable::operator*( const int k )
{
  if( k != 1 )
    return Factor( *this, k );
  else return *this;
}

Variable Variable::operator-( const int k )
{
  if( k )
    return Offset( *this, -k );
  else return *this;
}

Variable Variable::operator+( const int k )
{
  if( k )
    return Offset( *this, k );
  else return *this;
}


/**********************************************
 *  Offset Predicate Wrapper
 **********************************************/ 
Offset::Offset( Variable& x_, int o )
{
  var_ptr_ = CSP::_Offset( x_.var_ptr_, o );
}

/**********************************************
 *  Factor Predicate Wrapper
 **********************************************/ 
Factor::Factor( Variable& x_, int o )
{
  var_ptr_ = CSP::_Factor( x_.var_ptr_, o );
}

/**********************************************
 *  Negation Predicate Wrapper
 **********************************************/ 
Negation::Negation( Variable& x_ )
{
  var_ptr_ = CSP::_Negation( x_.var_ptr_ );
}

/**********************************************
 *  Not Predicate Wrapper
 **********************************************/ 
Not::Not( Variable& x_ )
{
  var_ptr_ = CSP::_Not( x_.var_ptr_ );
}

/**********************************************
 *  Abs Predicate Wrapper
 **********************************************/ 
Abs::Abs( Variable x_ )
{
  var_ptr_ = CSP::_Abs( x_.var_ptr_ );
}

/**********************************************
 *  And Predicate Wrapper
 **********************************************/ 
And::And( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_And( x_.var_ptr_, y_.var_ptr_ );
}

/**********************************************
 *  Or Predicate Wrapper
 **********************************************/ 
Or::Or( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Or( x_.var_ptr_, y_.var_ptr_ );
}

/**********************************************
 *  IfThenElse Predicate Wrapper
 **********************************************/ 
IfThenElse::IfThenElse( Variable& x_, Variable& y_, Variable& z_ )
{
  var_ptr_ = CSP::_IfThenElse( x_.var_ptr_, y_.var_ptr_, z_.var_ptr_ );
}

/**********************************************
 *  Disjunctive Predicate Wrapper
 **********************************************/ 
Disjunctive::Disjunctive( Variable& x_, int dx, Variable& y_, int dy )
{
  var_ptr_ = CSP::_Disjunctive( x_.var_ptr_, dx, y_.var_ptr_, dy );
}

/**********************************************
 *  Precedence Predicate Wrapper
 **********************************************/ 
Precedence::Precedence( Variable& x_, int d, Variable& y_ )
{
  var_ptr_ = CSP::_Precedence( x_.var_ptr_, d, y_.var_ptr_ );
}

Precedence::Precedence( Variable& x_, int b )
{
  var_ptr_ = CSP::_Precedence( x_.var_ptr_, b );
}

Precedence::Precedence( int b, Variable& x_ )
{
  var_ptr_ = CSP::_Precedence( b, x_.var_ptr_ );
}

/**********************************************
 *  Add Predicate Wrapper
 **********************************************/
Add::Add( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Add( x_.var_ptr_, y_.var_ptr_ );
}

/**********************************************
 *  Sub Predicate Wrapper
 **********************************************/
Sub::Sub( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Sub( x_.var_ptr_, y_.var_ptr_ );
}

/**********************************************
 *  Mul Predicate Wrapper
 **********************************************/
Mul::Mul( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Mul( x_.var_ptr_, y_.var_ptr_ );
}

/**********************************************
 *  Div Predicate Wrapper
 **********************************************/
Div::Div( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Div( x_.var_ptr_, y_.var_ptr_ );
}

/**********************************************
 *  Pow Predicate Wrapper
 **********************************************/
Pow::Pow( Variable& x_, Variable& y_ )
{
}

/**********************************************
 *  Mod Predicate Wrapper
 **********************************************/
Mod::Mod( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Mod( x_.var_ptr_, y_.var_ptr_ );
}

Mod::Mod( Variable& x_, const int k )
{
  var_ptr_ = CSP::_Mod( x_.var_ptr_, k );
}

/**********************************************
 *  Equal Predicate Wrapper
 **********************************************/ 
Equal::Equal( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Equal( x_.var_ptr_, y_.var_ptr_ );
}

Equal::Equal( Variable& x_, int k )
{
  var_ptr_ = CSP::_Equal( x_.var_ptr_, k );
}

/**********************************************
 *  NotEqual Predicate Wrapper
 **********************************************/ 
NotEqual::NotEqual( Variable& x_, Variable& y_ )
{
  var_ptr_ = CSP::_NotEqual( x_.var_ptr_, y_.var_ptr_ );
}

NotEqual::NotEqual( Variable& x_, int k )
{
  var_ptr_ = CSP::_NotEqual( x_.var_ptr_, k );
}

/**********************************************
 *  NotEqualWildcard Predicate Wrapper
 **********************************************/ 
NotEqualWildcard::NotEqualWildcard( Variable& x_, Variable& y_, const int w )
{
  var_ptr_ = CSP::_NotEqualWildcard( x_.var_ptr_, y_.var_ptr_, w );
}

/**********************************************
 * Sum Predicate Wrapper
 **********************************************/
Sum::Sum( VarArray& x_, const int *os )
{
  var_ptr_ = CSP::_Sum( x_.getArgs(), x_.size(), os );
}

/**********************************************
 * Min Predicate Wrapper
 **********************************************/
Min::Min( VarArray& x_ )
{
  var_ptr_ = CSP::_Min( x_.getArgs(), x_.size() );
}

/**********************************************
 * Max Predicate Wrapper
 **********************************************/
Max::Max( VarArray& x_ )
{
  var_ptr_ = CSP::_Max( x_.getArgs(), x_.size() );
}

/**********************************************
 * Element Predicate Wrapper
 **********************************************/
Element::Element( VarArray& x_, Variable& y_ )
{
  var_ptr_ = CSP::_Element( x_.getArgs(), x_.size(), y_.var_ptr_ );
}

Element::Element( Variable& y_, VarArray& x_, Variable& z_ )
{
  var_ptr_ = CSP::_Element( y_.var_ptr_, x_.getArgs(), x_.size(), z_.var_ptr_ );
}

Element::Element( VarArray& x_ )
{
  var_ptr_ = CSP::_Element( x_.getArgs(), x_.size() );
}

/**********************************************
 *  Clause Wrapper
 **********************************************/ 
Clause::Clause( VarArray& x_, const int* c )
{
  var_ptr_ = CSP::_Clause( x_.getArgs(), x_.size(), c );
}

/**********************************************
 *  Extensional Constraint Wrapper
 **********************************************/ 
int CSP::addTable( const int arity, const int nbTuples )
{
  return ENVIRONMENT.addConstraint( new BuildObjectTable(arity, nbTuples) );
}
void CSP::addTuple( const int idx, int *tuple )
{
  ((BuildObjectTable*)(ENVIRONMENT[idx]))->add( tuple );
}
Table::Table() {}
Table::Table( VarArray& x_, const bool sp )
{
  var_ptr_ = CSP::_Table( x_.getArgs(), x_.size(), sp ); 
  tabptr_ = (BuildObjectTable*)(((BuildObjectPredicate*)var_ptr_)->relation);
}
Table::Table( VarArray& x_, const int tidx, const bool sp )
{
  var_ptr_ = CSP::_Table( x_.getArgs(), x_.size(), tidx, sp );
  tabptr_ = (BuildObjectTable*)(((BuildObjectPredicate*)var_ptr_)->relation);
}

void Table::add( const int* sol )
{
  tabptr_->add( sol ) ;
}

/**********************************************
 * AllDifferent Constraint Wrapper
 **********************************************/
AllDifferent::AllDifferent( VarArray& x_ )
{
  var_ptr_ = CSP::_AllDifferent( x_.getArgs(), x_.size() );
}

/**********************************************
 * BoundAllDifferent Constraint Wrapper
 **********************************************/
BoundAllDifferent::BoundAllDifferent( VarArray& x_ )
{
  var_ptr_ = CSP::_BoundAllDifferent( x_.getArgs(), x_.size() );
}

/**********************************************
 * Gcc Constraint Wrapper
 **********************************************/
Gcc::Gcc( VarArray& x_,
	  const int firstDomainValue, 
	  const int lastDomainValue,
	  const int* minOccurrences,
	  const int* maxOccurrences )
{
  var_ptr_ = CSP::_Gcc( x_.getArgs(), x_.size(),
		       firstDomainValue,  lastDomainValue, 
		       minOccurrences, maxOccurrences );
}

/**********************************************
 * EdgeFinder Constraint Wrapper
 **********************************************/
EdgeFinder::EdgeFinder( VarArray& x_,
			const int *d )
{
  var_ptr_ = CSP::_EdgeFinder( x_.getArgs(), x_.size(), d );
}

/**********************************************
 * Lex < Constraint Wrapper
 **********************************************/
LexOrder::LexOrder( VarArray& x_, VarArray& y_, int eq )
{
  var_ptr_ = CSP::_LexOrder( x_.getArgs(), y_.getArgs(), x_.size(), eq );
}

/**********************************************
 * MaxOccurrence < Constraint Wrapper
 **********************************************/
MaxOccurrence::MaxOccurrence( VarArray& x_, const int k, const int ub  )
{
  var_ptr_ = CSP::_MaxOccurrence( x_.getArgs(), x_.size(), k, ub );
}

/**********************************************
 * MinOccurrence < Constraint Wrapper
 **********************************************/
MinOccurrence::MinOccurrence( VarArray& x_, const int k, const int lb  )
{
  var_ptr_ = CSP::_MinOccurrence( x_.getArgs(), x_.size(), k, lb );
}

/**********************************************
 * Slide Constraint Wrapper
 **********************************************/
Slide::Slide( VarArray& x_, const int n, const int m, CSP* p )
{
  var_ptr_ = CSP::_Slide( x_.getArgs(), n, m, p );
}


#ifdef _CPLEX

/**********************************************
 * CPlexMinDifference Predicate Wrapper
 **********************************************/
CPlexMinDifference::CPlexMinDifference( VarArray& x_, const int ft )
{
  var_ptr_ = CSP::_CPlexMinDifference( x_.getArgs(), x_.size(), ft );
}

#endif


#ifdef _COINLP

/**********************************************
 * MaxDifference Predicate Wrapper
 **********************************************/
MaxDifference::MaxDifference( VarArray& x, const int mt, const int ft )
{
  var_ptr_ = CSP::_MaxDifference( x_.getArgs(), x_.size(), mt, ft );
}

/**********************************************
 * MinDifference Predicate Wrapper
 **********************************************/
MinDifference::MinDifference( VarArray& x, const int mt, const int ft )
{
  var_ptr_ = CSP::_MinDifference( x_.getArgs(), x_.size(), mt, ft );
}

#endif

/**********************************************
 * Printing routines
 **********************************************/

ostream& operator<< (ostream& os, const Matrix& x) 
{
  x.print( os );
  return os;
}

ostream& operator<< (ostream& os, const Variable& x)
{
  x.print( os );
  return os;
}

ostream& operator<< (ostream& os, const VarArray& x)
{
  //  x.printshort( os );
  x.print( os );
  return os;
}

ostream& operator<< (ostream& os, const CSP& p)
{
  p.print( os );
  return os;
}



/**********************************************
 * Objective Function BuildObjects
 **********************************************/
BuildObjectObjective::BuildObjectObjective(BuildObject *x) : X(x) {}

void BuildObjectObjective::print( std::ostream& o ) const {}


/// Maximisation declaration
BuildObjectMaximiseVar::BuildObjectMaximiseVar(BuildObject *x) 
  : BuildObjectObjective(x)
{
}

void BuildObjectMaximiseVar::print( std::ostream& o ) const 
{
  o << "Maximise ";
  X->printshort( o ) ;
}

void BuildObjectMaximiseVar::build( Solver *s )
{
  s->goal = new MaximiseVar( s, X->getVariable() );
}

/// Minimisation declaration
BuildObjectMinimiseVar::BuildObjectMinimiseVar(BuildObject *x) 
  : BuildObjectObjective(x)
{
}

void BuildObjectMinimiseVar::print( std::ostream& o ) const 
{
  o << "Minimise ";
  X->printshort( o ) ;
}

void BuildObjectMinimiseVar::build( Solver *s )
{
  s->goal = new MinimiseVar( s, X->getVariable() );
}


/**********************************************
 * CSP
 **********************************************/

CSP::CSP() 
{
  nValues     = 0;
  nList       = 0;
  nBit        = 0;
  nBoolean    = 0;
  nRange      = 0;
  nConstant   = 0;
  nConstraint = 0;
  eValues     = 0;
  eList       = 0;
  eBit        = 0;
  eBoolean    = 0;
  eRange      = 0;
  eConstant   = 0;
  eConstraint = 0;
  nNeq        = 0;
  goald = NULL;
}

CSP::~CSP() 
{
  ENVIRONMENT.flush();
  
  cleanUp();
  
  delete goald;
  
  declarations.clear();
  goald = NULL;
}

void CSP::cleanUp()
{
  nValues     = 0;
  nList       = 0;
  nBit        = 0;
  nBoolean    = 0;
  nRange      = 0;
  nConstant   = 0;
  nConstraint = 0;
  eValues     = 0;
  eList       = 0;
  eBit        = 0;
  eBoolean    = 0;
  eRange      = 0;
  eConstant   = 0;
  eConstraint = 0;
  int i = declarations.size();
  while( i-- ) {
    delete declarations[i];
  }
}


BuildObject* CSP::_Variable( const int nvals ) 
{
  BuildObject *x = new BuildObject( 0, nvals-1 );
  x->setVar();
  return x;
}

BuildObject* CSP::_Variable( const int lb, const int ub ) 
{
  BuildObject *x = new BuildObject( lb, ub );
  x->setVar();
  return x;
}

BuildObject* CSP::_Variable( const int* array, const int length ) 
{
  BuildObject *x = new BuildObject(array, length);
  x->setVar();
  return x;
}

BuildObject* CSP::_Variable( const int* array, const int length, 
			    const int lb, const int ub ) 
{
  BuildObject *x = new BuildObject(array, length, lb, ub);
  x->setVar();
  return x;
}

BuildObject** CSP::_VarArray( const int nvars, const int nvals )
{
  BuildObject **array_ = new BuildObject*[nvars];
  int i=nvars;
  while( i-- )
    array_[i] = CSP::_Variable( nvals );
  return array_;
}

BuildObject** CSP::_VarArray( const int nvars, const int lb, const int ub ) 
{
  BuildObject **array_ = new BuildObject*[nvars];
  int i=nvars;
  while( i-- ) 
    array_[i] = CSP::_Variable( lb, ub );
  return array_;
}

BuildObject** CSP::_VarArray( const int nvars, const int* array, const int length ) 
{
  BuildObject **array_ = new BuildObject*[nvars];
  int i=nvars;
  while( i-- ) 
    array_[i] = CSP::_Variable( array, length );
  return array_;
}

BuildObject** CSP::_VarArray( const int nvars, const int* array, 
			     const int length, const int lb, const int ub )
{
  BuildObject **array_ = new BuildObject*[nvars];
  int i=nvars;
  while( i-- ) 
    array_[i] = CSP::_Variable( array, length, lb, ub );
  return array_;
}

BuildObject* CSP::_Offset( BuildObject *x_, int o )
{
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  int *params = new int[1];
  *params = o;
  return new BuildObjectPredicate( x, 1, x_->min()+o, x_->max()+o, 
				   ENVIRONMENT[ConstraintStore::OFFSET], params, 
				   (BuildObject::AWV | 
				    BuildObject::CRANGE) );
}

BuildObject* CSP::_Factor( BuildObject *x_, int o )
{
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  int *params = new int[1];
  *params = o;
  return new BuildObjectPredicate( x, 1,
				   (o < 0 ? x_->max()*o : x_->min()*o), 
				   (o < 0 ? x_->min()*o : x_->max()*o), 
				   ENVIRONMENT[ConstraintStore::FACTOR], 
				   params, (BuildObject::AWV | 
					    BuildObject::CRANGE) );
}
BuildObject* CSP::_Negation( BuildObject *x_ )
{
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  return new BuildObjectPredicate( x, 1, 
				   -x_->max(), 
				   -x_->min(), 
				   ENVIRONMENT[ConstraintStore::NEG],
				   NULL, (BuildObject::AWV | 
					  BuildObject::CRANGE) ) ;
}
BuildObject* CSP::_Not( BuildObject *x_ )
{
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  return new BuildObjectPredicate( x, 1, 0, 1, 
				   ENVIRONMENT[ConstraintStore::NOT],
				   NULL, (BuildObject::CRANGE | 
					  BuildObject::CONVEX) ) ;
}
BuildObject* CSP::_Abs( BuildObject *x_ )
{
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  int lb, ub, aux;     
  if( x_->min() >= 0 ) {
    lb = x_->min();
    ub = x_->max();
  } else if( x_->max() <= 0 ) {
    ub = -1*x_->min();
    lb = -1*x_->max();
  } else {
    aux = -1*(x_->minNegAbs());
    lb = x_->minPosAbs();
    if( aux < lb ) lb = aux;
    
    aux = -1*(x_->min());
    ub = x_->max();
    if( aux > ub ) ub = aux;
  }
  return new BuildObjectPredicate( x, 1, lb, ub, 
				   ENVIRONMENT[ConstraintStore::ABS], 
				   NULL, BuildObject::AWV );
}
BuildObject* CSP::_And( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, 0, 1, 
				  ENVIRONMENT[ConstraintStore::AND],
				  NULL, (BuildObject::CRANGE | 
					 BuildObject::CONVEX)) ;
}
BuildObject* CSP::_Or( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, 0, 1, 
				  ENVIRONMENT[ConstraintStore::OR],
				  NULL, (BuildObject::CRANGE | 
					 BuildObject::CONVEX) ) ;
}
BuildObject* CSP::_IfThenElse( BuildObject *x_, 
			      BuildObject *y_, 
			      BuildObject *z_ )
{
  BuildObject **x = new BuildObject*[4];
  x[0] = x_;
  x[1] = y_;
  x[2] = z_;
  return new BuildObjectPredicate(x, 3, std::min(y_->min(), z_->min()), 
				  std::max(y_->max(), z_->max()), 
				  ENVIRONMENT[ConstraintStore::IFTHENELSE],
				  NULL, (BuildObject::CRANGE | 
					 BuildObject::AWV)) ;
}
BuildObject* CSP::_Disjunctive( BuildObject *x_, int dx, 
			       BuildObject *y_, int dy )
{
  int *params = new int[2];
  params[0] = dx;
  params[1] = dy;
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, 0, 1, 
				  ENVIRONMENT[ConstraintStore::DISJUNCTIVE], 
				  params, (BuildObject::AWV | 
					   BuildObject::CRANGE) );
}
BuildObject* CSP::_Precedence( BuildObject *x_, int d, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  int *params = new int[1];
  params[0] = d;
  return new BuildObjectPredicate(x, 2, 0, 1, 
				  ENVIRONMENT[ConstraintStore::PRECEDENCE], 
				  params, (BuildObject::CRANGE | 
					   BuildObject::CONVEX));
}
BuildObject* CSP::_Precedence( BuildObject *x_, int b )
{
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  int *params = new int[1];
  *params = b;
  return new BuildObjectPredicate( x, 1, 0, 1, 
				   ENVIRONMENT[ConstraintStore::UPPERBOUND], 
				   params, (BuildObject::CRANGE | 
					    BuildObject::CONVEX) );
}
BuildObject* CSP::_Precedence( int b, BuildObject *x_ )
{
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  int *params = new int[1];
  *params = b;
  return new BuildObjectPredicate( x, 1, 0, 1, 
				   ENVIRONMENT[ConstraintStore::LOWERBOUND], 
				   params, (BuildObject::CRANGE | 
					    BuildObject::CONVEX) );
}
BuildObject* CSP::_Add( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, (x_->min()+y_->min()), 
				  (x_->max()+y_->max()), 
				  ENVIRONMENT[ConstraintStore::ADD], 
				  NULL, (BuildObject::AWV | 
					 BuildObject::CRANGE));
}
BuildObject* CSP::_Sub( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, (x_->min()-y_->max()), 
				  (x_->max()-y_->min()),
				  ENVIRONMENT[ConstraintStore::SUB],  
				  NULL, (BuildObject::AWV | 
					 BuildObject::CRANGE));
}
BuildObject* CSP::_Mul( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  int z[4], lb = INT_MAX, ub = -1*INT_MAX, i = 4;
  z[0] = x_->min() * y_->min();
  z[1] = x_->min() * y_->max();
  z[2] = x_->max() * y_->min();
  z[3] = x_->max() * y_->max();  
  while( i-- ) {
    if( z[i] > ub ) ub = z[i];
    if( z[i] < lb ) lb = z[i];
  }
  return new BuildObjectPredicate(x, 2, lb, ub, 
				  ENVIRONMENT[ConstraintStore::MUL], 
				  NULL, (BuildObject::AWV | 
					 BuildObject::CRANGE));
}
BuildObject* CSP::_Div( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  int lb, ub, aux;
      
  int mp1 = y_->minPosAbs();
  int mn1 = y_->minNegAbs();
  int mp0 = x_->minPosAbs();
  int mn0 = x_->minNegAbs();
      
  int m1 = y_->min();
  int x1 = y_->max();
  int m0 = x_->min();
  int x0 = x_->max();
      
  if( !mp1 ) {
    if( y_->max() )
      mp1 = 1;
    else mp1 = -1;
  }
  if( !mn1 ) {
    if( y_->min() )
      mn1 = -1;
    else mn1 = 1;
  }
  if( !mp0 ) {
    if( x_->max() )
      mp0 = 1;
    else mp0 = -1;
  }
  if( !mn0 ) {
    if( x_->min() )
      mn0 = -1;
    else mn0 = 1;
  }
      
  lb = x0/mn1;
  aux = m0/mp1;
  if( lb > aux ) lb = aux;
      
  ub = x0/mp1;
  aux = m0/mn1;
  if( ub < aux ) ub = aux;
      
  return new BuildObjectPredicate(x, 2, lb, ub,
				  ENVIRONMENT[ConstraintStore::MUL], 
				  NULL, (BuildObject::AWV | 
					 BuildObject::CRANGE));
}
BuildObject* CSP::_Pow( BuildObject *x_, BuildObject *y_ )
{
  return NULL;
}
BuildObject* CSP::_Mod( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  int lb = ( x_->min() < 0 ? -y_->max()+1 : 0 );
  int ub = ( x_->max() > 0 ? y_->max()-1 : 0 );
  return new BuildObjectPredicate(x, 2, lb, ub,
				  ENVIRONMENT[ConstraintStore::MOD],
				  NULL, BuildObject::AWV) ;
}
BuildObject* CSP::_Mod( BuildObject *x_, const int k )
{
  int *param = new int[1];
  param[0] = k;
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  int lb = ( x_->min() < 0 ? 1-k : 0 );
  int ub = ( x_->max() > 0 ? k-1 : 0 );
  return new BuildObjectPredicate(x, 1, lb, ub,
				  ENVIRONMENT[ConstraintStore::MODCONSTANT],
				  param, BuildObject::AWV) ;
}
BuildObject* CSP::_Equal( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, 0, 1, 
				  ENVIRONMENT[ConstraintStore::EQUAL],
				  NULL, BuildObject::CRANGE) ;
}
BuildObject* CSP::_Equal( BuildObject *x_, int k )
{
  int *param = new int[1];
  param[0] = k;
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  return new BuildObjectPredicate(x, 1, 0, 1, 
				  ENVIRONMENT[ConstraintStore::EQCONSTANT],
				  param, (BuildObject::CRANGE | 
					  BuildObject::CONVEX)) ;
}
BuildObject* CSP::_NotEqual( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, 0, 1, 
				  ENVIRONMENT[ConstraintStore::NEQ],
				  NULL, BuildObject::CLIST) ;
}
BuildObject* CSP::_NotEqual( BuildObject *x_, int k )
{
  int *param = new int[1];
  param[0] = k;
  BuildObject **x = new BuildObject*[2];
  x[0] = x_;
  return new BuildObjectPredicate(x, 1, 0, 1, 
				  ENVIRONMENT[ConstraintStore::NEQCONSTANT],
				  param, BuildObject::CLIST) ;
}
BuildObject* CSP::_NotEqualWildcard( BuildObject *x_, 
				    BuildObject *y_, const int w )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  int *params = new int[1];
  *params = w;
  return new BuildObjectPredicate(x, 2, 0, 1, 
				  ENVIRONMENT[ConstraintStore::NEQW],
				  params) ;
}
BuildObject* CSP::_Sum( BuildObject **x_, const int nx, const int *os )
{
  BuildObject **x = new BuildObject*[nx+1];
  int lb = 0, ub = 0, realsize=0, *params = NULL;
  bool simple = true;
  if( os ) {
    //bool simple = false;
    for(int i=0; simple && i<nx; ++i) 
      simple = ( os[i] == 1 );
    if( !simple ) {
      params = new int[nx+1];
      //params[n] = 0;
    }
  }
  for(int i=0; i<nx; ++i) {
    if( params ) {
      lb+=( os[i] * ( os[i] < 0 ? x_[i]->max() : x_[i]->min() ) );
      ub+=( os[i] * ( os[i] < 0 ? x_[i]->min() : x_[i]->max() ) );
      if( os[i] ) {
	params[realsize] = os[i];
	x[realsize] = x_[i];
	++realsize;
      }
    } else {
      x[realsize] = x_[i];
      lb+=x[realsize]->min();
      ub+=x[realsize]->max();
      ++realsize;
    }
  }

  if( !simple ) {
    params[realsize] = 0;
  }

  return new BuildObjectPredicate( x, realsize, lb, ub, 
				   ENVIRONMENT[ConstraintStore::SUM], 
				   params, (BuildObject::AWV | 
					    BuildObject::CRANGE | 
					    BuildObject::CONVEX) );
}
BuildObject* CSP::_Min( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, (std::min(x_->min(),y_->min())), 
				  (std::max(x_->max(),y_->max())), 
				  ENVIRONMENT[ConstraintStore::MIN], 
				  NULL, (BuildObject::AWV | 
					 BuildObject::CRANGE | 
					 BuildObject::CONVEX));
}
BuildObject* CSP::_Max( BuildObject *x_, BuildObject *y_ )
{
  BuildObject **x = new BuildObject*[3];
  x[0] = x_;
  x[1] = y_;
  return new BuildObjectPredicate(x, 2, (std::min(x_->min(),y_->min())), 
				  (std::max(x_->max(),y_->max())), 
				  ENVIRONMENT[ConstraintStore::MAX], 
				  NULL, (BuildObject::AWV | 
					 BuildObject::CRANGE | 
					 BuildObject::CONVEX));
}
BuildObject* CSP::_Min( BuildObject **x_, const int nx )
{
  BuildObject **x = new BuildObject*[nx+1];
  int lb = INT_MAX, ub = -INT_MAX, i=nx;
  while( i-- ) {
    x[i] = x_[i];
    if(lb > x_[i]->min()) lb = x_[i]->min();
    if(ub < x_[i]->max()) ub = x_[i]->max();
  }
  return new BuildObjectPredicate( x, nx, lb, ub, 
				   ENVIRONMENT[ConstraintStore::MIN], 
				   NULL, (BuildObject::AWV | 
					  BuildObject::CRANGE | 
					  BuildObject::CONVEX) );
}
BuildObject* CSP::_Max( BuildObject **x_, const int nx )
{
  BuildObject **x = new BuildObject*[nx+1];
  int lb = INT_MAX, ub = -INT_MAX, i=nx;
  while( i-- ) {
    x[i] = x_[i];
    if(lb > x_[i]->min()) lb = x_[i]->min();
    if(ub < x_[i]->max()) ub = x_[i]->max();
  }
  return new BuildObjectPredicate( x, nx, lb, ub, 
				   ENVIRONMENT[ConstraintStore::MAX], 
				   NULL, (BuildObject::AWV | 
					  BuildObject::CRANGE | 
					  BuildObject::CONVEX) );
}
BuildObject* CSP::_Element( BuildObject **x_, const int nx, BuildObject *y_ )
{

  //  cout << "nx " << nx << endl;


  //int n = x.size() ;
  BuildObject **scope = new BuildObject*[nx+2];
  int lb = INT_MAX, ub = -INT_MAX;
  for(int i=0; i<nx; ++i) {

//     cout << "x_[" << i << "]: ";
//     cout.flush();
//     x_[i]->print( cout );
//     cout << endl;

    scope[i] = x_[i];
    if(lb > scope[i]->min()) lb = scope[i]->min() ;
    if(ub < scope[i]->max()) ub = scope[i]->max() ;
  }
  scope[nx] = y_;
  //delete [] x_;
  for(int i=0; i<=nx; ++i) 
    scope[i]->unsetPar() ;
  return new BuildObjectPredicate( scope, nx+1, lb, ub, 
				   ENVIRONMENT[ConstraintStore::ELEMENT],
				   NULL, BuildObject::AWV ) ;
}
BuildObject* CSP::_Element( BuildObject *y_, BuildObject **x_, 
			   const int nx, BuildObject *z_ )
{
  //int n = x.size() ;
  BuildObject **scope = new BuildObject*[nx+2];
  int lb = z_->min(), ub = z_->max() ;
  for(int i=0; i<nx; ++i) 
    scope[i] = x_[i];
  scope[nx] = y_;
  BuildObject* pred = new BuildObjectPredicate( scope, nx+1, lb, ub, 
						ENVIRONMENT[ConstraintStore::ELEMENT],
						NULL, BuildObject::AWV ) ;
  //  delete [] x_;
  for(int i=0; i<nx+2; ++i) 
    scope[i]->unsetPar() ;
  z_->setEqual( pred ) ;
  return pred;
}
BuildObject* CSP::_Element( BuildObject **x_, const int nx )
{
  //  int n = x.size() ;
  BuildObject **scope = new BuildObject*[nx];
  int lb = x_[nx-1]->min(), ub = x_[nx-1]->max() ;
  for(int i=0; i<nx-2; ++i) 
    scope[i] = x_[i+1];
  scope[nx-2] = x_[0];
  BuildObject* pred = new BuildObjectPredicate( scope, nx-1, lb, ub, 
						ENVIRONMENT[ConstraintStore::ELEMENT],
						NULL, BuildObject::AWV ) ;
  //  delete [] x_;
  for(int i=0; i<nx; ++i) 
    scope[i]->unsetPar() ;
  x_[nx-1]->setEqual( pred ) ;
  return pred;
}
BuildObject* CSP::_Clause( BuildObject **x_, const int nx, const int* c )
{
  BuildObject **x = new BuildObject*[nx+1];
  int cidx = ENVIRONMENT.addConstraint( new BuildObjectClause() );
  int *params = new int[nx];
  for(int i=0; i<nx; ++i) {
    x[i] = x_[i];
    params[i] = c[i];
  }
  return new BuildObjectPredicate( x, nx, 0, -1, 
				   ENVIRONMENT[cidx],
				   params, BuildObject::CLIST ) ;
}
BuildObject* CSP::_Table( BuildObject **x_, const int nx, const bool sp )
{
  BuildObject **x = new BuildObject*[nx+1];
  int cidx = ENVIRONMENT.addConstraint( new BuildObjectTable(nx) ) ;
 
  BuildObjectTable* tab = (BuildObjectTable*)(ENVIRONMENT[cidx]) ;
  tab->spin = sp;

  for(int i=0; i<nx; ++i) {
    x[i] = x_[i];
    x_[i]->unsetPar() ;  
    tab->setMinDom(i, x_[i]->min()) ;
    tab->setMaxDom(i, x_[i]->max()+1) ;
  }
  BuildObject *pred = new BuildObjectPredicate( x, nx, 0, -1, 
						tab, NULL, 
						BuildObject::CLIST ) ;
  return pred;
}
BuildObject* CSP::_Table( BuildObject **x_, const int nx, 
			 const int tidx, const bool sp )
{
  BuildObject **x = new BuildObject*[nx+1];
  BuildObjectTable* tab = (BuildObjectTable*)(ENVIRONMENT[tidx]) ;
  tab->spin = sp;

  for(int i=0; i<nx; ++i) {
    x[i] = x_[i];
    x_[i]->unsetPar() ;  
    tab->setMinDom(i, x_[i]->min()) ;
    tab->setMaxDom(i, x_[i]->max()+1) ;
  }
  return new BuildObjectPredicate( x, nx, 0, -1, 
				   tab, NULL, 
				   BuildObject::CLIST ) ;
}
BuildObject* CSP::_AllDifferent( BuildObject **x_, const int nx )
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  BuildObject *pred = new BuildObjectPredicate( x, nx, 0, -1, 
						ENVIRONMENT[ConstraintStore::ALLDIFF],
						NULL ) ;
  for(int i=0; i<nx; ++i)
    x_[i]->unsetPar() ;
  pred->unsetPar() ;

  return pred;
}
BuildObject* CSP::_BoundAllDifferent( BuildObject **x_, const int nx )
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int *par = new int[1];
  BuildObject *pred = new BuildObjectPredicate( x, nx, 0, -1, 
						ENVIRONMENT[ConstraintStore::ALLDIFF],
						par, (BuildObject::CRANGE | 
						      BuildObject::CONVEX) ) ;
  pred->unsetPar() ;   

  return pred;
}
BuildObject* CSP::_Gcc( BuildObject **x_, const int nx,
		       const int firstDomainValue, 
		       const int lastDomainValue,
		       const int* minOccurrences,
		       const int* maxOccurrences )
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int n = (lastDomainValue - firstDomainValue + 1) ;
  int *params = new int[((n+1) << 1)];
  params[0] = firstDomainValue;
  params[1] = lastDomainValue;
  std::memcpy(params+2, minOccurrences, n*sizeof(int)) ;
  std::memcpy(params+2+n, maxOccurrences, n*sizeof(int)) ;
  
  BuildObject *pred = new BuildObjectPredicate( x, nx, 0, -1, 
						ENVIRONMENT[ConstraintStore::GCC],
						params ) ;
  pred->unsetPar() ;

  return pred;
}
BuildObject* CSP::_EdgeFinder( BuildObject **x_, const int nx, const int *d )
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int *params = new int[nx];
  for(int i=0; i<nx; ++i) 
    params[i] = d[i];
  BuildObject *pred = new BuildObjectPredicate( x, nx, 0, -1, 
						ENVIRONMENT[ConstraintStore::EDGEFINDER],
						params, BuildObject::CRANGE ) ;
  pred->unsetPar() ;
  return pred;
}
BuildObject* CSP::_LexOrder( BuildObject **x_, 
			     BuildObject **y_, const int nx, int eq )
{
  int *params = new int[1];
  *params = eq;
  BuildObject **scope = new BuildObject*[(nx << 1)+1];
  for(int i=0; i<nx; ++i) {
    scope[i] = x_[i];
    scope[i+nx] = y_[i];
  }
  return new BuildObjectPredicate( scope, (nx << 1), 0, -1, 
				   ENVIRONMENT[ConstraintStore::LEX],
				   params, (BuildObject::CRANGE | 
					    BuildObject::CONVEX) ) ;
}
BuildObject* CSP::_MaxOccurrence( BuildObject **x_, const int nx, 
				 const int k, const int ub  )
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int *params = new int[2];
  params[0] = k;
  params[1] = ub;
  return new BuildObjectPredicate( x, nx, 0, -1, 
				   ENVIRONMENT[ConstraintStore::MAXOCCUR],
				   params ) ;
}
BuildObject* CSP::_MinOccurrence( BuildObject **x_, const int nx, 
				 const int k, const int lb  )
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int *params = new int[2];
  params[0] = k;
  params[1] = lb;
  return new BuildObjectPredicate( x, nx, 0, -1, 
				   ENVIRONMENT[ConstraintStore::MINOCCUR],
				   params ) ;
}
BuildObject* CSP::_Slide( BuildObject **x_,
			 const int n, const int m, CSP *p )
{
  BuildObject **x = new BuildObject*[n+1];
  std::memcpy( x, x_, n*(sizeof(BuildObject *)) );
  int *params = new int[2];
  params[0] = n;
  params[1] = m;
  BuildObjectSlide *rel = (BuildObjectSlide*)(ENVIRONMENT[ConstraintStore::SLIDE]) ;
  rel->sub_problem = p;
  return new BuildObjectPredicate( x, n, 0, -1, rel, params ) ; 
}

#ifdef _CPLEX
BuildObject* CSP::_CPlexMinDifference( BuildObject **x_, const int nx, const int ft ) 
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int *params = new int[1];
  params[0] = ft;
  //int n = x_.size() ;
  var_ptr_ = new BuildObjectPredicate( x, nx, 0, (nx*(nx-1)/2), 
				       new BuildObjectCPlexMinDifference(), 
				       params, (BuildObject::AWV | BuildObject::CRANGE) ) ;
}
#endif

#ifdef _COINLP
BuildObject* CSP::_MaxDifference( BuildObject **x_, const int mt, const int ft ) 
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int *params = new int[2];
  params[0] = ft;
  params[1] = mt;
  return new BuildObjectPredicate( x, nx, 0, (nx*(nx-1)/2), 
				   ENVIRONMENT[ConstraintStore::MAXDIFF],
				   params, (BuildObject::AWV | 
					    BuildObject::CRANGE) ) ;
}
BuildObject* CSP::_MinDifference( BuildObject **x_, const int mt, const int ft ) 
{
  BuildObject **x = new BuildObject*[nx+1];
  std::memcpy( x, x_, nx*(sizeof(BuildObject *)) );
  int *params = new int[2];
  params[0] = ft;
  params[1] = mt;
  return new BuildObjectPredicate( x, nx, 0, (nx*(nx-1)/2), 
				   ENVIRONMENT[ConstraintStore::MINDIFF],
				   params, (BuildObject::AWV | 
					    BuildObject::CRANGE) ) ;
}

#endif


BuildObjectObjective* CSP::_Maximise( BuildObject *x )
{
  return new BuildObjectMaximiseVar( x ) ; 
}

BuildObjectObjective* CSP::_Minimise( BuildObject *x )
{
  return new BuildObjectMinimiseVar( x ) ; 
}


void CSP::print(std::ostream& o) const
{

  int i, n=declarations.size();
  for(i=0; i<n; ++i) {

    cout << "- ";
    cout << declarations[i]->isReferenced() << " ";
    cout << declarations[i]->id << " ";
    cout << declarations[i]->getVarId() << " ";
    declarations[i]->print( cout );
    cout << endl;

  }
  
}

int CSP::size() 
{
  return declarations.size() ;
}

void CSP::printXML(std::ostream& o) const
{
  int nVariables = 0;
  int nConstraints = 0;
  int skippedConstraints = 0;
  int skippedVariables = 0;
  int maxarity = 0;

  int i, j=declarations.size(), k, l;

  int var2dec[j], con2dec[j], dec2var[j], subconstraint[j];
  for( i=0; i<j; ++i )
    { 
      subconstraint[i] = 0;
      if( declarations[i]->isRel() ) {
	if( declarations[i]->isConstraint() ) {
	  con2dec[nConstraints++] = i;	
	  if( declarations[i]->isVar() ) {
	    dec2var[declarations[i]->getVarId()] = nVariables;
	    var2dec[nVariables++] = i;
	  }
	}
	k = ((BuildObjectPredicate*)(declarations[i]))->arity;
	if( k > maxarity )
	  maxarity = k;
      } else if( declarations[i]->isVar() && (!declarations[i]->isPar() || declarations[i]->size() > 1) ) {
	dec2var[declarations[i]->getVarId()] = nVariables;
	var2dec[nVariables++] = i;
      }
    }

  int var2dom[nVariables];
  int con2pre[nConstraints];
  int con_type[nConstraints];
  map<string, int> domains_map;
  map<string, int> predicates_map;
  vector<string> domains;
  vector<string> predicates;
  vector<int> domainsize;
  vector<int> predicatesize;

  vector<string> global;
  vector<int> table;

  int nDomains=0;
  int nPredicates=0;
  int nparams=0;

  i=nConstraints;
  while( i-- ) {
    if( !subconstraint[con2dec[i]] ) {

      nparams=0;
      string pred = declarations[con2dec[i]]->xmlPred( nparams, 0, subconstraint );           
      subconstraint[con2dec[i]] = 0;
      
      if( pred.substr(0,6) == "global" ) {
	//cout << "GLOBAL: " << pred << endl;
	global.push_back( pred );
	con_type[i] = 2;
      } else if( pred == "table" ) {
	//cout << "TABLE: " << pred << endl;
	table.push_back( i );
	con_type[i] = 1;
      } else {
	//cout << "PREDICATE: " << pred << endl;
	if( declarations[con2dec[i]]->isVar() )
	  pred = "eq(" + pred + ",X" + int2string(nparams++) + ")";
	con_type[i] = 0;
	if( predicates_map.find(pred) != predicates_map.end() ) {
	  con2pre[i] = predicates_map[pred];
	} else {
	  predicates_map[pred] = nPredicates;
	  predicates.push_back(pred);
	  predicatesize.push_back(nparams);
	  con2pre[i] = nPredicates++;
	}
      }
    } else {
      ++skippedConstraints;
    }
  }



  for(i=0; i<nVariables; ++i) {
    string dom = declarations[var2dec[i]]->xmlDom();
    if( domains_map.find( dom ) != domains_map.end() ) {
      var2dom[i] = domains_map[dom];      
    } else {
      domains_map[dom] = nDomains;
      domains.push_back( dom );
      domainsize.push_back( declarations[var2dec[i]]->size() );

      var2dom[i] = nDomains++;
    }
  }


  // header	  
  o << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" 
    << endl << endl << "<instance>" << endl 
    << "<presentation maxConstraintArity=\""
    << maxarity <<"\" format=\"XCSP 2.1\"/>" 
    << endl << endl;
	
  // domains
  o << "  <domains nbDomains=\"" << nDomains << "\">" << endl;
  for(i=0; i<nDomains; ++i)
    {
      o << "    <domain name=\"D" << i 
	<< "\" nbValues=\"" << domainsize[i] << "\">" 
	<< domains[i]
	<< "</domain>" << endl; 
    }
  o << "  </domains>" << endl << endl;

  // variables
  i=nVariables;
  while( i-- )
    if( subconstraint[var2dec[i]] ) {
      var2dec[i] = var2dec[--nVariables];
      var2dom[i] = var2dom[nVariables];
      dec2var[var2dec[i]] = i;
      //++skippedVariables;
    }
  k=0;
  o << "  <variables nbVariables=\"" << (nVariables-skippedVariables) << "\">" << endl;
  for(i=0; i<nVariables; ++i)
    {
      //if( !subconstraint[var2dec[i]] )
      o << "    <variable name=\"V" << i
	<< "\" domain=\"D" << var2dom[i] << "\"/>" << endl;
    }
  o << "  </variables>" << endl << endl;
  
  // predicates
  if( nPredicates ) {
    o << "  <predicates nbPredicates=\"" 
      << (nPredicates) << "\">" << endl;
    for(i=0; i<nPredicates; ++i)
      {
	o << "    <predicate name=\"P"<< i << "\">" << endl
	  << "      <parameters>";
	for(j=0; j<predicatesize[i]; ++j)
	  o << " int X" << j;
	o << "</parameters>" << endl
	  << "      <expression> " << endl
	  << "        <functional>" 
	  << predicates[i] 
	  << "</functional>" << endl
	  << "      </expression>" << endl
	  << "    </predicate>" << endl;
      }
    o << "  </predicates>" << endl << endl;
  }

  // constraints
  int arity;
  BuildObjectPredicate *bcon;
  MistralSet scope(0, nVariables-1, MistralSet::empt);
  o << "  <constraints nbConstraints=\"" 
    << nConstraints-skippedConstraints << "\">" << endl;
  k=0;
  l=0;

  i=nConstraints;
  while( i-- ) 
    if( !subconstraint[con2dec[i]] )
      {
	bcon = ((BuildObjectPredicate*)(declarations[con2dec[i]]));
	
	/// PREDICATES
	if( con_type[i] == 0 ) {
	  scope.clear();
	  
	  string params = declarations[con2dec[i]]->xmlCon( scope, dec2var ); 
	  if( declarations[con2dec[i]]->isVar() )
	    {
	      if( declarations[con2dec[i]]->size() > 1 ) 
		{
		  j = dec2var[declarations[con2dec[i]]->getVarId()];
		  scope.insert(j);
		  params += (" V" + int2string(j));
		}
	      else
		{
		  params += (" " + int2string(declarations[con2dec[i]]->min()));
		}
	    }
	  arity = scope.size();
	  if( arity ) {
	    o << "    <constraint name=\"C" << l++ <<"\" arity=\""
	      << arity << "\" scope=\"";
	    j=scope.min();
	    while(true) {
	      o << "V" << j;//dec2var[j] ;
	      j = scope.next( j );
	      if( j != INT_MAX )
		o << " ";
	      else break;
	    } 
	  o << "\" reference=\"P" << con2pre[i] << "\">" << endl
	    << "      <parameters>" 
	    << params
	    << "</parameters> " << endl
	    << "    </constraint>" << endl; 
	  }
	}
	
	/// RELATIONS
	else if( con_type[i] == 1 ) {
	  ;
	}
      
	/// GLOBAL CONSTRAINTS
	else if( con_type[i] == 2 ) {

// 	  cout << i << " -- " << bcon->arity << " ";
// 	  bcon->print( cout );
// 	  cout << " " << global[k] << endl;

	  scope.clear();
	  arity = bcon->arity;
	  for(j=0; j<arity; ++j) {
	    scope.insert(dec2var[bcon->scope[j]->getVarId()]);
	  }
	  if( bcon->isVar() ) scope.insert( dec2var[bcon->scope[arity]->getVarId()] );
	  arity = scope.size();
	  o << "    <constraint name=\"C" << l++ <<"\" arity=\""
	    << arity << "\" scope=\"";
	  j = scope.min();
	  do { 
	    o << "V" << j << " ";
	    j = scope.next( j );
	  } while(j != INT_MAX);

	  o << "\" reference=\"" << global[k] ;
	  
	  if( global[k] == "global:element" )
	    {

	      o << "\">" << endl
		<< "      <parameters>" << endl
		<< "        V" << dec2var[bcon->scope[bcon->arity-1]->getVarId()] 
		<< endl << "        [ ";
	      for(j=0; j<bcon->arity-1; ++j)
		o << "V" << dec2var[bcon->scope[j]->getVarId()] << " ";
	      o << "]\n        V" 
		<< dec2var[bcon->scope[bcon->arity]->getVarId()] 
		<< endl << "      </parameters>"
		<< endl << "    </constraint>" << endl;
	    }
	  else if( global[k] == "global:cumulative" )
	    {
	      o << "\"/>" << endl;
	    }	  
	  else if( global[k] == "global:weightedSum" )
	    {
	      o << "\"/>" << endl;
	    }
	  else if( global[k] == "global:allDifferent" )
	    {
	      o << "\"/>" << endl;
	    }
	  ++k;
	}
      }
  o << "  </constraints>" 
    << endl << endl
    << "</instance>" << endl;

}



int ConstraintStore::addConstraint(BuildObjectConstraint *c)
{
  store.push( c );
  return (store.size()-1);
}

void ConstraintStore::remConstraint(int idx)
{
  delete store[idx];
  store[idx] = NULL;
  while( !store.back() )
    --store.size_;
}

void ConstraintStore::flush()
{
  int i = store.size();
  while( i-- > NUMCONS ) {    
    if( store[i] ) 
      delete store[i]; 
    store.pop();
  }
}

BuildObjectConstraint* ConstraintStore::getConstraint(const int c)
{
  if( !store[c] ) {
    switch( c ) {
    case ABS         : store[c] = new BuildObjectAbs();              break;
    case ALLDIFF     : store[c] = new BuildObjectAllDiff();          break;
    case AND         : store[c] = new BuildObjectAnd();              break;
    case CLAUSE      : store[c] = new BuildObjectClause();           break;
    case DISJUNCTIVE : store[c] = new BuildObjectDisjunctive();      break;
    case DIV         : store[c] = new BuildObjectDiv();              break;
    case EDGEFINDER  : store[c] = new BuildObjectEdgeFinder();       break;
    case ELEMENT     : store[c] = new BuildObjectElement();          break;
    case EQUAL       : store[c] = new BuildObjectEqual();            break;
    case NEQ         : store[c] = new BuildObjectNotEqual();         break;
    case NEQCONSTANT : store[c] = new BuildObjectNotEqualConstant(); break;
    case EQCONSTANT  : store[c] = new BuildObjectEqualConstant();    break;
    case FACTOR      : store[c] = new BuildObjectFactor();           break;
    case GCC         : store[c] = new BuildObjectGcc();              break;
    case LEX         : store[c] = new BuildObjectLex();              break;
    case MAX         : store[c] = new BuildObjectMax();              break;
    case MIN         : store[c] = new BuildObjectMin();              break;
    case MAXOCCUR    : store[c] = new BuildObjectMaxOccurrence();    break;
    case MINOCCUR    : store[c] = new BuildObjectMinOccurrence();    break;
    case MOD         : store[c] = new BuildObjectMod();              break;
    case MODCONSTANT : store[c] = new BuildObjectModConstant();      break;
    case MUL         : store[c] = new BuildObjectMul();              break;
    case ADD         : store[c] = new BuildObjectAdd();              break;
    case SUB         : store[c] = new BuildObjectSub();              break;
    case NEG         : store[c] = new BuildObjectNegation();         break;
    case NOT         : store[c] = new BuildObjectNot();              break;
    case OFFSET      : store[c] = new BuildObjectOffset();           break;
    case OR          : store[c] = new BuildObjectOr();               break;
    case POW         : store[c] = new BuildObjectPow();              break;
    case PRECEDENCE  : store[c] = new BuildObjectPrecedence();       break;
    case SUM         : store[c] = new BuildObjectSum();              break;
    case SUMEQ       : store[c] = new BuildObjectSumEq();            break;
    case NEQW        : store[c] = new BuildObjectNotEqualWildcard(); break;
    case SLIDE       : store[c] = new BuildObjectSlide();            break;
    case LOWERBOUND  : store[c] = new BuildObjectLowerBound();       break;
    case UPPERBOUND  : store[c] = new BuildObjectUpperBound();       break;
    case IFTHENELSE  : store[c] = new BuildObjectIfThenElse();       break;
      //case REGULAR     : store[c] = new BuildObjectRegular();          break;

#ifdef _COINLP
    case MAXDIFF     : store[c] = new BuildObjectMaxDifference();    break;
    case MINDIFF     : store[c] = new BuildObjectMinDifference();    break;
#endif

    }
  }
  return store[c];
}

ConstraintStore::ConstraintStore()
{
  store.init(NUMCONS, 1024);
  for(int i=0; i<NUMCONS; ++i)
    store[i] = NULL;
}

ConstraintStore::~ConstraintStore()
{
  int i = store.size();
  while( i-- )
    if( store[i] )
      delete store[i];
}

BuildObjectConstraint *ConstraintStore::operator[]( const int c ) 
{
  return getConstraint( c );
}

Objective::Objective(BuildObject *x) : X(x) 
{ 
  obj = NULL; 
}

Maximise::Maximise(Variable x) : Objective( x.var_ptr_ ) 
{
  obj = new BuildObjectMaximiseVar( X ) ; 
}

Minimise::Minimise(Variable x) : Objective( x.var_ptr_ ) 
{
  obj = new BuildObjectMinimiseVar( X ) ;
}

